@@ -26,6 +26,7 @@
 // Application
 /////////////////////////////////////////////////////////////////////////////////////////
 namespace Clasp {
+#define CLASP_APP_EXIT(x) (prepareExit(), _exit(x))
 #if !defined(CLASP_USAGE)
 #define CLASP_USAGE   "clasp [number] [options] [file]"
 #endif
@@ -36,9 +37,9 @@ namespace Clasp {
 #define SIGUSR2 SIGTERM
 #endif
 #if !defined(COMPETITION_MODE) || COMPETITION_MODE == 0
-#define CLASP_APP_RETURN(x) return x
+#define CLASP_APP_RETURN(x) prepareExit(); return x
 #else
-#define CLASP_APP_RETURN(x) _exit(x)
+#define CLASP_APP_RETURN(x) CLASP_APP_EXIT(x)
 #endif
 namespace {
 std::ostream& operator << (std::ostream& os, Literal l) {
@@ -91,9 +92,7 @@ void Application::kill(int sig) {
 				printResult(sig);
 			}
 			bool hasModel = config_.ctx.enumerator()->enumerated > 0;
-			fflush(stdout);
-			fflush(stderr);
-			_exit(hasModel ?  S_SATISFIABLE : S_UNKNOWN);
+			CLASP_APP_EXIT(hasModel ?  S_SATISFIABLE : S_UNKNOWN);
 		}
 		else {
 			// multiple threads are active - shutdown was initiated
@@ -154,7 +153,7 @@ void Application::printHelp(const ProgramOptions::OptionContext& root) {
 }
 
 void Application::printVersion(const ProgramOptions::OptionContext&) {
-	printf("clasp %s\n", CLASP_VERSION);	
+	printf("clasp %s (%s-bit)\n", CLASP_VERSION, sizeof(void*)==8?"64":"32");	
 	printf("Configured with: WITH_CLASPRE=%d WITH_THREADS=%d", WITH_CLASPRE, WITH_THREADS);
 #ifndef DISABLE_MULTI_THREADING
 	printf(" (TBB_VERSION=%d.%d)", TBB_VERSION_MAJOR, TBB_VERSION_MINOR);
@@ -233,8 +232,6 @@ int Application::run(int argc, char** argv) {
 		if      (clasp.result() == ClaspFacade::result_unsat) retStatus = S_UNSATISFIABLE;
 		else if (clasp.result() == ClaspFacade::result_sat)   retStatus = S_SATISFIABLE;
 		else                                                  retStatus = S_UNKNOWN;
-		fflush(stdout);
-		fflush(stderr);
 	}
 	catch (const std::bad_alloc&  ) { retStatus = exception(S_MEMORY, "std::bad_alloc"); }
 	catch (const std::exception& e) { retStatus = exception(S_ERROR, e.what()); }
@@ -250,9 +247,7 @@ int Application::exception(int status, const char* what) {
 			printResult(stopped_);
 		}
 	}
-	fflush(stdout);
-	fflush(stderr);
-	_exit(status);
+	CLASP_APP_EXIT(status);
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // State & Result functions
@@ -262,6 +257,7 @@ int Application::exception(int status, const char* what) {
 // for printing.
 void Application::printResult(int sig) {
 	OutputFormat::RunSummary sol(config_.ctx);
+	sol.winner           = 0;
 	sol.complete         = (sig == 0 && !facade_->more());
 	sol.term             = sig;
 	sol.consequences     = config_.consequences() ? config_.cbType() : 0;
@@ -296,11 +292,15 @@ void Application::printResult(int sig) {
 		else {
 			stats.reserve(config_.numThreads()+1);
 			const SolveStats* m = &config_.master()->solver().stats;
+			bool  para          = false;
 			if (m->jumps)    { agg.enableJumpStats(); }
-			if (m->parallel) { agg.enableParallelStats(); }
+			if (m->parallel) { agg.enableParallelStats(); para = true;}
 			stats.push_back(&agg);
 			for (uint32 i = 0; i != config_.numThreads(); ++i) {
 				agg.accu(config_.threadConfig(i)->solver().stats);
+				if (para && config_.threadConfig(i)->solver().stats.parallel->win == 1) {
+					sol.winner = i;
+				}
 				if (cmdOpts_.basic.stats > 1) {
 					stats.push_back(&config_.threadConfig(i)->solver().stats);
 				}
