@@ -49,25 +49,28 @@ bool parsePositional(const std::string& t, std::string& out) {
 	else                 { out = "file";   }
 	return true;
 }
+
 // overload for parsing a clasp schedule - needed
 // to make parsing of schedule composable, e.g. in a pair 
 ProgramOptions::StringSlice parseValue(const ProgramOptions::StringSlice& in, ScheduleStrategy& sched, int) {
 	std::pair<std::string, std::vector<double> > v;
 	bool ok = false;
+	ScheduleStrategy::Type t = ScheduleStrategy::geometric_schedule;
 	if (in.size() > 0) {
 		if      (parseValue(in, v.second, 0).ok()) { ok = true; }
 		else if (parseValue(in, v, 0).ok())        {
 			v.first = toLower(v.first);
 			if      (v.first == "no")                      { ok = v.second.empty(); }
-			else if (v.first == "+" || v.first == "arith") { ok = v.second.size() >= 2; v.first = "+"; }
-			else if (v.first == "*" || v.first == "geom")  { ok = v.second.size() >= 2; }
+			else if (v.first == "+" || v.first == "arith") { ok = v.second.size() >= 2; t = ScheduleStrategy::arithmetic_schedule; }
+			else if (v.first == "*" || v.first == "geom")  { ok = v.second.size() >= 2;  }
 			else if (v.first == "l" || v.first == "luby")  { ok = v.second.size() >= 1; v.second.resize(1); }
 			else if (v.first == "b" || v.first == "biere") { ok = v.second.size() >= 3; v.second.resize(3); }
 		}
 	}
-	if (!ok) { return in.parsed(false); }
+	if (!ok)                  { return in.parsed(false); }
+	if (v.second.size() == 1) { t = ScheduleStrategy::luby_schedule; }
 	v.second.resize(3, 0.0);
-	sched.init(uint32(v.second[0]), v.second[1], (uint64)v.second[2], v.first == "+");
+	sched = ScheduleStrategy(t, uint32(v.second[0]), v.second[1], uint64(v.second[2]));
 	return in.parsed(true, in.size());
 }
 // maps value to the corresponding enum-constant
@@ -109,7 +112,7 @@ void BasicOptions::initOptions(ProgramOptions::OptionContext& root) {
 	ProgramOptions::OptionGroup hidden("Hidden Options");
 	hidden.hidden(true);
 	hidden.addOptions()
-		("file,f", storeTo(input)->composing(), "Input files\n")
+		("file,f", storeTo(input)->composing(), "Input files")
 		("ifs", storeTo(ifs, &BasicOptions::mapIfs), "Internal field separator\n")
 	;
 	root.add(basic);
@@ -137,15 +140,15 @@ void GeneralOptions::initOptions(ProgramOptions::OptionContext& root) {
 	GlobalOptions* global = config;
 	OptionGroup general("Clasp - General Options");
 	general.addOptions()
-		("enum-mode", storeTo(global->enumerate.mode, &parseEnumMode)->defaultsTo("auto")->state(STATE_DEFAULTED), 
+		("enum-mode", storeTo(global->enumerate.mode, &parseEnumMode)->defaultsTo("auto")->state(Value::value_defaulted), 
 			"Configure enumeration algorithm\n"
 			"      Default: %D\n"
 			"      Valid:   auto, bt, record, brave, cautious\n"
 			"        auto    : bt for (projected) enumeration, record for optimization\n"
 			"        bt      : Backtrack decision literals from solutions\n"
 			"        record  : Add nogoods for computed solutions\n"
-			"        brave   : Compute brave consequences\n"
-			"        cautious: Compute cautious consequences")
+			"        brave   : Compute brave consequences (union of all models)\n"
+			"        cautious: Compute cautious consequences (intersection of all models)")
 		("number,n", storeTo(global->enumerate.numModels)->arg("<n>"), 
 			"Compute at most %A models (0 for all)\n"
 			"      Default: 1 / 0 when optimizing/computing consequences")
@@ -194,8 +197,8 @@ void GeneralOptions::initOptions(ProgramOptions::OptionContext& root) {
 		("brave"   , storeTo(global->enumerate.mode, &parseEnumMode)->flag()->implicit("brave"), "Alias for --enum-mode=%I")
 		("cautious"   , storeTo(global->enumerate.mode, &parseEnumMode)->flag()->implicit("cautious") , "Alias for --enum-mode=%I")
 		("solution-recording" , storeTo(global->enumerate.mode, &parseEnumMode)->flag()->implicit("record"), "Alias for --enum-mode=%I")
-		("project-opt", storeTo(global->enumerate.projectOpts), "Additional options for projection as octal digit\n")
-		("eq-dfs"     , flag(global->eq.dfOrder)    , "Enable df-order in eq-preprocessing\n")
+		("project-opt", storeTo(global->enumerate.projectOpts), "Additional options for projection as octal digit")
+		("eq-dfs"     , flag(global->eq.dfOrder)    , "Enable df-order in eq-preprocessing")
 		("learn-explicit", flag(this, GeneralOptions::mapLearnExplicit), "Don't use Short Implication Graph for learning\n")
 	;
 	root.add(hidden);
@@ -264,10 +267,9 @@ bool GeneralOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Me
 SearchOptions::SearchOptions(LocalOptions* o) : local(o) {}
 SolverStrategies* SearchOptions::solverOpts() const { return &local->solver().strategies(); }
 void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
-	SolverStrategies*  solverOpts = this->solverOpts();
 	OptionGroup search("Clasp - Search Options");
 	search.addOptions()
-		("no-lookback"   , notify(this, &SearchOptions::mapCustom)->flag(), "Disable all lookback strategies\n")
+		("no-lookback"   , notify(this, &SearchOptions::mapSolverOpts)->flag(), "Disable all lookback strategies\n")
 		("lookahead"     , storeTo(local->heuristic().lookahead, &mapEnum<lookTypes>)->implicit("atom"),
 			"Configure failed-literal detection\n"
 			"      Default: no (atom, if --nolookback)\n"
@@ -287,7 +289,7 @@ void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
 			"        Vsids  : Apply Chaff-like heuristic\n"
 			"        Unit   : Apply Smodels-like heuristic\n"
 			"        None   : Select the first free variable")
-		("opt-heuristic" , storeTo(local->solver().strategies().heuOpts)->implicit("1")->arg("<n>"), 
+		("opt-heuristic" , notify(this, &SearchOptions::mapSolverOpts)->implicit("1")->arg("<n>"), 
 		 "Configure optimize heuristic\n"
 		  "      Valid: 0 <= <n> <= 3 / Implicit: %I \n"
 		  "         1: Apply sign heuristic\n"
@@ -302,19 +304,19 @@ void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
 			"      Valid:   no (0), <n1,n2> (<n1> >= 0, <n2> > 0) / Implicit: %I\n"
 			"        no     : Do not run random probing\n"
 			"        <n1,n2>: Run <n1> random passes up to at most <n2> conflicts each\n")
-		("rand-watches", storeTo(local->solver().strategies().randomWatches)->implicit("true")->defaultsTo("true"),
+		("rand-watches", notify(this, &SearchOptions::mapSolverOpts)->implicit("true")->defaultsTo("true"),
 			"Configure watched literal initialization\n"
 			"      Default: %D / Implicit: %I\n"
 			"      Valid:   true (yes, 1), false (no, 0)\n"
 			"        true : Randomly determine watched literals\n"
 			"        false: Watch least watched literals in a nogood")
 		("seed"    , storeNotify(aNumber, this, &SearchOptions::mapNumber)->arg("<n>"),    "Set random number generator's seed to %A\n")
-		("search-limit", notify(this, &SearchOptions::mapCustom)->arg("<n,m>"), "Stop search after <n> conflicts or <m> restarts\n")
+		("search-limit", notify(this, &SearchOptions::mapPair)->arg("<n,m>"), "Stop search after <n> conflicts or <m> restarts\n")
 	;
 	
 	OptionGroup lookback("Clasp - Lookback Options");
 	lookback.addOptions()
-		("restarts,r", storeTo(local->solve.restart.sched, &parseSchedule)->defaultsTo("100,1.5")->state(STATE_DEFAULTED),
+		("restarts,r", notify(this, &SearchOptions::mapRestart)->defaultsTo("100,1.5")->state(Value::value_defaulted),
 			"Configure restart policy\n"
 			"      Default: %D\n"
 			"      Valid:   [type,]<n1[,n2,n3]> (<n1> >= 0, <n2>,<n3> > 0), no\n"
@@ -323,33 +325,33 @@ void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
 			"      +,<n1>,<n2>       : Run arithmetic sequence of <n1>+(<n2>*i) conflicts\n"
 			"      [+,]<n1>,<n2>,<n3>: Run Biere's inner-outer sequence (<n3>=outer)\n"
 			"          <n1> = 0, no  : Disable restarts")
-		("local-restarts"  , flag(local->solve.restart.local), "Enable Ryvchin et al.'s local restarts")
+		("local-restarts"  , notify(this, &SearchOptions::mapRestart)->flag(), "Enable Ryvchin et al.'s local restarts")
 		("bounded-restarts", flag(local->solve.restart.bounded), "Enable (bounded) restarts during model enumeration")
 		("reset-restarts",   flag(local->solve.restart.resetOnModel), "Reset restart strategy during model enumeration")
-		("save-progress"   , storeTo(solverOpts->saveProgress)->implicit("1")->arg("<n>"), "Enable RSat-like progress saving on backjumps > %A")
+		("save-progress"   , notify(this, &SearchOptions::mapSolverOpts)->implicit("1")->arg("<n>"), "Enable RSat-like progress saving on backjumps > %A")
 
-		("shuffle,s", notify(this, &SearchOptions::mapCustom)->arg("<n1,n2>"),
+		("shuffle,s", notify(this, &SearchOptions::mapPair)->arg("<n1,n2>"),
 			"Configure shuffling after restarts\n"
 			"      Valid:   <n1,n2> (<n1> >= 0, <n2> >= 0)\n"
 			"        <n1> > 0: Shuffle problem after <n1> and re-shuffle every <n2> restarts\n"
 			"        <n1> = 0: Do not shuffle problem after restarts\n"
 			"        <n2> = 0: Do not re-shuffle problem\n")
 
-		("deletion,d", notify(this, &SearchOptions::mapCustom)->defaultsTo("3.0,1.1,3.0")->state(STATE_DEFAULTED), 
+		("deletion,d", notify(this, &SearchOptions::mapReduceOpts)->defaultsTo("3.0,1.1,3.0")->state(STATE_DEFAULTED), 
 			"Configure size of learnt nogood database\n"
 			"      Default: %D\n"
 			"      Valid:   <n1[,n2,n3]> (<n3> >= <n1> >= 0, <n2> >= 1.0), no\n"
 			"        <n1,n2,n3>: Store at most min(P/<n1>*(<n2>^i),P*<n3>) learnt nogoods,\n"
 			"                    P and i being initial problem size and number of restarts\n"
 			"        no        : Do not restrict learnt db size")
-		("dsched", storeTo(local->solve.reduce.sched, &parseSchedule), "Configure secondary deletion policy (see restarts)")
-		("dglue", storeTo(solverOpts->reduceGlue), "Don't delete nogoods with lbd <= x")
-		("update-lbd", flag(solverOpts->updateLbd), "Dynamically update lbds of learnt nogoods")
-		("reduce-on-restart", flag(local->solve.reduce.reduceOnRestart), "Delete some learnt nogoods after every restart\n")
 		("estimate", flag(local->solve.reduce.estimate), "Use estimated problem complexity to init learnt db")
-		("dinit",notify(this, &SearchOptions::mapCustom)->arg("<min,max>"), "Limit initial learnt db size to range [min,max]\n")
+		("dinit",notify(this, &SearchOptions::mapReduceOpts)->arg("<min,max>"), "Limit initial learnt db size to range [min,max]")
+		("dsched", storeTo(local->solve.reduce.cflSched, &parseSchedule), "Configure secondary deletion policy (see restarts)")
+		("dglue", notify(this, &SearchOptions::mapReduceOpts), "Don't delete nogoods with lbd <= x")
+		("update-lbd", notify(this, &SearchOptions::mapSolverOpts)->implicit("1"), "Update lbds of learnt nogoods (1=yes,2=strict)")
+		("reduce-on-restart", flag(local->solve.reduce.reduceOnRestart), "Delete some learnt nogoods after every restart\n")
 
-		("strengthen", storeTo(solverOpts->cflMinAntes, &mapEnum<anteTypes>)->defaultsTo("all")->state(STATE_DEFAULTED),
+		("strengthen", notify(this, &SearchOptions::mapSolverOpts)->defaultsTo("all")->state(STATE_DEFAULTED),
 			"Configure conflict nogood strengthening\n"
 			"      Default: %D\n"
 			"      Valid:   bin, tern, all, no\n"
@@ -357,9 +359,9 @@ void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
 			"        tern: Check binary and ternary antecedents for self-subsumption\n"
 			"        all : Check all antecedents for self-subsumption\n"
 			"        no  : Do not check antecedents for self-subsumption")
-		("recursive-str", flag(solverOpts->strengthenRecursive), "Enable MiniSAT-like conflict nogood strengthening")
-		("otfs",storeTo(solverOpts->otfs)->implicit("1"), "Enable on-the-fly subsumption (1=partial, 2=full)")
-		("reverse-arcs",storeTo(solverOpts->reverseArcs)->implicit("1"), "Enable ManySAT-like reverse-arcs\n")
+		("recursive-str", notify(this, &SearchOptions::mapSolverOpts)->flag(), "Enable MiniSAT-like conflict nogood strengthening")
+		("otfs",notify(this, &SearchOptions::mapSolverOpts)->implicit("1"), "Enable on-the-fly subsumption (1=partial, 2=full)")
+		("reverse-arcs",notify(this, &SearchOptions::mapSolverOpts)->implicit("1"), "Enable ManySAT-like inverse-arc learning\n")
 		
 		("loops", storeTo(local->loopRep, &mapEnum<loopTypes>)->defaultsTo("common")->state(STATE_DEFAULTED),
 			"Configure representation/learning of loop formulas\n"
@@ -382,16 +384,20 @@ void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
 	hidden.hidden(true);
 	hidden.addOptions()
 		("berk-max", storeTo(local->heuristic().extra.berkMax), "Consider at most <n> nogoods in Berkmin heuristic")
-		("berk-moms", flag(local->heuristic().berkMoms), "Enable/Disable MOMs in Berkmin")
-		("berk-huang",flag(local->heuristic().berkHuang), "Enable/Disable Huang-scoring in Berkmin")
+		("berk-moms", storeTo(local->heuristic().berkMoms)->implicit("1"), "Enable/Disable MOMs in Berkmin")
+		("berk-huang",flag(local->heuristic().berkHuang), "Enable Huang-scoring in Berkmin")
 		("berk-once",flag(local->heuristic().berkOnce), "Score sets (instead of multisets) in Berkmin")
 		("vmtf-mtf",storeTo(local->heuristic().extra.vmtfMtf), "In Vmtf move <n> conflict-literals to the front")
+		("vsids-decay",storeTo(local->heuristic().extra.vsidsDecay), "In Vsids use 1.0/0.<n> as decay factor")
 		("nant",flag(local->heuristic().nant), "In Unit count only atoms in NAnt(P)\n")
 		("dfrac", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Set fraction of nogoods to delete on reduction")
-		("dalgo", storeTo(solverOpts->reduceAlgo), "Use {0=basic, 1=sort-in-place, 2=sorted} deletion")
-		("dscore", storeTo(solverOpts->reduceScore), "Use {0=activity, 1=lbd, 2=combined} nogood scores")
-		("cir", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Enable CIR with interval %A (0=disable)")
+		("dalgo", notify(this, &SearchOptions::mapReduceOpts), "Use {0=basic, 1=sort-in-place, 2=sorted} deletion")
+		("dscore", notify(this, &SearchOptions::mapReduceOpts), "Use {0=activity, 1=lbd, 2=combined} nogood scores")
+		("dgrowS", notify(this, &SearchOptions::mapReduceOpts), "Set deletion grow schedule (see restarts)")
+		("digng", notify(this, &SearchOptions::mapReduceOpts)->flag(), "Do not count glue clauses in learnt limit")
+		("cir", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Do CI Restart every %A restarts (0=disable)")
 		("cir-bump", storeTo(local->solve.restart.cirBump), "Set CIR bump factor to %A")
+		("dynamic-restarts", notify(this, &SearchOptions::mapRestart), "Enable restarts based on lbd/cfl progression\n")
 	;
 	root.add(search);
 	root.add(lookback);
@@ -438,43 +444,92 @@ bool SearchOptions::parseSchedule(const std::string& s, ScheduleStrategy& sched)
 
 bool SearchOptions::mapNumber(SearchOptions* this_, const std::string& opt, const double* v) {
 	if      (opt == "seed")        { this_->solverOpts()->rng.srand(static_cast<uint32>(*v)); }
-	else if (opt == "contraction") { this_->solverOpts()->setCompressionStrategy(static_cast<uint32>(*v)); }
-	else if (opt == "dfrac")       { this_->local->solve.reduce.setReduceFraction(*v); }
+	else if (opt == "contraction") { this_->solverOpts()->compress = static_cast<uint32>(*v); }
+	else if (opt == "dfrac")       { this_->local->solve.reduce.remFrac = (float)*v; }
 	else if (opt == "cir")         { this_->local->solve.restart.cir = (uint8)*v; }
 	this_->aNumber = 0.0;
 	return true;
 }
 
-bool SearchOptions::mapCustom(SearchOptions* this_, const std::string& n, const std::string& value) {
+bool SearchOptions::mapRestart(SearchOptions* this_, const std::string& n, const std::string& value) {
 	bool ok = false;
-	if (n == "no-lookback") {
-		bool b;
-		ok    = FlagStr::store_true(value, b);
-		if (ok && b) this_->solverOpts()->search = Clasp::SolverStrategies::no_learning;
+	if (n == "restarts") {
+		uint8 t = this_->local->solve.restart.type;
+		ok      = true;
+		if (t != RestartParams::dynamic_restarts) {
+			this_->local->solve.restart.type = (t & RestartParams::local_restarts);
+			ok = parseSchedule(value, this_->local->solve.restart.sched);
+		}
 	}
-	else if (n == "deletion") {
+	if (n == "dynamic-restarts") {
+		std::vector<double> v; v.reserve(3);
+		ok = parse(value, v) && !v.empty();
+		if (ok && v.size() == 1) { v.push_back(0.7); }
+		if (ok && v.size() == 2) { v.push_back(0.0); }
+		ok = ok && v.size() == 3;
+		if (ok) { 
+			this_->local->solve.restart.initDynamic((uint32)v[0], v[1], v[2]);
+		}
+	}
+	else if (n == "local-restarts") {
+		bool b = false;
+		ok     = FlagStr::store_true(value, b);
+		uint8 t= this_->local->solve.restart.type;
+		if (b && t != RestartParams::dynamic_restarts) t |= RestartParams::local_restarts;
+		this_->local->solve.restart.type = t;
+	}
+	return ok;
+}
+
+bool SearchOptions::mapReduceOpts(SearchOptions* this_, const std::string& n, const std::string& value) {
+	ReduceParams& p = this_->local->solve.reduce;
+	bool ok = false, b; int num;
+	if (n == "deletion") {
 		std::vector<double> v;
-		if (value == "no" || parse(value, v)) {
-			ok = true;
-			if (v.size()==1) { v.push_back(1.1); }
-			if (v.size()==2) { v.push_back(3.0); }
-			if (v.empty())   { this_->local->solve.reduce.disable = true; }
-			else             { this_->local->solve.reduce.setStrategy(v[0], v[1], v[2]); }
-		}
+		ok = value == "no" || parse(value, v);
+		if (v.empty())     { p.baseFrac = 0; }
+		if (v.size() >= 1) { p.baseFrac = std::max(0.0001f, (float)v[0]);    }
+		if (v.size() >= 2) { p.dbGrow   = std::max(1.0f   , (float)v[1]);    }
+		if (v.size() >= 3) { p.dbMaxGrow= std::max(p.baseFrac, (float)v[2]); }
 	}
-	else {
-		std::pair<int, int> p(-1, -1);
-		if ((ok = parse(value, p))) {
-			if (n == "dinit") {
-				this_->local->solve.reduce.setInit((uint32)p.first, (uint32)p.second);
-			}
-			else if (n == "shuffle") {
-				this_->local->solve.setShuffleParams(p.first, p.second);
-			}
-			else if (n == "search-limit") {
-				this_->local->solve.limits = SolveLimits(static_cast<uint64>(p.first), static_cast<uint64>(p.second));
-			}
-		}
+	else if (n == "dinit") {
+		std::pair<int, int> x(-1, -1);
+		ok = parse(value, x);
+		p.baseMin = (uint32)x.first; 
+		p.baseMax = std::max((uint32)x.second, p.baseMin); 
+	}
+	else if (n == "dgrowS"){ ok = parseSchedule(value, p.growSched); }
+	else if (n == "dglue") { ok = parse(value, num) && (p.strategy.glue = (uint32)num) == (uint32)num; }
+	else if (n == "dalgo") { ok = parse(value, num) && (p.strategy.algo = (uint32)num) == (uint32)num && num < 3;}
+	else if (n == "dscore"){ ok = parse(value, num) && (p.strategy.score= (uint32)num) == (uint32)num && num < 3;}
+	else if (n == "digng") { ok = FlagStr::store_true(value, b) && (p.strategy.noGlue= (uint32)b) == (uint32)b;  }
+	return ok;
+}
+
+bool SearchOptions::mapSolverOpts(SearchOptions* this_, const std::string& n, const std::string& v) {
+#define SET(M, N) ((opts->M = (uint32)N) == (uint32)N)
+	SolverStrategies* opts = this_->solverOpts();
+	int  i; bool b;
+	if      (n == "strengthen")    { return mapEnum<anteTypes>(v, i) && SET(cflMinAntes, i); }
+	else if (n == "rand-watches")  { return parse(v, b) && SET(randomWatches, b);}
+	else if (n == "save-progress") { return parse(v, i) && SET(saveProgress, i); }
+	else if (n == "opt-heuristic") { return parse(v, i) && SET(heuOpts, i);      }
+	else if (n == "otfs")          { return parse(v, i) && SET(otfs, i);         }
+	else if (n == "reverse-arcs")  { return parse(v, i) && SET(reverseArcs, i);  }
+	else if (n == "update-lbd")    { return parse(v, i) && SET(updateLbd, i);    }
+	else if (n == "recursive-str") { return FlagStr::store_true(v, b) && SET(strRecursive, b); }
+	else if (n == "no-lookback")   { return FlagStr::store_true(v, b) && SET(search, b);       }
+#undef SET
+	return false;
+}
+
+bool SearchOptions::mapPair(SearchOptions* this_, const std::string& n, const std::string& value) {
+	std::pair<int, int> p(-1, -1);
+	bool ok = parse(value, p);
+	if (ok) {
+		if      (n == "shuffle")      { this_->local->solve.setShuffleParams(p.first, p.second); }
+		else if (n == "search-limit") { this_->local->solve.limits = SolveLimits(static_cast<uint64>(p.first), static_cast<uint64>(p.second)); }
+		else                          { return false; }
 	}
 	return ok;
 }
@@ -483,12 +538,16 @@ bool SearchOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Mes
 	if (solverOpts()->search == Clasp::SolverStrategies::no_learning) {	
 		if (vm.count("heuristic") == 0) { local->heuristic().name = "unit"; }
 		if (vm.count("lookahead") == 0) { local->heuristic().lookahead = Lookahead::atom_lookahead; }
-		bool warn = local->solve.restart.local || local->solve.restart.bounded || local->solve.reduce.reduceOnRestart;
+		bool warn = local->solve.restart.type != 0 || local->solve.restart.bounded || local->solve.reduce.reduceOnRestart;
 		if (warn || vm.count("restarts") || vm.count("deletion") || vm.count("rand-prob") || vm.count("shuffle")) {
 			m.warning.push_back("lookback-options ignored because lookback strategy is not used!");     
 		}
 	}
-	return true;
+	if (local->solve.restart.type == RestartParams::dynamic_restarts) {
+		if      (vm.count("restarts"))       { m.error = "'restarts' and 'dynamic-restarts' are mutually exclusive!"; }
+		else if (vm.count("local-restarts")) { m.error = "'local-restarts' and 'dynamic-restarts' are mutually exclusive!"; }
+	}
+	return m.error.empty();
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -516,8 +575,8 @@ void ClaspOptions::initThreadOptions(ProgramOptions::OptionContext& root) {
 		("portfolio,p", storeTo(portfolio)->arg("<file>"), 
 		 "Use portfolio to configure threads\n"
 		 "      Valid: %A or 'default'")
-		("create-template,g" , flag(genTemplate), "Print a template portfolio and exit\n")
-		("force-gp", flag(thread->forceGP), "Force guiding path scheme even if portfolio is used")
+		("force-gp", flag(thread->forceGP), "Force guiding path scheme in portfolio search")
+		("create-template,g" , flag(genTemplate), "Print default portfolio and exit\n")
 		("global-restarts", notify(this, &ClaspOptions::mapThreadOpts)->implicit("5,100,1.5"),
 		 "Configure global restart policy (0 if not given)\n"
 		 "      Valid: <n>[,strat>] / Implicit: %I\n"
@@ -613,7 +672,7 @@ bool ClaspOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Mess
 				port = mem.data();
 			}
 		}
-		populateThreadConfigs(port, vm, m);
+		ok = populateThreadConfigs(port, vm, m);
 	}
 	return ok && config->validate(m.error);
 }
@@ -649,22 +708,22 @@ bool ClaspOptions::parsePortfolio(std::istream& in, uint32 max, std::string& mem
 }
 
 bool ClaspOptions::populateThreadConfigs(const char* port, const ProgramOptions::ParsedOptions& vm, Messages& m) {
-	if (port == 0) {
-		config->thread.forceGP = true;
-		for (uint32 i = 1; i < config->numThreads(); ++i) {
-			config->threadConfig(i)->initFrom(*config->threadConfig(0));
-		}
+	for (uint32 i = 1; i < config->numThreads(); ++i) {
+		// copy all from command-line
+		// NOTE: this also copies any default values
+		config->threadConfig(i)->initFrom(*config->threadConfig(0));
 	}
-	else {
+	if (port != 0) { 
 		bool addSeed = false;
 		const char* p= port;
 		for (uint32 i = 0; i != config->numThreads(); ++i) {
-			if (!parseConfig(p, i, vm, m)) { return false; }
-			if (addSeed)                   { config->threadConfig(i)->solver().strategies().rng.srand(i); }
-			if (*p)                        { p += strlen(p) + 1; }
-			if (!*p)                       { p  = port; addSeed = true; }
+			if (!parseConfig(p, i, vm,m)){ return false; }
+			if (addSeed)                 { config->threadConfig(i)->solver().strategies().rng.srand(i); }
+			if (*p)                      { p += strlen(p) + 1; }
+			if (!*p)                     { p  = port; addSeed = true; }
 		}
 	}
+	else { config->thread.forceGP = true; }
 	return true;
 }
 
@@ -689,25 +748,33 @@ bool ClaspOptions::parseConfig(const char* in, uint32 id, const ProgramOptions::
 		ProgramOptions::OptionContext root;
 		SearchOptions opts(config->threadConfig(id));
 		opts.initOptions(root);
-		// 1. all from command-line
-		if (id != 0) { opts.local->initFrom(*config->master()); }
 		ProgramOptions::ParsedOptions parsed(vm);
-		// 2. all from current config
+		// HACK: ignore all restarts once command-line has restarts
+		bool hasR            = vm.count("dynamic-restarts") || vm.count("restarts") || vm.count("local-restarts");
+		ScheduleStrategy old = opts.local->solve.restart.sched;
+		uint8 oldType        = opts.local->solve.restart.type;
+		// add all from current config
 		parsed.assign(ProgramOptions::parseCommandString(configOpts, root));
-		// 3. any defaults
-		root.assignDefaults(parsed);
 		if (!opts.validateOptions(parsed, m)) {
-			return false;
+			if (!hasR || m.error.find("restarts") == std::string::npos) {
+				throw std::runtime_error(m.error);
+			}
+			m.error.clear();
 		}
+		if (hasR) {
+			opts.local->solve.restart.sched = old;
+			opts.local->solve.restart.type  = oldType;
+		}
+		return true;
 	}
 	catch(const std::exception& e) {
 		std::string err("Config '");
 		err.append(nameBeg, (nameEnd-nameBeg));
 		err += "': ";
 		err += e.what();
-		throw std::runtime_error(err);
+		m.error = err;
+		return false;
 	}
-	return true;
 }
 
 const char* portfolio_g = {
