@@ -94,9 +94,12 @@ struct SearchOptions {
 	static bool parseHeuristic(const std::string& s, std::string&);
 	static bool parseSchedule(const std::string& s, ScheduleStrategy& sched);
 	// value mapping
-	static bool mapCustom(SearchOptions*  , const std::string& opt, const std::string& v);
-	static bool mapNumber(SearchOptions*  , const std::string& opt, const double* v);
-	static bool mapRandOpts(SearchOptions*, const std::string& opt, const std::string& v);
+	static bool mapSolverOpts(SearchOptions*, const std::string& opt, const std::string& v);
+	static bool mapPair(SearchOptions*      , const std::string& opt, const std::string& v);
+	static bool mapRestart(SearchOptions*   , const std::string& opt, const std::string& v);
+	static bool mapNumber(SearchOptions*    , const std::string& opt, const double* v);
+	static bool mapRandOpts(SearchOptions*  , const std::string& opt, const std::string& v);
+	static bool mapReduceOpts(SearchOptions*, const std::string& opt, const std::string& v);
 	LocalOptions*      local;
 	SolverStrategies*  solverOpts() const;
 	double             aNumber;
