@@ -115,7 +115,7 @@ void OutputFormat::reportResult(const RunSummary& sol, const SolveStats** stats,
 			printConsequences(sol.ctx.symTab(), en, sol.consequences);
 		}
 		else {
-			Model m(curr_, curr_ > next_ ? curr_ - next_ : next_ - curr_);
+			Model m(curr_, curr_ > next_ ? static_cast<uint32>(curr_ - next_) : static_cast<uint32>(next_ - curr_));
 			printModel(m, sol.ctx.symTab(), en);
 		}
 	}
@@ -139,7 +139,7 @@ OutputFormat::Model OutputFormat::storeModel(const Assignment& a) {
 	}
 	curr_      = next_;
 	next_      = curr_ == &saved_[0] ? x : &saved_[0];
-	return Model(curr_, curr_ > next_ ? curr_ - next_ : next_ - curr_);
+	return Model(curr_, curr_ > next_ ? static_cast<uint32>(curr_ - next_) : static_cast<uint32>(next_ - curr_));
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // DefaultOutput
@@ -397,12 +397,13 @@ void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 		printKeyValue("CPU Time", "%.3fs\n", sol.cpuTime);
 	}
 	if (verbosity() >= 0 && num != 0) {
-		printStats(sol.ctx.stats(), stats, num);
+		printStats(sol, stats, num);
 	}
 }
 
-void DefaultOutput::printStats(const ProblemStats& pr, const SolveStats** stats, std::size_t num) {
-	const SolveStats* st = stats[0];
+void DefaultOutput::printStats(const RunSummary& sol, const SolveStats** stats, std::size_t num) {
+	const ProblemStats& pr = sol.ctx.stats();
+	const SolveStats* st   = stats[0];
 	printSolveStats(*st);
 	printf("\n");
 	if (lpStats()) {
@@ -412,14 +413,18 @@ void DefaultOutput::printStats(const ProblemStats& pr, const SolveStats** stats,
 	printProblemStats(pr);
 	printLemmaStats(*st);
 	printJumpStats(*st);
-	printThreadStats(stats, num);
+	printThreadStats(stats, num, sol.winner);
 	fflush(stdout);
 }
 
 void DefaultOutput::printSolveStats(const SolveStats& st) const {
 	printKeyValue("Choices", "%"PRIu64"\n", st.choices);
 	printKeyValue("Conflicts", "%"PRIu64"\n", st.conflicts);
-	printKeyValue("Restarts", "%"PRIu64"\n", st.restarts);
+	printKeyValue("Restarts", "%-6"PRIu64"", st.restarts);
+	if (st.restarts) {
+		printf(" (Average: %.2f Last: %"PRIu64")", average(st.analyzed, st.restarts), st.analyzed - st.lRestart);
+	}
+	printf("\n");
 }
 
 void DefaultOutput::printLpStats(const PreproStats& lp) const {
@@ -489,26 +494,27 @@ void DefaultOutput::printJumpStats(const SolveStats& st) const {
 		const JumpStats& js = *st.jumps;
 		w_ += 8;
 		printf("\n");
-		printKeyValue("Backtracks", "%-6"PRIu64"\n", st.conflicts-js.jumps);
-		printKeyValue("Backjumps", "%-6"PRIu64, js.jumps);
+		printKeyValue("Backtracks", "%-6"PRIu64"\n", st.conflicts-st.analyzed);
+		printKeyValue("Backjumps", "%-6"PRIu64, st.analyzed);
 		printf(" (Bounded: %"PRIu64")\n", js.bJumps);
 		printKeyValue("Skippable Levels", "%-6"PRIu64, js.jumpSum);
 		printf(" (Skipped: %"PRIu64" Rate: %5.1f%%)\n", js.jumpSum - js.boundSum, percent(js.jumpSum - js.boundSum, js.jumpSum));
 		printKeyValue("Max Jump Length", "%-6u", js.maxJump);
 		printf(" (Executed: %u)\n", js.maxJumpEx);
 		printKeyValue("Max Bound Length", "%-6u\n", js.maxBound);
-		printKeyValue("Average Jump Length", "%-6.1f", js.avgJumpLen());
-		printf(" (Executed: %.1f)\n", js.avgJumpLenEx());
+		printKeyValue("Average Jump Length", "%-6.1f", average(js.jumpSum, st.analyzed));
+		printf(" (Executed: %.1f)\n", average(js.jumpSum-js.boundSum, st.analyzed));
 		printKeyValue("Average Bound Length", "%-6.1f\n", average(js.boundSum,js.bJumps));
 		printKeyValue("Average Model Length", "%-6.1f\n", average(st.modLits,st.models));
 		w_ -= 8;
 	}
 }
 
-void DefaultOutput::printThreadStats(const SolveStats** stats, std::size_t num) const {
+void DefaultOutput::printThreadStats(const SolveStats** stats, std::size_t num, uint32 winner) const {
 	if (num > 1 || stats[0]->parallel) {
 		printf("\n");
 		comment(0, "============ PARALLEL STATISTICS ============\n\n");
+		printKeyValue("Winner",  "%u\n", winner);
 		DefaultOutput::printParallelStats(*stats[0], true);
 		for (std::size_t i = 1; i < num; ++i) {
 			comment(0, "[Thread %u]\n", (uint32)i-1);
@@ -524,7 +530,7 @@ void DefaultOutput::printThreadStats(const SolveStats** stats, std::size_t num)
 void DefaultOutput::printParallelStats(const SolveStats& stats, bool accu) const {
 	if (stats.parallel) {
 		const ParallelStats& ps = *stats.parallel;
-		uint64 solved = ps.gps ? ps.gps-uint64(ps.terminated) : 0;
+		uint64 solved = ps.gps ? ps.gps-uint64(ps.term) : 0;
 		if (!accu) {
 			printKeyValue("CPU Time",  "%.3fs\n", ps.cpuTime);
 			printKeyValue("Models", "%"PRIu64"\n", stats.models);
@@ -741,7 +747,7 @@ void JsonOutput::printResult(const RunSummary& sol, const SolveStats** stats, st
 		}
 		printLemmaStats(*st);
 		printJumpStats(*st);
-		printThreadStats(stats, num);
+		printThreadStats(stats, num, sol.winner);
 		endObject(); // Stats
 	}
 	else {
@@ -822,26 +828,27 @@ void JsonOutput::printJumpStats(const SolveStats& st) {
 	if (st.jumps) {
 		const JumpStats& js = *st.jumps;
 		startObject("Jumps");
-		printKeyValue("Backtracks", st.conflicts-js.jumps);
-		printKeyValue("Backjumps", js.jumps);
+		printKeyValue("Backtracks", st.conflicts-st.analyzed);
+		printKeyValue("Backjumps", st.analyzed);
 		printKeyValue("Bounded", js.bJumps);
 		printKeyValue("Skippable", js.jumpSum);
 		printKeyValue("Skipped", js.jumpSum - js.boundSum);
 		printKeyValue("MaxJump", js.maxJump);
 		printKeyValue("MaxJumpEx", js.maxJumpEx);
 		printKeyValue("MaxBound", js.maxBound);
-		printKeyValue("AvgJump", js.avgJumpLen());
-		printKeyValue("AvgJumpEx", js.avgJumpLenEx());
+		printKeyValue("AvgJump", average(js.jumpSum, st.analyzed));
+		printKeyValue("AvgJumpEx", average(js.jumpSum-js.boundSum, st.analyzed));
 		printKeyValue("AvgBound", average(js.boundSum,js.bJumps));
 		printKeyValue("AvgModel", average(st.modLits,st.models));
 		endObject();
 	}
 }
 
-void JsonOutput::printThreadStats(const SolveStats** stats, std::size_t num) {
+void JsonOutput::printThreadStats(const SolveStats** stats, std::size_t num, uint32 winner) {
 	if (stats[0]->parallel) {
 		startObject("Parallel");
 		const ParallelStats& ps = *stats[0]->parallel;
+		printKeyValue("Winner", winner);
 		printKeyValue("Distributed", ps.shared);
 		printKeyValue("Splits", ps.splits);
 		printKeyValue("Problems", ps.gps);
@@ -864,7 +871,7 @@ void JsonOutput::printThreadStats(const SolveStats** stats, std::size_t num) {
 void JsonOutput::printParallelStats(const SolveStats& stats) {
 	if (stats.parallel) {
 		const ParallelStats& ps = *stats.parallel;
-		uint64 solved = ps.gps ? ps.gps-uint64(ps.terminated) : 0;
+		uint64 solved = ps.gps ? ps.gps-uint64(ps.term) : 0;
 		startObject("Basic");
 		printKeyValue("CPU Time", ps.cpuTime);
 		printKeyValue("Models", stats.models);
@@ -985,7 +992,7 @@ void ClaspreOutput::reportProgress(Enumerator::ProgressType t, const Solver& s,
 		printf("%-22s: %.3f\n", "Vars/Constraints", ((double)s.numVars())/s.numConstraints());
 		printf("%-22s: %.3f\n", "maxLearnt/Constraints", ((double)maxL)/s.numConstraints());
 		if (s.stats.jumps) {
-			printf("%-22s: %.3f\n", "Average Jump Length", s.stats.jumps->avgJumpLen());
+			printf("%-22s: %.3f\n", "Average Jump Length", average(s.stats.jumps->jumpSum, s.stats.analyzed));
 		}
 		printf("=================================\n\n");
 	}
@@ -1049,9 +1056,9 @@ void ClaspreOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 			uint64 learntSum      = std::accumulate(st.learnts, st.learnts+Constraint_t::max_value, uint64(0));
 			temp.precision(3);
 			temp << std::fixed << "," << st.models << "," << st.choices << "," << st.conflicts << "," << st.restarts << "," << st.deleted << "," 
-					 << (st.conflicts-js.jumps) << "," << js.jumps << "," << js.bJumps << ","
+					 << (st.conflicts-st.analyzed) << "," << st.analyzed << "," << js.bJumps << ","
 					 << js.jumpSum << "," << (js.jumpSum - js.boundSum) << "," << percent(js.jumpSum - js.boundSum, js.jumpSum) << ","
-					 << js.maxJump << "," << js.maxJumpEx << "," << js.maxBound << "," << js.avgJumpLen() << "," << js.avgJumpLenEx() << ","
+					 << js.maxJump << "," << js.maxJumpEx << "," << js.maxBound << "," << average(js.jumpSum, st.analyzed) << "," << average(js.jumpSum-js.boundSum, st.analyzed) << ","
 					 << average(js.boundSum,js.bJumps) << "," << average(st.modLits,st.models) << ","
 					 << learntSum << "," << percent(st.binary, learntSum) << "," << percent(st.ternary, learntSum) << "," 
 					 << percent(learntSum-st.binary-st.ternary, learntSum) << ","
