@@ -57,15 +57,16 @@ public:
 			not_a_result=5 
 		};
 		RunSummary(const SharedContext& a_ctx) : ctx(a_ctx) {}
-		const SharedContext& ctx;      /**< context object of the run */ 
-		double               totalTime;/**< total wall clock time */
-		double               solveTime;/**< time solving */
-		double               modelTime;/**< time to first model */
-		double               unsatTime;/**< time to prove unsat */
-		double               cpuTime;  /**< total cpu time */
-		const char*       consequences;/**< type of consequences computed or 0 */
-		int                  term;     /**< termination signal or 0 */
-		bool                 complete; /**< search space exhausted? */
+		const SharedContext& ctx;       /**< context object of the run */ 
+		double               totalTime; /**< total wall clock time */
+		double               solveTime; /**< time solving */
+		double               modelTime; /**< time to first model */
+		double               unsatTime; /**< time to prove unsat */
+		double               cpuTime;   /**< total cpu time */
+		const char*       consequences; /**< type of consequences computed or 0 */
+		int                  term;      /**< termination signal or 0 */
+		uint32               winner:31; /**< id of solver that terminated the algorithm */
+		uint32               complete:1;/**< search space exhausted? */
 		Result result() const;
 		uint64 models() const;
 	private: RunSummary& operator=(const RunSummary&);
@@ -248,11 +249,11 @@ private:
 	void printSolveStats(const SolveStats& st) const;
 	void printLemmaStats(const SolveStats& st) const;
 	void printJumpStats(const SolveStats& st) const;
-	void printThreadStats(const SolveStats** st, std::size_t num) const;
+	void printThreadStats(const SolveStats** st, std::size_t num, uint32 winner) const;
 	void printParallelStats(const SolveStats& st, bool accu) const;
 	void printProblemStats(const ProblemStats& st) const;
 	void printLpStats(const PreproStats& st) const;
-	void printStats(const ProblemStats& pr, const SolveStats** st, std::size_t num);
+	void printStats(const RunSummary& sol, const SolveStats** st, std::size_t num);
 	void comment(int v, const char* fmt, ...) const;
 	void configureFormat(Format f, std::pair<int, int> q, int v, char ifs);
 	const SatPreprocessor* prepro_;
@@ -276,7 +277,7 @@ private:
 	void startModel();
 	void printLemmaStats(const SolveStats& st);
 	void printJumpStats(const SolveStats& st);
-	void printThreadStats(const SolveStats** st, std::size_t num);
+	void printThreadStats(const SolveStats** st, std::size_t num, uint32 winner);
 	void printParallelStats(const SolveStats& st);
 	void printLpStats(const PreproStats& st);
 	inline void printKey(const char* k) ;
