@@ -29,29 +29,29 @@
 
 namespace Clasp { 
 
-//! An array of literals that can be shared
+//! An array of literals that can be shared between threads.
 class SharedLiterals {
 public:
-	//! creates a shareable (ref-counted) object containing the literals in lits
+	//! Creates a shareable (ref-counted) object containing the literals in lits.
 	/*!
-	 * \note The reference count is set to numRefs
+	 * \note The reference count is set to numRefs.
 	 */
 	static SharedLiterals* newShareable(const LitVec& lits, ConstraintType t, uint32 numRefs = 1) {
 		return newShareable(!lits.empty() ? &lits[0]:0, static_cast<uint32>(lits.size()), t, numRefs);
 	}
 	static SharedLiterals* newShareable(const Literal* lits, uint32 size, ConstraintType t, uint32 numRefs = 1);
 	
-	//! returns a pointer to the beginning of the literal array
+	//! Returns a pointer to the beginning of the literal array.
 	const Literal* begin() const { return lits_; }
-	//! returns a pointer to the end of the literal array
+	//! Returns a pointer to the end of the literal array.
 	const Literal* end()   const { return lits_+size(); }
-	//! returns the number of literals in the array
+	//! Returns the number of literals in the array.
 	uint32         size()  const { return size_type_ >> 2; }
-	//! returns the type of constraint from which the literals originated
+	//! Returns the type of constraint from which the literals originated.
 	ConstraintType type()  const { return ConstraintType( size_type_ & uint32(3) ); }
-	//! simplifies the literals w.r.t to the assignment in s
+	//! Simplifies the literals w.r.t to the assignment in s.
 	/*!
-	 * returns the number of non-false literals in this object or 0 if
+	 * Returns the number of non-false literals in this object or 0 if
 	 * the array contains a true literal.
 	 * \note If the object is currently not shared, simplify() removes
 	 * all false literals from the array.
@@ -81,97 +81,104 @@ private:
  */
 class ClauseCreator {
 public:
-	//! Creates a new ClauseCreator object
+	//! Creates a new ClauseCreator object.
 	/*!
 	 * \param s the Solver in which to store created clauses.
 	 */
 	explicit ClauseCreator(Solver* s = 0);
-	//! sets the solver in which created clauses are stored.
+	//! Sets the solver in which created clauses are stored.
 	void setSolver(Solver& s) { solver_ = &s; }
-	//! reserve space for a clause of size s
+	//! Reserve space for a clause of size s.
 	void reserve(LitVec::size_type s) { literals_.reserve(s); }
-	//! sets the initial activity for the to be created clause
+	//! Sets the initial activity for the to be created clause.
 	void setActivity(uint32 a)        { extra_.setActivity(a);  }
-	//! sets the initial literal block distance for the to be created clause
+	//! Sets the initial literal block distance for the to be created clause.
 	void setLbd(uint32 l1, uint32 l2) { extra_.setLbd(l1, l2); }
-	//! discards the current clause
+	//! Discards the current clause.
 	void clear() { literals_.clear(); }
 	
-	//! starts the creation of a new clause
+	//! Starts the creation of a new clause.
 	/*!
 	 * \pre s.decisionLevel() == 0 || t != Constraint_t::static_constraint
 	 */
 	ClauseCreator& start(ConstraintType t = Constraint_t::static_constraint);
 	
-	//! start the creation of an asserting or unit clause.
+	//! Start the creation of an asserting or unit clause.
 	/*!
    * All literals except the one passed to startAsserting() must be
-	 * false and at least one of the false literals must be assigned on
-	 * the current decision level.
-	 * \param t The type of the learnt clause: either conflict or loop.
-	 * \param assertingLit the literal to be asserted.
-	 * \note position 0 is always reserved for the asserting literal.
+	 * false.
+	 * \param t The type of the learnt clause.
+	 * \param assertingLit The literal to be asserted.
+	 * \note Position 0 is always reserved for the asserting literal.
 	 */
 	ClauseCreator& startAsserting(ConstraintType t, const Literal& assertingLit = Literal());
 	
-	//! adds the literal p to the clause.
+	//! Adds the literal p to the clause.
 	/*!
-	 * \note p is only added if p is free or the current DL is > 0
-	 * \pre clause neither contains p nor ~p.
-	 * \pre if clause was started with startAsserting() p is false!
+	 * \note p is only added if p is free or the current DL is > 0.
+	 * \pre Clause neither contains p nor ~p.
+	 * \pre If clause was started with startAsserting() p is false!
 	 */
 	ClauseCreator& add(const Literal& p);
 
-	//! returns the literal at position i.
+	//! Returns the literal at position i.
 	/*!
 	 * \pre i < size()
 	 */
 	Literal& operator[](LitVec::size_type i)       { return literals_[i]; }
 	Literal  operator[](LitVec::size_type i) const { return literals_[i]; }
-	//! returns the literals of the clause
+	//! Returns the literals of the clause.
 	const LitVec&     lits()    const { return literals_; }
 	LitVec&           lits()          { return literals_; }
-	//! returns the current size of the clause.
+	//! Returns the current size of the clause.
 	LitVec::size_type size()    const { return literals_.size(); }
 	bool              empty()   const { return literals_.empty();}
-	//! returns the clause's type.
+	//! Returns the clause's type.
 	ConstraintType    type()    const { return extra_.type(); }
-	//! returns the current position of the second watch
+	//! Returns the current position of the second watch.
 	uint32            sw()      const { return 1; }
+	ClauseInfo        info()    const { return extra_; }
 		
-	//! Possible status values of a clause
+	//! Possible status values of a clause.
 	enum Status {
-		status_subsumed, status_sat, status_empty, status_conflicting, status_unit, status_open
+		status_open        = 0,  /**< More than one literal is free */
+		status_sat         = 1,  /**< At least one literal is true  */
+		status_unit        = 2,  /**< All but one literal false     */
+		status_asserting   = 3,  /**< Unit after backtracking to second highest dl in clause */
+		status_conflicting = 4,  /**< All literals are false        */
+		status_empty       = 12, /**< All literals are false on level 0 */
+		status_subsumed    = 17  /**< At least one literal is true on level 0 */
 	};
 
-	//! returns the current status of the clause
+	//! Returns the current status of the clause.
 	/*!
 	 * 
 	 * \return Given a clause with literals [l1,...,ln] returns
+	 *  - status_empty      : if n == 0 or for all li, (s.isFalse(li) && s.level(li) == 0)
 	 *  - status_subsumed   : if exists li, s.isTrue(li) && s.level(li) == 0
 	 *  - status_sat        : if exists li, s.isTrue(li)
-	 *  - status_empty      : if n == 0 or for all li, (s.isFalse(li) && s.level(li) == 0)
-	 *  - status_conflicting: if n != 0 and for all li, s.isFalse(li) and for some lj, s.level(lj) > 0
-	 *  - status_unit       : if s.isFree(l1) and for all i > 1, s.isFalse(li)
+	 *  - status_conflicting: if for all li, s.isFalse(li) and for some lj, s.level(lj) > 0
+	 *  - status_unit       : if s.isFree(li) and for all j != i, s.isFalse(lj)
+	 *  - status_asserting  : if exists li, s.th. for all lj != li, s.isFalse(lj) and s.level(lj) != s.level(li).
 	 *  - status_open       : else
 	 */
 	Status status() const;
 
-	//! returns the implication level of the unit-resulting literal
+	//! Returns the implication level of the unit-resulting literal.
 	/*! 
-	 * \pre  status() == status_unit
+	 * \pre  status() == status_unit || status() == status_asserting
 	 */
 	uint32 implicationLevel() const { return swLev_; }
-	//! returns the (numerical) highest DL on which the clause is conflicting
+	//! Returns the (numerical) highest DL on which the clause is conflicting.
 	/*!
 	 * \pre status() == status_conflicting || status_empty
 	 */
 	uint32 conflictLevel()    const { return fwLev_; }
 
-	//! removes duplicate lits. Marks the clause as sat if it contains p and ~p
+	//! Removes duplicate lits. Marks the clause as sat if it contains p and ~p.
 	void simplify();
 	
-	//! A type for storing the result of a clause insertion operation
+	//! A type for storing the result of a clause insertion operation.
 	/*!
 	 * The insertion operations return a tuple r s.th.
 	 *   - r.ok is true if the clause was successfully created/integrated.
@@ -187,43 +194,47 @@ public:
 		operator bool() const { return ok; }
 	};
 
-	//! adds the clause to the solver if possible
+	//! Adds the clause to the solver if possible.
 	/*!
 	 * Adds the clause only if it is not conflicting.
 	 * \note If the clause to be added is empty, end() fails and s.hasConflict() is set to true.
 	 */
 	Result end();
 
-	//! reorders lits such that the first two literals are valid watches
+	//! Reorders lits such that the first two literals are valid watches.
 	/*!
 	 * \note 
 	 *   If allFree is true, all literals in lits shall be free w.r.t
 	 *   to s. In that case, the watches are picked according to
 	 *   the strategy stored in s.
 	 */
-	static void initWatches(Solver& s, LitVec& lits, bool allFree);
+	static Status initWatches(Solver& s, LitVec& lits, bool allFree);
+
+	//! Returns the status of the given clause w.r.t s.
+	static Status status(const Solver& s, const Literal* clause_begin, const Literal* clause_end);
 
 	/*!
 	 * \name factory functions
 	 * Functions for creating and integrating clauses
 	 */
 		
-	//! Flags controlling clause creation and integration
+	//! Flags controlling clause creation and integration.
 	enum CreateFlag {
 		clause_no_add        = 1, /**< do not add clause to solver db */
 		clause_explicit      = 2, /**< force creation of explicit clause even if size <= 3 */
 		clause_not_sat       = 4, /**< integrate only if clause is not satisfied w.r.t current assignment */
 		clause_not_root_sat  = 8, /**< integrate only if clause is not satisfied w.r.t root level */
-		clause_no_release    = 16,/**< do not call release on shared literals */
-		clause_known_order   = 32,/**< assume clause is already ordered w.r.t watches */
+		clause_not_conflict  = 16,/**< integrate only if clause is not conflicting */
+		clause_no_release    = 32,/**< do not call release on shared literals */
+		clause_known_order   = 64,/**< assume clause is already ordered w.r.t watches */
 	};
 
-	//! creates a clause from the literals given in lits
+	//! Creates a clause from the literals given in lits.
 	/*!
-	 * \param s     The solver to which the clause should be added
-	 * \param lits  The literals of the clause
-	 * \param flags flag set controlling creation (see CreateFlag)
-	 * \param info  Initial information (e.g. type) for the new clause
+	 * \param s     The solver to which the clause should be added.
+	 * \param lits  The literals of the clause.
+	 * \param flags Flag set controlling creation (see CreateFlag).
+	 * \param info  Initial information (e.g. type) for the new clause.
 	 *
 	 * \pre !s.hasConflict() and s.decisionLevel() == 0 or extra.type() != Constraint_t::static_constraint
 	 * \pre lits does not contain duplicate/complementary literals
@@ -231,10 +242,6 @@ public:
 	 * \pre if lits[1] is false all lits except for lits[0] are false and no literal was assigned on a higher dl
 	 *
 	 * \note 
-	 *   The function always assumes clause_known_order, i.e. it is the caller's job
-	 *   to provide valid watches.
-	 *
-	 * \note 
 	 *   If the given clause is unit (or asserting), the unit-resulting literal is
 	 *   asserted on the (numerical) lowest level possible but the new information
 	 *   is not immediately propagated, i.e. on return queueSize() may be greater than 0.
@@ -250,20 +257,28 @@ public:
 	 *
 	 */
 	static Result create(Solver& s, LitVec& lits, uint32 flags = 0, const ClauseInfo& info = ClauseInfo());
+	/*!
+	 * \overload Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flags, const ClauseInfo& info);
+	 * \note 
+	 *   The function always assumes clause_known_order, i.e. it is the caller's job
+	 *   to guarantee the first two literals in lits are valid watches.
+	 */
+	static Result create(Solver& s, LitVec& lits, const ClauseInfo& info = ClauseInfo()) {
+		return create(s, lits, clause_known_order, info);
+	}
 	
-	//! integrates the given clause into the current search of s
+	//! Integrates the given clause into the current search of s.
 	/*!
 	 * \pre the assignment in s is not conflicting
-	 * \param s           The solver in which the clause should be integrated
-	 * \param clause      The clause to be integrated
-	 * \param modeFlags   A set of flags controlling integration (see CreateFlag)
-	 * \param e           Initial data for the local representation 
+	 * \param s           The solver in which the clause should be integrated.
+	 * \param clause      The clause to be integrated.
+	 * \param modeFlags   A set of flags controlling integration (see CreateFlag).
+	 * \param e           Initial data for the local representation.
 	 * 
 	 * \note 
-	 *   The function behaves similar to ClauseCreator::create() but it does not
-	 *   assume clause_known_order, i.e. by default it determines the correct watches
-	 *   automatically. Furthermore, the function does not add local representations for 
-	 *   implicit clauses (i.e. size <= 3) unless modeFlags contains clause_explicit. 
+	 *   The function behaves similar to ClauseCreator::create() with the exception that
+	 *   it does not add local representations for implicit clauses (i.e. size <= 3) 
+	 *   unless modeFlags contains clause_explicit. 
 	 *   In that case, an explicit representation is created. 
 	 *   Implicit representations can only be created via ClauseCreator::createClause().
 	 *
@@ -285,14 +300,15 @@ public:
 	 */
 	static Result integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags, const ClauseInfo& e);
 	/*!
-	 * \overload Result integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags)
+	 * \overload Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags)
 	 */
 	static Result integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags);
 	//@}
 private:
 	void init(ConstraintType t);
-	static inline Status status(const Solver& s, Literal* lits, uint32 flags);
 	struct Watches;
+	static inline Status status(uint32 DL, uint32 fw, uint32 sw);
+	static inline Status status(const Solver& s, const Watches& w, uint32 modeFlags);
 	static ClauseHead* newProblemClause(Solver& s, LitVec& lits, const ClauseInfo& e, uint32 flags);
 	static ClauseHead* newLearntClause(Solver& s,  LitVec& lits, const ClauseInfo& e, uint32 flags);
 	Solver*         solver_;    // solver in which new clauses are stored
@@ -300,10 +316,9 @@ private:
 	ClauseInfo      extra_;     // extra info 
 	uint32          fwLev_;     // decision level of the first watched literal
 	uint32          swLev_;     // decision level of the second watched literal
-	bool            asserting_; // true if clause was started with startAsserting
 };
 
-//! class for representing a clause in a solver
+//! Class for representing a clause in a solver.
 class Clause : public ClauseHead {
 public:
 	typedef Constraint::PropResult PropResult;
@@ -320,7 +335,7 @@ public:
 	 *  - lits does not contain any duplicate literals.
 	 *  - if lits contains literal p it must not contain ~p.
 	 *  . 
-	 * \note The clause must be destroyed using Clause::destroy
+	 * \note The clause must be destroyed using Clause::destroy.
 	 */ 
 	static ClauseHead*  newClause(Solver& s, const LitVec& lits) {
 		return newClause(s, &lits[0], static_cast<uint32>(lits.size()), ClauseInfo());
@@ -361,20 +376,20 @@ public:
 	 */ 
 	static ClauseHead*  newContractedClause(Solver& s, const LitVec& lits, const ClauseInfo& extra, LitVec::size_type tail, bool extend);
 
-	//! creates a new specialized short clause object
+	//! Creates a new specialized short clause object.
 	/*!
 	 * \pre size <= MAX_SHORT_LEN
 	 * \pre mem is either 0 or points to a memory block that was allocated via s.allocSmall();
 	 */
 	static ClauseHead*  newShortClause(Solver& s, void* mem, const Literal* lits, uint32 size, const ClauseInfo& extra);
 
-	//! Creates a new local surrogate for shared_lits to be used in the given solver
+	//! Creates a new local surrogate for shared_lits to be used in the given solver.
 	/*!
-	 * \param s The solver in which this clause will be used
-	 * \param shared_lits The shared literals of this clause
-	 * \param e Initial data for the new (local) clause
-	 * \param lits watches and cache literal for the new (local) clause
-	 * \param addRef increment ref count of shared_lits
+	 * \param s The solver in which this clause will be used.
+	 * \param shared_lits The shared literals of this clause.
+	 * \param e Initial data for the new (local) clause.
+	 * \param lits Watches and cache literal for the new (local) clause.
+	 * \param addRef Increment ref count of shared_lits.
 	 */
 	static ClauseHead* newShared(Solver& s, SharedLiterals* shared_lits, const ClauseInfo& e, const Literal* lits, bool addRef = true);
 
@@ -383,7 +398,7 @@ public:
 	Constraint* cloneAttach(Solver& other);
 
 	/*!
-	 * for a clause [x y p] the reason for p is ~x and ~y. 
+	 * For a clause [x y p] the reason for p is ~x and ~y. 
 	 * \pre *this previously asserted p
 	 * \note if the clause is a learnt clause, calling reason increases
 	 * the clause's activity.
@@ -394,18 +409,18 @@ public:
 
 	bool isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN);
 
-	//! returns true if clause is SAT.
+	//! Returns true if clause is SAT.
 	/*!
-	 * removes from the clause all literals that are false.
+	 * Removes from the clause all literals that are false.
 	 */
 	bool simplify(Solver& s, bool = false);
 
-	//! destroys the clause and frees its memory.
+	//! Destroys the clause and frees its memory.
 	void destroy(Solver* s = 0, bool detach = false);
 	
 	// LearntConstraint interface
 
-	//! returns true if the clause is currently satisfied.
+	//! Returns true if the clause is currently satisfied.
 	/*!
 	 * If the clause is currently not satisfied all free literals of
 	 * the clause are copied to freeLits.
@@ -417,28 +432,25 @@ public:
 	void     detach(Solver&);
 	uint32   size()                 const;
 	void     toLits(LitVec& out)    const;
-	bool     contracted()           const { return tail_.data.contracted(); }
-	bool     isSmall()              const { return tail_.data.isSmall(); }
+	bool     contracted()           const { return data_.local.contracted(); }
+	bool     isSmall()              const { return data_.local.isSmall(); }
 private:
 	Clause(Solver& s, const ClauseInfo& extra, const Literal* lits, uint32 size, uint32 tail, bool extend);
 	Clause(Solver& s, const Clause& other);
+	typedef std::pair<Literal*, Literal*> LitRange;
 	static void* alloc(Solver& s, uint32 lits);
 	void         undoLevel(Solver& s);
 	bool         updateWatch(Solver& s, uint32 pos);
-	typedef std::pair<Literal*, Literal*> LitRange;
-	Literal*       longEnd()   { return head_+tail_.data.size(); }
-	Literal*       smallEnd()  { return smallLits() + !isSentinel(*smallLits()) + !isSentinel(smallLits()[1]); }
-	Literal*       smallLits() { return reinterpret_cast<Literal*>(&tail_); }
-	const Literal* smallLits() const { return reinterpret_cast<const Literal*>(&tail_); }
 	Literal*     removeFromTail(Solver& s, Literal* it, Literal* end);
+	Literal*     longEnd()   { return head_+data_.local.size(); }
 	LitRange     tail() {
-		return !isSmall()
-			? LitRange(head_+ClauseHead::HEAD_LITS, longEnd())
-			: LitRange(smallLits(), smallEnd());
+		if (!isSmall()) { return LitRange(head_+ClauseHead::HEAD_LITS, longEnd()); }
+		uint32 ts = (data_.lits[0] != 2) + (data_.lits[1] != 2);
+		return LitRange((Literal*)data_.lits, (Literal*)(data_.lits + ts));
 	}
 };
 
-//! Constraint for Loop-Formulas
+//! Constraint for Loop-Formulas.
 /*!
  * \ingroup constraint
  * Special purpose constraint for loop formulas of the form: R -> ~a1, ~a2, ..., ~an
@@ -473,27 +485,27 @@ public:
 	 * Creates a new loop-formula for numAtoms atoms sharing the literals contained in bodyLits.
 	 * 
 	 * \param s Solver in which the new loop-formula is to be used.
-	 * \param bodyLits Pointer to an array of numBodies body-literals
-	 * \param numBodies Number of body-literals in bodyLits
+	 * \param bodyLits Pointer to an array of numBodies body-literals.
+	 * \param numBodies Number of body-literals in bodyLits.
 	 * \param numAtoms Number of atoms in the loop-formula.
 	 *
 	 * \pre all body-literals are currently false.
 	 */ 
-	static LoopFormula* newLoopFormula(Solver& s, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, uint32 numAtoms) {
+	static LoopFormula* newLoopFormula(Solver& s, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, uint32 numAtoms, const ClauseInfo& extra = ClauseInfo()) {
 		void* mem = ::operator new( sizeof(LoopFormula) + (numBodies+numAtoms+3) * sizeof(Literal) );
-		return new (mem) LoopFormula(s, numBodies+numAtoms, bodyLits, numBodies, bodyToWatch);
+		return new (mem) LoopFormula(s, numBodies+numAtoms, bodyLits, numBodies, bodyToWatch, extra);
 	}
 
-	//! Adds an atom to the loop-formula
+	//! Adds an atom to the loop-formula.
 	/*!
 	 * \pre the loop-formula currently contains fewer than numAtoms atoms
 	 */
 	void addAtom(Literal atom, Solver& s);
 	
-	//! notifies the installed heuristic about the new constraint.
+	//! Notifies the installed heuristic about the new constraint.
 	void updateHeuristic(Solver& s);
 	
-	//! returns the size of the loop-formula.
+	//! Returns the size of the loop-formula.
 	uint32 size() const;
 	
 	// Constraint interface
@@ -507,7 +519,7 @@ public:
 	// LearntConstraint interface
 	bool locked(const Solver& s) const;
 	
-	//! returns true if the loop-formula is satisfied
+	//! Returns true if the loop-formula is satisfied.
 	/*!
 	 * A loop-formula is satisfied if one body is true or all contained atoms are false.
 	 * If the loop-formula is currently not satisfied isSatisfied copies all free literals
@@ -515,31 +527,32 @@ public:
 	 */
 	bool isSatisfied(const Solver& s, LitVec& freeLits);
 	
-	//! returns the loop-formula's activity
+	//! Returns the loop-formula's activity.
 	/*!
 	 * The activity of a loop-formula is increased, whenever reason() is called.
 	 */
-	Activity activity() const { return makeActivity(activity_, ClauseHead::MAX_LBD); }
+	Activity activity() const { return makeActivity(activity_, lbd_); }
 	
-	//! halves the loop-formula's activity
+	//! Halves the loop-formula's activity.
 	void decreaseActivity() { activity_ >>= 1; }
 	
-	//! returns Constraint_t::learnt_loop
+	//! Returns Constraint_t::learnt_loop.
 	ConstraintType type() const { return Constraint_t::learnt_loop; }
 private:
-	LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch);
+	LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, const ClauseInfo& info);
 	bool watchable(const Solver& s, uint32 idx);
 	bool isTrue(const Solver& s, uint32 idx);
-	uint32  activity_; // activity of this loop formula
-	uint32  end_;      // position of second sentinel
-	uint32  size_;     // size of lits_
-	uint32  other_;    // stores the position of a literal that was recently true
-	Literal lits_[0];  // S B1...Bm ai S a1...an
+	uint32  activity_:27; // activity of this loop formula
+	uint32  lbd_     : 5; // lbd of active clause
+	uint32  end_;         // position of second sentinel
+	uint32  size_;        // size of lits_
+	uint32  other_;       // stores the position of a literal that was recently true
+	Literal lits_[0];     // S B1...Bm ai S a1...an
 };
 
 namespace mt {
 
-//! Stores the local part of a shared clause
+//! Stores the local part of a shared clause.
 /*!
  * The local part of a shared clause consists of a
  * clause head and and a pointer to the shared literals.
@@ -550,13 +563,13 @@ namespace mt {
  */
 class SharedLitsClause : public ClauseHead {
 public:
-	//! Creates a new SharedLitsClause to be used in the given solver
+	//! Creates a new SharedLitsClause to be used in the given solver.
 	/*!
-	 * \param s The solver in which this clause will be used
-	 * \param shared_lits The shared literals of this clause
-	 * \param e Initial data for the new (local) clause
-	 * \param lits watches and cache literal for the new (local) clause
-	 * \param addRef increment ref count of shared_lits
+	 * \param s The solver in which this clause will be used.
+	 * \param shared_lits The shared literals of this clause.
+	 * \param e Initial data for the new (local) clause.
+	 * \param lits Watches and cache literal for the new (local) clause.
+	 * \param addRef Increment ref count of shared_lits.
 	 */
 	static ClauseHead* newClause(Solver& s, SharedLiterals* shared_lits, const ClauseInfo& e, const Literal* lits, bool addRef = true);
 	
