@@ -50,7 +50,7 @@ class  SatPreprocessor;
  */
 class Enumerator {
 public:
-	enum ProgressType    { progress_restart, progress_reduce, progress_model };
+	enum ProgressType    { progress_restart, progress_reduce, progress_model, num_progress_types };
 	//! Interface used by the enumerator to report restart events
 	class ProgressReport {
 	public:
