@@ -310,7 +310,7 @@ public:
 	/*!
 	 * \param considerLoops If true, activity counters are increased for variables that occure in loop-formulas
 	 */
-	explicit ClaspVsids(bool considerLoops = false);
+	explicit ClaspVsids(double d = 0.95, bool considerLoops = false);
 	void startInit(const Solver& s);
 	void endInit(Solver&);
 	void reinit(bool b) { reinit_ = b; }
@@ -357,11 +357,12 @@ private:
 		const Scores& sc_;
 	};
 	typedef bk_lib::indexed_priority_queue<GreaterActivity> VarOrder;
-	Scores    score_;
-	VarOrder  vars_;
-	double    inc_;
-	bool      scoreLoops_;
-	bool      reinit_;
+	Scores       score_;
+	VarOrder     vars_;
+	const double decay_;
+	double       inc_;
+	bool         scoreLoops_;
+	bool         reinit_;
 };
 
 }
