@@ -31,7 +31,7 @@
 
 namespace Clasp {
 class MinimizeConstraint;
-//! Supported minimization modes
+//! Supported minimization modes.
 /*! 
  * Defines the possible minimization modes used during solving.
  * If optimize is used, an optimal solution is one that is
@@ -47,7 +47,7 @@ struct MinimizeMode_t {
 };
 typedef MinimizeMode_t::Mode MinimizeMode;
 
-//! A type holding data (possibly) shared between a set of minimize constraints
+//! A type holding data (possibly) shared between a set of minimize constraints.
 /*!
  * \ingroup shared
  */
@@ -64,11 +64,11 @@ struct SharedMinimizeData {
 		uint32   next  :  1; /**< does this weight belong to a sparse vector of weights? */
 		weight_t weight;     /**< the weight at this level */
 	};
-	//! A type for holding sparse vectors of level weights of a multi-level constraint
+	//! A type for holding sparse vectors of level weights of a multi-level constraint.
 	typedef PodVector<LevelWeight>::type WeightVec;
-	//! A dense vector of weights
+	//! A dense vector of weights.
 	typedef PodVector<wsum_t>::type      SumVec;
-	//! A type for holding an optimum and the information for how to proceed
+	//! A type for holding an optimum and the information for how to proceed.
 	struct Optimum {
 		typedef std::pair<uint32, wsum_t> Step;
 		typedef std::atomic<uint32>       Atomic;
@@ -76,34 +76,26 @@ struct SharedMinimizeData {
 		SumVec opt;     /**< current optimum / one value per level      */
 		Step   step;    /**< (opt - step) gives the next bound to check */
 	};
-	explicit SharedMinimizeData(const SumVec& initSum, const SumVec& initOpt, MinimizeMode m = MinimizeMode_t::optimize);
+	explicit SharedMinimizeData(const SumVec& lhsSum, const SumVec& lhsAdjust, const SumVec& rhsSum, MinimizeMode m = MinimizeMode_t::optimize);
 	~SharedMinimizeData();
 	
 	void destroy() const;
-	
-	//! sets the enumeration mode used during solving
-	/*!
-	 * \note if m is MinimizeMode::enumerate, the caller must explicitly
-	 * set an optimum. Otherwise, *all* solutions are considered optimal.
-	 */
-	void setMode(MinimizeMode m, int hierarchical);
-	
-	//! number of minimize statements contained in this constraint
+	//! Number of minimize statements contained in this constraint.
 	uint32         numRules()      const { return static_cast<uint32>(sum_.size()); }
 	uint32         maxLevel()      const { return numRules()-1; }
-	//! returns the active minimization mode
+	//! Returns the active minimization mode.
 	MinimizeMode   mode()          const { return mode_; }
-	//! hierarchical optimization enabled?
+	//! Hierarchical optimization enabled?
 	bool           hierarchical()  const { return hierarch_ > 0; }
-	//! returns the initial sum
-	const SumVec&  sum()           const { return sum_; }
-	//! returns the current optimum
+	//! Returns the initial sum of level lev.
+	wsum_t         sum(uint32 lev) const { return sum_[lev] + (adjust_ ? adjust_[lev] : 0); }
+	//! Returns the current optimum.
 	const Optimum* optimum()       const { return opt_; }
-	//! returns the level of the literal with the given idx
+	//! Returns the level of the literal with the given idx.
 	uint32         getLevel(uint32 litIdx) const {
 		return numRules() == 1 ? 0 : weights[lits[litIdx].second].level;
 	}
-	//! set optOut to (opt - (applyStep * opt->step))
+	//! Sets optOut to (opt - (applyStep * opt->step)).
 	bool           convert(const Optimum* opt, bool applyStep, wsum_t* optOut) const;
 	
 	/*!
@@ -112,13 +104,27 @@ struct SharedMinimizeData {
 	 */
 	//@{
 	
-	//! makes opt the new optimum
+	//! Sets the enumeration mode used during solving.
+	/*!
+	 * \note If m is MinimizeMode::enumerate, the caller must explicitly
+	 * set an optimum. Otherwise, *all* solutions are considered optimal.
+	 */
+	void setMode(MinimizeMode m, int hierarchical);
+	
+	//! Attaches a new minimize constraint to this data object.
+	/*!
+	 * If makeOwned is true, the returned minimize constraint
+	 * owns the shared data object.
+	 */
+	MinimizeConstraint* attach(Solver& s, bool makeOwned = false);
+	
+	//! Makes opt the new optimum.
 	/*!
 	 * \pre opt is a pointer to an array of size numRules()
 	 */
 	const Optimum* setOptimum(const wsum_t* opt);
 	
-	//! sets the next level to optimize
+	//! Sets the next level to optimize.
 	/*!
 	 * \return 
 	 *   - true if there is a next level to optimize
@@ -128,10 +134,11 @@ struct SharedMinimizeData {
 	//@}
 private:
 	typedef std::atomic<Optimum*> OptPtr;
-	SumVec       sum_;     // initial sum
+	SumVec       sum_;     // initial lhs sum
 	Optimum      opts_[2]; // buffers for update via "double buffering"
 	OptPtr       opt_;     // active optimum
 	Optimum*     next_;    // used during update of optimum
+	wsum_t*      adjust_;  // initial lhs adjustments
 	wsum_t       low_;     // current lower bound
 	uint32       hierarch_;// hierarchical optimization?
 	MinimizeMode mode_;    // how to compare assignments?
@@ -145,27 +152,27 @@ private:
 	bool optimizeNext(Optimum* opt);
 };
 
-//! Helper class for creating minimize constraints
+//! Helper class for creating minimize constraints.
 class MinimizeBuilder {
 public:
 	typedef SharedMinimizeData SharedData;
 	MinimizeBuilder();
 	~MinimizeBuilder();
 	
-	bool                hasRules() const { return !initial_.empty(); }
-	uint32              numRules() const { return (uint32)initial_.size(); }
+	bool                hasRules() const { return !sum_.empty(); }
+	uint32              numRules() const { return (uint32)sum_.size(); }
 
-	//! adds a minimize statement
+	//! Adds a minimize statement.
 	/*!
 	 * \param lits the literals of the minimize statement
 	 * \param adjustSum the initial sum of the minimize statement
 	 */
 	MinimizeBuilder&    addRule(const WeightLitVec& lits, wsum_t adjustSum = 0);
 
-	//! sets the initial optimum of level lev to opt
+	//! Sets the initial optimum of level lev to opt
 	MinimizeBuilder&    setOptimum(uint32 lev, wsum_t opt);
 	
-	//! creates a new data object from previously added minimize statements
+	//! Creates a new data object from previously added minimize statements.
 	/*!
 	 * The function creates a new minimize data object from 
 	 * the previously added minimize statements. The returned
@@ -176,15 +183,11 @@ public:
 	 *  or 0 if minimize statements are initially inconsistent!
 	 */
 	SharedData*         build(SharedContext& ctx, Literal ma = posLit(0));
-
-	//! attaches a minimize constraint to a previously built data object 
-	static MinimizeConstraint* attach(Solver& s, SharedData* data);
-
 	/*!
 	 * Effect:
 	 *  if (SharedData* d = build(ctx)) {
 	 *    d->setMode(m);
-	 *    return attach(*ctx.master(), d);
+	 *    return d->attach(*ctx.master(), true);
 	 *  }
 	 *  return 0;
 	 */
@@ -209,17 +212,18 @@ private:
 	};
 	bool     prepare(SharedContext& ctx);
 	void     addAssumption(Literal ma);
-	void     adjustSum(LitRep l);
+	void     addTo(LitRep l, SumVec& vec);
 	void     mergeReduceWeight(LitRep& x, LitRep& by);
 	weight_t addFlattened(SharedData::WeightVec& x, const Weight& w);
 	bool     eqWeight(const SharedData::LevelWeight* lhs, const Weight& rhs);
-	LitRepVec lits_;
-	SumVec    initial_;
-	SumVec    opts_;
-	bool      ready_;
+	LitRepVec lits_;  // all literals
+	SumVec    sum_;   // lhs sum
+	SumVec    adjust_;// lhs adjustments
+	SumVec    rhs_;   // right hand side
+	bool      ready_; // prepare was called
 };
 
-//! Implements a (meta-)constraint for supporting Smodels-like minimize statements
+//! Implements a (meta-)constraint for supporting Smodels-like minimize statements.
 /*!
  * \ingroup constraint
  * A solver contains at most one minimize constraint, but a minimize constraint
@@ -243,36 +247,39 @@ private:
  */
 class MinimizeConstraint : public Constraint {
 public:
-	friend class MinimizeBuilder;
-	typedef MinimizeBuilder::SharedData SharedData;
-	typedef SharedData::Optimum         Optimum;
-	typedef PodVector<wsum_t>::type SumVec;
+	friend struct SharedMinimizeData;
+	typedef SharedMinimizeData       SharedData;
+	typedef SharedData::Optimum      Optimum;
+	typedef PodVector<wsum_t>::type  SumVec;
 
+	//! Attaches this object to the given solver.
 	bool attach(Solver& s);
 
-	//! number of minimize statements contained in this constraint
+	//! Number of minimize statements contained in this constraint.
 	uint32 numRules() const { return shared()->numRules(); }
 	
-	//! resets the local optimum to the one stored in the shared data object
+	//! Resets the local optimum to the one stored in the shared data object.
 	bool restoreOptimum();
 
-	//! sets the local sum as the local and global optimum.
+	//! Sets the local sum as the global optimum.
 	/*!
 	 * commitCurrent shall be called whenever the solver finds a model.
-	 * The current local sum is recorded as new optimum both in *this
-	 * and the shared data object. 
+	 * The current local sum is recorded as new optimum in the shared data object
+	 * and in *this if optimization (not enumeration) is active.
+	 *
 	 * \pre s currently stores a valid model and sum() < shared()->optimum()
-	 * \post opt() == shared()->optimum()
 	 *
 	 * \return The earliest decision level on which the search for the next
 	 *          tentative optimum could continue or UINT32_MAX if the 
 	 *          search space is exhausted.
 	 * \note
-	 *   integrateNext() must be called in order to continue enumeration.
+	 *   If optimization is active, integrateNext() must be called in order to 
+	 *   continue enumeration. If enumeration is active, it is not necessary to
+	 *   call integrateNext().
 	 */
 	uint32 commitCurrent(const Solver& s);
 
-	//! tries to integrate the next tentative optimum into this constraint
+	//! Tries to integrate the next tentative optimum into this constraint.
 	/*!
 	 * Starting from the current optimum stored in the shared data object,
 	 * the function tries to integrate the next candidate optimum into
@@ -281,7 +288,7 @@ public:
 	 * \pre !s.hasConflict() and the tag literal (if any) is true
 	 *
 	 * \return The function returns true if integration succeeded. Otherwise
-	 * false is returned and s.hasConflict() is true
+	 * false is returned and s.hasConflict() is true.
 	 *
 	 * \note If integrateNext() failed, the optimum of this constraint
 	 *       is *not* changed. The caller has to resolve the conflict first
@@ -289,12 +296,14 @@ public:
 	 *
 	 * \note The caller has to call s.propagate() to propagate any new information
 	 *       from the new bound.
+	 *
+	 * \note If mode() is MinimizeMode_t::enumerate, integrateNext() does nothing.
 	 */
 	bool integrateNext(Solver& s);
 	
 	const SharedData* shared() const { return shared_; }
 
-	//! tries to assign as many literals from the last model to false
+	//! Tries to assign as many literals from the last model to false.
 	bool modelHeuristic(Solver& s);
 
 	// base interface
@@ -308,8 +317,9 @@ public:
 	// FOR TESTING ONLY!
 	wsum_t sum(uint32 i) const { return sum_[i]; }
 protected:
-	MinimizeConstraint(SharedData* d);
+	MinimizeConstraint(SharedData* d, bool owner);
 	~MinimizeConstraint();
+	enum PropMode { propagate_new_sum, propagate_new_opt };
 	union UndoInfo {
 		UndoInfo() : rep(0) {}
 		struct {
@@ -324,7 +334,6 @@ protected:
 	typedef const WeightLiteral*         Iter;
 	uint32    lastUndoLevel(const Solver& s) const;
 	void      pushUndo(Solver& s, uint32 litIdx);
-	enum PropMode { propagate_new_sum, propagate_new_opt };
 	bool      litSeen(uint32 i) const { return undo_[i].data.idxSeen != 0; }
 	uint32    updateOpt(const Solver* s, bool applyStep, const Optimum* opt);
 	bool      propagateImpl(Solver& s, PropMode m);
