@@ -35,14 +35,14 @@
 
 /*!
  * \file 
- * Defines classes controlling multi-threaded parallel solving
+ * Defines classes controlling multi-threaded parallel solving.
  * 
  */
 namespace Clasp { namespace mt {
 
 class ParallelHandler;
 
-//! A parallel algorithm for multi-threaded solving with and without search-space splitting
+//! A parallel algorithm for multi-threaded solving with and without search-space splitting.
 /*!
  * The class adapts clasp's basic solve algorithm
  * to a parallel solve algorithm that solves
@@ -56,40 +56,40 @@ public:
 	~ParallelSolve();
 	// "setup" interface
 	
-	//! returns number of threads that can run concurrently on the current hardware
+	//! Returns number of threads that can run concurrently on the current hardware.
 	static uint32  hardwareThreads();
-	//! used to signal an error in a client thread
+	//! Used to signal an error in a client thread.
 	static const   uint32 invalidId = UINT32_MAX;
 	
-	//! initializes this object for solving with at most maxThreads
+	//! Initializes this object for solving with at most maxThreads.
 	/*!
 	 * \pre numThreads >= 1
 	 * \pre ctx.shareCount() >= maxThreads
 	 */
 	SharedContext* init(int maxThreads, SharedContext& ctx);
-	//! enables/disables search-space splitting
+	//! Enables/disables search-space splitting.
 	void   setForceGP(bool gp);
-	//! configures nogood distribution
+	//! Configures nogood distribution.
 	void   setIntegrate(uint32 grace, uint8 filter);
-	//! configures "global" restarts (only used if search-space splitting is active)
+	//! Configures "global" restarts (only used if search-space splitting is active).
 	void   setRestarts(uint32 maxR, const ScheduleStrategy& rs);
-	//! attaches a new thread to s and runs it in this algorithm
+	//! Attaches a new thread to s and runs it in this algorithm.
 	/*!
-	 * shall be called once for each solver except the master
+	 * Shall be called once for each solver except the master.
 	 * \pre s.id() != ParallelSolve::invalidId
 	 */
 	void   addThread(Solver& s, SolveParams& p);
 	
 	// solve interface 
 	
-	//! returns the number of active threads
+	//! Returns the number of active threads.
 	uint32 numThreads()            const;
 	bool   integrateUseHeuristic() const { return intHeuristic_; }
 	uint32 integrateGrace()        const { return intGrace_; }
 	uint32 integrateFlags()        const { return intFlags_; }
-	//! terminates current solving process and all client threads
+	//! Terminates current solving process and all client threads.
 	bool   terminate();
-	//! requests a global restart
+	//! Requests a global restart.
 	void   requestRestart();
 	bool   handleMessages(Solver& s);
 	void   pushWork(LitVec& gp);
@@ -131,7 +131,7 @@ private:
 };
 
 
-//! A per-solver (i.e. thread) class that implements message handling and knowledge integration
+//! A per-solver (i.e. thread) class that implements message handling and knowledge integration.
 /*!
  * The class adds itself as a post propagator to the given solver. Each time
  * propagateFixpoint() is called (i.e. on each new decision level), it checks
@@ -140,24 +140,24 @@ private:
  */
 class ParallelHandler : public PostPropagator {
 public:
-	//! creates a new parallel handler to be used in the given solve group
+	//! Creates a new parallel handler to be used in the given solve group.
 	/*!
-	 * \param ctrl The object controlling the parallel solve operation
+	 * \param ctrl The object controlling the parallel solve operation.
 	 */
 	explicit ParallelHandler(ParallelSolve& ctrl);
 	~ParallelHandler();
 
-	//! attaches this parallel handler to the given solver
+	//! Attaches this parallel handler to the given solver.
 	/*!
-	 * \param s The solver in which this object is used
-	 * \param p The solving parameters under which the solver operates
+	 * \param s The solver in which this object is used.
+	 * \param p The solving parameters under which the solver operates.
 	 */
 	void attach(Solver& s, const SolveParams& p);
 
-	//! detaches this object from its solver
+	//! Detaches this object from its solver.
 	void detach();
 
-	//! executes F in a new thread
+	//! Executes F in a new thread.
 	template <class F>
 	void run(F f, Solver& s, SolveParams& p) {
 		assert(!joinable() && solver_ == 0);
@@ -168,38 +168,40 @@ public:
 	void setError(int e) { error_ = e; }
 	int  error() const   { return error_; }
 	
-	//! true if *this has an associated thread of execution, false otherwise. 
+	//! True if *this has an associated thread of execution, false otherwise.
 	bool joinable() const { return thread_.joinable(); }
-	//! waits for the thread of execution associated with *this to finish.
+	//! Waits for the thread of execution associated with *this to finish.
 	/*!
-	 * \note the function is a noop of !joinable()
+	 * \note The function is a noop of !joinable().
 	 */
 	int join() { if (joinable()) { thread_.join(); } return error_; }
 	
 	// overridden methods
 	
-	//! returns a priority suited for a post propagators that is non-deterministic
+	//! Returns a priority suited for a post propagators that is non-deterministic.
 	uint32 priority() const { return priority_general + 100; }
 
-	//! integrates new information
+	//! Integrates new information.
 	bool propagateFixpoint(Solver& s);
 	bool propagate(Solver& s) { return ParallelHandler::propagateFixpoint(s); }
 	
-	//! checks whether new information has invalidated current model
+	//! Checks whether new information has invalidated current model.
 	bool isModel(Solver& s);
 
 	// own interface
 	
 	// TODO: make functions virtual once necessary 
 	
-	//! returns true if handler's guiding path is disjoint from all others
+	//! Returns true if handler's guiding path is disjoint from all others.
 	bool disjointPath() const { return gp_.split; }
+	//! Returns true if handler has a guiding path.
+	bool hasPath()      const { return gp_.impl != UINT32_MAX; }
 	
-	//! called before solver starts to solve given guiding path
+	//! Called before solver starts to solve given guiding path.
 	/*!
-	 * \param gp      the new guiding path
-	 * \param restart request restart after restart number of conflicts
-	 * \param isSplit true if gp resulted from a split
+	 * \param gp      The new guiding path.
+	 * \param restart Request restart after restart number of conflicts.
+	 * \param isSplit True if gp resulted from a split.
 	 */
 	void prepareForGP(const LitVec& gp, uint64 restart, bool isSplit);
 
@@ -211,22 +213,22 @@ public:
 	 */
 	//@{
 	
-	//! algorithm is about to terminate
+	//! Algorithm is about to terminate.
 	/*!
-	 * removes this object from the solver's list of post propagators
+	 * Removes this object from the solver's list of post propagators.
 	 */
 	void handleTerminateMessage();
 
-	//! request for split
+	//! Request for split.
 	/*!
 	 * Splits off a new guiding path and adds it to the control object.
-	 * \pre the guiding path of this object is "splittable"
+	 * \pre The guiding path of this object is "splittable"
 	 */
 	void handleSplitMessage();
 
-	//! request for (global) restart
+	//! Request for (global) restart.
 	/*!
-	 * \return true if restart is valid, else false
+	 * \return true if restart is valid, else false.
 	 */
 	bool handleRestartMessage();
 
