@@ -33,6 +33,7 @@
 #include <map>
 #include <algorithm>
 #include <iosfwd>
+#include <stdexcept>
 
 namespace Clasp {
 
@@ -168,12 +169,16 @@ struct PreproStats {
 	}*     trStats;   /**< Rule translation statistics (optional) */
 };
 
+//! Exception type for signaling an invalid incremental program update.
+class RedefinitionError : public std::logic_error {
+public:
+	explicit RedefinitionError(const std::string& m) : std::logic_error(m) {}
+};
 
 //! Interface for defining a logic program.
 /*!
  * Use this class to specify a logic program. Once the program is completly defined,
- * call endProgram to load the logic program into a Solver-object.
- * 
+ * call endProgram() to load the logic program into a SharedContext object.
  */
 class ProgramBuilder {
 public:
@@ -210,88 +215,179 @@ public:
 		bool             normalize;/**< Canonically order program */
 	};
 
-	//! Starts the definition of a logic program
 	/*!
-	 * This function shall be called exactly once before any rules or atoms are added.
-	 * \param ctx    The context object in which the program builder should store the preprocessed problem
-	 * \param eqOpts Options for the eq-preprocessor
-	 * \note If dg is not 0 it shall point to a heap-allocated object. The ownership is transferred!
+	 * \name Setup and step control functions
+	 */
+	//@{
+
+	//! Starts the definition of a logic program. 
+	/*!
+	 * This function shall be called exactly once before a new program is defined.
+	 * It discards any previously added program.
+	 *
+	 * \param ctx    The context object in which the program builder should store the preprocessed problem.
+	 * \param eqOpts Options for the eq-preprocessor.
 	 */
 	ProgramBuilder& startProgram(SharedContext& ctx, const EqOptions& eqOpts = EqOptions());
 
-	//! Unfreezes a program and starts the next step if program is defined incrementally
-	/*!
-	 * If a program is to be defined incrementally, this function must be called
-	 * exactly once for each step before any new rules or atoms are added.
-	 * \note Program update only works correctly under the following assumptions:
-	 *  - Atoms introduced in step i are either:
-	 *    - solely defined in step i OR,
-	 *    - marked as frozen in step i and solely defined in step i+k OR,
-	 *    - forced to false by a acompute statement in step 0
-	 *  - Atom ids are monotonically increasing, i.e. atoms defined in step i have lower ids than those defined in step i+k.
-	 *  - Rule bodies are either empty or contain at least one atom defined in the current step
-	 */
-	ProgramBuilder& updateProgram();
-
-	//! disposes (parts of) the internal representation of the logic program.
-	/*!
-	 * \param forceFullDispose If set to true, the whole program is disposed. Otherwise,
-	 * if updateProgram() was called, only the last incremental step is disposed.
-	 */
-	void disposeProgram(bool forceFullDispose);
-
 	//! Sets the mode for handling extended rules.
 	/*!
 	 * The default mode is to handle all extended rules natively.
 	 */
 	void setExtendedRuleMode(ExtendedRuleMode m) { eqOpts_.ext(m); }
 
-	//! creates a new atom and returns the new atom's id.
+	//! Unfreezes a currently frozen program and starts an incremental step.
 	/*!
-	 * \pre The program is not frozen
+	 * \pre The program is either frozen or at step 0.
+	 *
+	 * If a program is to be defined incrementally, this function must be called
+	 * exactly once for each step before any new rules or atoms are added.
+	 * \note Program update only works correctly under the following assumptions:
+	 *  - Atoms introduced in step i are either:
+	 *    - solely defined in step i OR,
+	 *    - marked as frozen in step i and solely defined in step i+k OR,
+	 *    - forced to false by a acompute statement in step 0
+	 *
+	 * \post The program is no longer frozen and calling program mutating functions is valid again. 
+	 * \throws std::logic_error precondition is violated.
+	 */
+	ProgramBuilder& updateProgram();
+
+	//! Finishes the definition of the logic program (or its current increment).
+	/*!
+	 * Applies program mutating operations issued in the current step and transforms 
+	 * the new program into the solver's internal representation. 
+	 *
+	 * \return false if the program is initially conflicting, true otherwise.
+	 *
+	 * \post
+	 *  - If true is returned, the program is considered to be "frozen" and calling 
+	 *    program mutating functions is invalid until the next call to updateProgram().
+	 *  - If false is returned, the state of the object is undefined and startProgram() 
+	 *    and disposeProgram() are the only remaining valid operations. 
+	 *  . 
+	 */
+	bool endProgram();
+
+	//! Adds all minimize statements contained in the program to m.
+	/*!
+	 * \pre The program is frozen.
+	 */
+	void addMinimize(MinimizeBuilder& m);
+
+	//! Writes the (possibly simplified) program in lparse-format to the given stream.
+	void writeProgram(std::ostream& os);
+	
+	//! Disposes (parts of) the internal representation of the logic program.
+	/*!
+	 * \param forceFullDispose If set to true, the whole program is disposed. Otherwise,
+	 *  only the rules (of the current step) are disposed but atoms and any incremental
+	 *  control data remain.
+	 */
+	void disposeProgram(bool forceFullDispose);
+
+	//! Clones the program and adds it to the given ctx.
+	/*
+	 * \pre The program is currently frozen.
+	 */
+	bool cloneProgram(SharedContext& ctx);
+
+	//@}
+
+	/*!
+	 * \name Program mutating functions
+	 * 
+	 * Functions in this group shall only be called if the program is currently not 
+	 * frozen. That is, only between the call to startProgram() (resp. updateProgram() if in 
+	 * incremental setting) and endProgram(). A std::logic_error is raised if this precondition is violated. 
+	 *
+	 */
+	//@{
+
+	//! Adds a new atom to the program.
+	/*!
+	 * \return The new atom's id. 
 	 */
 	Var newAtom();
-	//! returns the number of atoms in the current logic program
-	uint32 numAtoms() const  { return (uint32)atoms_.size()-1; }
-	//! returns the number of bodies in the current (slice of the) logic program
-	uint32 numBodies() const { return (uint32)bodies_.size(); }
 
-	//! Sets the name of the Atom with the given id.
+	//! Sets the name of the given atom and adds it to the program's symbol table.
 	/*!
+	 * \pre 
+	 *   - The atom is either not yet known or was added in the current step (atomId >= startAtom()).
+	 *   - The atom was not yet added to the symbol table, i.e. 
+	 *     setAtomName() is called at most once for an atom.
+	 *   . 
 	 * \param atomId The id of the atom for which a name should be set
 	 * \param name The new name of the atom with the given id.
-	 * \pre name != 0 && atomId > 0 && atomId < varMax
-	 * \note if atomId is not yet known, an atom with the given id is implicitly created.
-	 * \pre shall be called at most once per atom
+	 * \note If atomId is not yet known, an atom with the given id is implicitly created. 
+	 *
+	 * \throws RedefinitionError precondition is violated. 
+	 * \throws std::logic_error  program is frozen.
 	 */
 	ProgramBuilder& setAtomName(Var atomId, const char* name);
 
-	//! forces the atom with the given atom id to true/false.
+	//! Forces the atom's truth-value to value. 
 	/*!
+	 * \pre The atom is either not yet known, false, or an atom from the current step.
 	 * \param atomId Id of the Atom for which a truth-value should be set.
 	 * \param pos If true, atom is set to true (forced to be in every answer set). Otherwise
-	 * atom is set to false (not part of any answer set).
+	 *            atom is set to false (not part of any answer set).
+	 * \note If atomId is not yet known, an atom with the given id is implicitly created. 
 	 */
-	ProgramBuilder& setCompute(Var atomId, bool pos);
+	ProgramBuilder& setCompute(Var atomId, bool value);
 
-	//! Marks the atom with the given atom id as frozen.
+	//! Protects an otherwise undefined atom from preprocessing. 
 	/*!
-	 * \note An atom a stays frozen until unfreeze(a) is called.
-	 * \note As long as an atom is frozen it shall not be defined.
+	 * If the atom is defined in this or a previous step, the operation has no effect. 
+	 * \note If atomId is not yet known, an atom with the given id is implicitly created. 
 	 */
 	ProgramBuilder& freeze(Var atomId);
 
-	//! Unfreezes a previously frozen atom.
+	//! Removes any protection from the given atom. 
 	/*!
-	 * \pre atomId must refer to an atom that is currently frozen.
+	 * If the atom is defined in this or a previous step, the operation has no effect. 
+	 * \note
+	 *   - The effect is logically equivalent to adding a rule atomId :- false. 
+	 *   - A call to unfreeze() always overwrites a call to freeze() even if the 
+	 *     latter comes after the former
+	 *   . 
 	 */
 	ProgramBuilder& unfreeze(Var atomId);
+
+	//! Adds the given rule to the program.
+	/*!
+	 * \pre The head of the rule does not contain an atom defined in a 
+	 *      previous incremental step. 
+	 *
+	 * Simplifies the given rule and adds it to the program if it
+	 * is neither tautological (e.g. a :- a) nor contradictory (e.g. a :- b, not b). 
+	 * Atoms in the simplified rule that are not yet known are implicitly created. 
+	 *
+	 * \throws RedefinitionError if the precondition is violated. 
+	 * \note If the head of the simplified rule mentions an atom from a previous step,
+	 *       that atom shall either be frozen or false. In the former case, 
+	 *       unfreeze() is implicitly called. In the latter case, the rule is interpreted 
+	 *       as an integrity constraint. 
+	 */
+	ProgramBuilder& addRule(PrgRule& r);
 	
+	//@}
+
+	/*!
+	 * \name Rule creation functions
+	 * 
+	 * Functions in this group may be used to construct logic program rules.
+	 * The construction of a rule must start with a call to startRule() and
+	 * ends with a call to endRule(). Functions for adding elements to a
+	 * rule shall only be called between calls to startRule()/endRule() and
+	 * only one rule can be under construction at any one time.
+	 */
+	//@{
+
 	//! Starts the construction of a rule.
 	/*! 
 	 * \param t The type of the new rule.
 	 * \param bound The lower bound (resp. min weight) of the rule to be created.
-	 * \pre no rule is active
 	 *
 	 * \note the bound-parameter is only interpreted if the rule to be created is
 	 * either a constraint- or a weight-rule.
@@ -308,7 +404,7 @@ public:
 	//! Sets the bound (resp. min weight) of the currently active rule.
 	/*!
 	 * \param bound The lower bound (resp. min weight) of the rule to be created.
-	 * \pre The active rule is either a constraint or weight rule.
+	 * \pre The rule under construction is either a constraint or weight rule.
 	 */
 	ProgramBuilder& setBound(weight_t bound) { // only valid for CONSTRAINT and WEIGHTRULES
 		rule_.setBound(bound);
@@ -316,10 +412,6 @@ public:
 	}
 
 	//! Adds the atom with the given id as a head to the currently active rule.
-	/*!
-	 * \pre startRule was called && atomId > 0
-	 * \note if atomId is not yet known, an atom with the given id is implicitly created.
-	 */
 	ProgramBuilder& addHead(Var atomId) {
 		assert(atomId > 0);
 		rule_.addHead(atomId);
@@ -328,85 +420,75 @@ public:
 	
 	//! Adds a subgoal to the currently active rule.
 	/*!
+	 * \pre atomId > 0 && weight >= 0
 	 * \param atomId The id of the atom to be added to the rule.
 	 * \param pos true if the atom is positive. Fals otherwise
 	 * \param weight The weight the new predecessor should have in the rule.
-	 * \pre atomId > 0 && weight >= 0
 	 * \note The weight parameter is only used if the active rule is a weight or optimize rule.
-	 * \note if atomId is not yet known, an atom with the given id is implicitly created.
 	 */
 	ProgramBuilder& addToBody(Var atomId, bool pos, weight_t weight = 1) {
 		rule_.addToBody(atomId, pos, weight);
 		return *this;
 	}
 	
-	//! Finishes the creation of the active rule and adds it to the program
-	/*! 
-	 * \pre The program is not frozen
-	 * \note Must be called once for each rule.
+	//! Finishes the construction of the active rule and adds it to the program.
+	/*!
+	 * \see ProgramBuilder::addRule(PrgRule&);
 	 */
 	ProgramBuilder& endRule() {
 		return addRule(rule_);
 	}
-
-	//! Adds the rule r to the program
-	/*!
-	 * \pre The program is not frozen
-	 */
-	ProgramBuilder& addRule(PrgRule& r);
 	
-	//! Finishes the definition of the logic program (or its current increment).
-	/*!
-	 * Call this method to load the program (increment) into the solver of the maintained context
-	 *
-	 * \return false if the program is initially conflicting, true otherwise.
-	 *
-	 * \note If endProgram returned false, the state of the ProgramBuilder object is undefined. In that case,
-	 * only startProgram() and disposeProgram() remain valid operations.
-	 *
-	 * \note After endProgram was called, the program is considered "frozen". One shall not add atoms/rules
-	 * to a frozen program. Call ProgramBuilder::updateProgram() to "unfreeze" a program.
-	 *
-	 */
-	bool endProgram();
+	//@}
 
-	//! Clones the program and adds it to the given ctx
-	/*
-	 * \pre The program is currently frozen
+	/*!
+	 * \name Query functions
 	 * 
+	 * Functions in this group are useful to query important information
+	 * once the program is frozen, i.e. after endProgram() was called.
 	 */
-	bool cloneProgram(SharedContext& ctx);
-
-	//! returns the stored context object
-	SharedContext* context() const { return ctx_; }
+	 //@{
 	
-	//! returns a reference to the program's dependency graph
+	//! Returns true if the program contains any minimize statements.
+	bool   hasMinimize() const { return minimize_ != 0; }
+	//! Returns the number of atoms in the logic program.
+	uint32 numAtoms() const  { return (uint32)atoms_.size()-1; }
+	//! Returns the number of bodies in the current (slice of the) logic program.
+	uint32 numBodies() const { return (uint32)bodies_.size(); }
+	//! Returns the id of the first atom of the current step.
+	Var    startAtom() const { return incData_?incData_->startAtom_:1; }
+	
+	//! Returns a reference to the program's dependency graph.
 	/*!
 	 * \note A dependency graph is only created if the program is
 	 * not tight and unfounded set checking was not explicity disabled.
 	 * Otherwise, the function returns 0.
 	 * \note If release is true, ownership of the graph is transferred
 	 * to the caller.
-	 *
 	 */
 	GraphPtr dependencyGraph(bool release = false) const { return !release ? graph_.get() : graph_.release(); }
 	
-	//! returns true if the program contains at least one minimize statement
-	bool hasMinimize() const { return minimize_ != 0; }
-	
-	//! adds all minimize statements contained in the program to m
-	/*!
-	 * \pre The program is frozen and not trivially UNSAT, ie. endProgram was already called and did not return false.
-	 */
-	void addMinimize(MinimizeBuilder& m);
+	//! Returns the stored context object
+	SharedContext* context() const { return ctx_; }
 
+	//! Returns the internal literal that is associated with the given atom.
 	/*!
-	 * Writes the (possibly simplified) program in lparse-format to the given stream.
-	 * \pre endProgram was called and the program is currently "frozen".
+	 * \pre atomId is a known atom
+	 * \return A literal that is valid in the current solving context. 
+	 * \note Untill endProgram() is called, atoms from the current step are
+	 *       associated with the always-false literal negLit(0).
+	 * \throws std::logic_error if precondition is violated.
 	 */
-	void writeProgram(std::ostream& os);  // Pre: endProgram
+	Literal getLiteral(Var atomId) const;
 	
+	//! Returns a vector of internal literals that, when assumed true, makes all frozen atoms false. 
+	/*!
+	 * \pre The program is currently frozen. 
+	 */
+	void    getAssumptions(LitVec& out) const;
+
 	PreproStats stats;
+	//@}
 
 	// FOR TESTING
 	PrgAtomNode*  getAtom(Var atomId) const { return atoms_[atomId]; }
@@ -429,6 +511,7 @@ private:
 	// ------------------------------------------------------------------------
 	// Program definition
 	PrgAtomNode*  resize(Var atomId);
+	bool          inCompute(Literal x) const;
 	void          addRuleImpl(PrgRule& r, const PrgRule::RData& rd);
 	bool          handleNatively(const PrgRule& r, const PrgRule::RData& rd) const;
 	bool          transformNoAux(const PrgRule& r, const PrgRule::RData& rd) const;
@@ -457,7 +540,6 @@ private:
 	void          setConflict();
 	void          updateFrozenAtoms();
 	void          normalize();
-	uint32        startAtom() const { return incData_?incData_->startAtom_:1; }
 	// ------------------------------------------------------------------------
 	// Statistics
 	void upRules(RuleType r, int i)   { stats.rules[0] += i; stats.rules[r] += i; }
