@@ -37,8 +37,8 @@ class SharedContext;
 class MinimizeBuilder;
 struct ObjectiveFunction {
 	WeightLitVec lits;
-	int32        adjust;
-	int32        rhs;
+	wsum_t       adjust;
+	wsum_t       rhs;
 };
 
 class Input {
@@ -137,6 +137,8 @@ public:
 	 * \pre system uses ASCII
 	 */
 	bool parseInt(int& val);
+	bool parseInt64(int64& val);
+	bool parseInt(int& val, int min, int max);
 	//! skips horizontal white-space
 	void skipWhite();
 	//! works like std::getline
