@@ -25,117 +25,39 @@
 #endif
 
 #include <clasp/solver_types.h>
+#include <clasp/solver_strategies.h>
 #include <clasp/shared_context.h>
-#include <string>
-#include <utility>
 
 namespace Clasp { 
-///////////////////////////////////////////////////////////////////////////////
-// classes used by the Solver
-///////////////////////////////////////////////////////////////////////////////
-class   DecisionHeuristic;
-
-/**
- * \defgroup solver Solver and related classes.
- */
-//@{
-
-//! Parameter-Object for configuring a solver.
-struct SolverStrategies {
-	typedef std::auto_ptr<DecisionHeuristic>  Heuristic;
-	//! Clasp's two general search strategies
-	enum SearchStrategy {
-		use_learning, /*!< Analyze conflicts and learn First-1-UIP-clause */
-		no_learning   /*!< Don't analyze conflicts - chronological backtracking */
-	};
-	//! Antecedents to consider during conflict clause minimization.
-	enum CflMinAntes {
-		no_antes              = 0,  /*!< Don't minimize first-uip-clauses. */
-		binary_antes          = 2,  /*!< Consider only binary antecedents.         */
-		binary_ternary_antes  = 6,  /*!< Consider binary- and ternary antecedents. */
-		all_antes             = 7   /*!< Consider all antecedents.                 */
-	};
-	//! Reduction algorithm to use during solving.
-	enum ReduceAlgo {
-		reduce_linear   = 0, /*!< Linear algorithm from clasp-1.3.x. */
-		reduce_in_place = 1, /*!< Reorder learnt db and remove inactive constraints. */
-		reduce_stable   = 2  /*!< Similar to reduce_in_place but without changing the order of learnt db. */
-	};
-	//! Score to measure "activity" of learnt constraints.
-	enum ReduceScore {
-		score_act, /*!< Activity only: how often constraint is used during conflict analysis. */
-		score_lbd, /*!< Use literal block distance as activity. */
-		score_both /*!< Use activity and lbd together. */
-	};
-	enum HeuOpts {
-		heu_use_static_sign = 1,
-		heu_use_model       = 2
-	};
-	Heuristic           heuristic;           /*!< Active decision heuristic */
-	SearchStrategy      search;              /*!< Current search strategy */
-	RNG                 rng;                 /*!< RNG used during search */
-	uint32              heuOpts;             /*!< Set of HeuOpts */
-	int                 saveProgress;        /*!< Enable progress saving if > 0 */
-	int                 reverseArcs;         /*!< Use "reverse-arcs" during learning if > 0 */
-	int                 otfs;                /*!< Enable "on-the-fly" subsumption if > 0 */
-	CflMinAntes         cflMinAntes;         /*!< Antecedents to look at during conflict clause minimization */
-	int                 reduceAlgo;          /*!< Shall be one of ReduceAlgo */
-	int                 reduceScore;         /*!< Shall be one of ReduceScore */
-	uint32              reduceGlue;          /*!< Clauses with lbd <= reduceGlue are exempt from deletion   */
-	bool                strengthenRecursive; /*!< If true, use more expensive recursive nogood minimization */
-	bool                randomWatches;       /*!< Initialize watches randomly in clauses */
-	bool                updateLbd;           /*!< Update lbds of antecedents during conflict analysis */
-	uint32              compress()    const { return compress_; }
-	void setCompressionStrategy(uint32 len) {
-		compress_ = len == 0 ? static_cast<uint32>(-1) : len;
-	}
-	int                 compareScore(const LearntConstraint::Activity& lhs, const LearntConstraint::Activity& rhs) const {
-		switch (reduceScore) {
-			case score_act: 
-				if (lhs.scoreAct() != rhs.scoreAct()) { 
-					return ((int)lhs.scoreAct()) - ((int)rhs.scoreAct()); 
-				}
-				// activity scores are equal - fall through
-			case score_lbd:  
-				if (lhs.scoreLbd() != rhs.scoreLbd()) {
-					return ((int)lhs.scoreLbd()) - ((int)rhs.scoreLbd()); 
-				}
-				// lbd scores are equal - fall through
-			case score_both: 
-			default: break;
-		}
-		return ((int)lhs.scoreBoth()) - ((int)rhs.scoreBoth());
-	}
-	uint32              score(const LearntConstraint::Activity& act) const {
-		switch (reduceScore) {
-			case score_act:  return act.scoreAct();
-			case score_lbd:  return act.scoreLbd();
-			case score_both: return act.scoreBoth();
-		}
-		return 0;
-	}
-private:
-	friend class Solver;
-	SolverStrategies(const SolverStrategies&);
-	SolverStrategies& operator=(const SolverStrategies&);
-	//! creates a default-initialized object.
-	SolverStrategies();
-	uint32 compress_;
-};
 
 //! Parameter-Object for controlling search limits.
 struct SearchLimits {
 	explicit SearchLimits(uint64 conf = UINT64_MAX) 
 		: conflicts(conf)
 		, local(UINT64_MAX)
-		, learnts(UINT32_MAX) {
+		, learnts(UINT32_MAX)
+		, xLbd(0.0f)
+		, xCfl(0.0f) {
 	}
-	bool   reached() const { return conflicts == 0 || local == 0; }
-	uint64 conflicts; /*!< Search for at least number of conflicts */
-	uint64 local;     /*!< Search for at least number of conflicts in branch  */
-	uint32 learnts;   /*!< Limit on number of learnt lemmas        */
+	bool   reached(const SolveStats& st) const { 
+		return conflicts == 0 || local == 0 || dynamicRestart(st);
+	}
+	bool   dynamicRestart(const SolveStats& st) const {
+		return st.queue && st.queue->full() 
+			&&   (((st.queue->avgLbd()*xLbd) > st.avgLbd()) || ((st.queue->avgCfl()*xCfl) > st.avgCfl()));
+	}
+	uint64 conflicts; /*!< Search for at least number of conflicts.             */
+	uint64 local;     /*!< Search for at least number of conflicts in branch.   */
+	uint32 learnts;   /*!< Limit on number of learnt lemmas.                    */
+	float  xLbd;      /*!< Adjustment factor for dynamic restarts based on LBD. */
+	float  xCfl;      /*!< Adjustment factor for dynamic restarts based on CFL. */
 };
 
+/**
+ * \defgroup solver Solver and related classes.
+ */
+//@{
+
 //! clasp's Solver class.
 /*!
  * A Solver-object maintains the state and provides the functions 
@@ -499,11 +421,15 @@ public:
 	
 	//! Removes upto remMax percent of the learnt nogoods.
 	/*!
-	 * \param remMax Percantage of learnt nogoods that should be removed ([0.0f, 1.0f])
-	 * \note Nogoods that are the reason for a literal to be in the assignment
-	 * are said to be locked and won't be removed.
+	 * \param remMax  Fraction of nogoods to remove ([0.0,1.0]).
+	 * \param rs      Strategy to apply during nogood deletion.
+	 * \return        The number of locked and active/glue clauses currently exempt from deletion.
+	 * \note 
+	 *   Nogoods that are the reason for a literal to be in the assignment
+	 *   are said to be locked and won't be removed.
 	 */
-	void reduceLearnts(float remMax);
+	struct DBInfo { uint32 size; uint32 locked; uint32 pinned; };
+	DBInfo reduceLearnts(float remMax, const ReduceStrategy& rs = ReduceStrategy());
 
 	//! Resolves the active conflict using the selected strategy.
 	/*!
@@ -571,16 +497,16 @@ public:
 	//! Wraps SharedContext::addBinary() resp. SharedContext::learnBinary() depending on the type t.
 	bool addBinary(Literal p, Literal q, const ClauseInfo& t) {
 		t.type() == Constraint_t::static_constraint
-			? shared_->addBinary(p,q), true
-			: shared_->learnBinary(*this, p, q, t);
+			? shared_->addBinary(p,q)
+			: (void)shared_->learnBinary(*this, p, q, t);
 		return true;
 	}
 	
 	//! Wraps SharedContext::addTernary() resp. SharedContext::learnTernary() depending on the type t.
 	bool addTernary(Literal p, Literal q, Literal r, const ClauseInfo& t) {
 		t.type() == Constraint_t::static_constraint
-			? shared_->addTernary(p,q, r), true
-			: shared_->learnTernary(*this, p, q, r, t);
+			? shared_->addTernary(p,q, r)
+			: (void)shared_->learnTernary(*this, p, q, r, t);
 		return true;
 	}
 	
@@ -834,6 +760,9 @@ public:
 		if (assign_.saved(v) == value_free) { assign_.setSavedValue(v, val); }
 	}
 	//! Computes the literal block distance of p's antecedent given as the range [first, last).
+	/*!
+	 * \note Returns strict lbd (i.e. lbd+1) if strategies().updateLbd > 1
+	 */
 	uint32 computeLbd(Literal p, const Literal* first, const Literal* last);
 	//! Visitor function for antecedents used during conflict clause minimization.
 	bool ccMinimize(Literal p, CCMinRecursive* rec) const {
@@ -888,18 +817,21 @@ private:
 		PostPropagator* look;
 	};
 	struct CmpScore {
-		CmpScore(const ConstraintDB& learnts, const SolverStrategies& st) : db(learnts), strat(st) {}
-		bool operator()(LearntConstraint::Activity lhs, LearntConstraint::Activity rhs) const { 
-			return strat.compareScore(lhs, rhs) < 0; 
-		}
-		bool operator()(uint32 lhsId, uint32 rhsId) const { return (*this)(db[lhsId], db[rhsId]); }
+		typedef LearntConstraint::Activity Activity;
+		typedef std::pair<uint32, Activity> ViewPair;
+		CmpScore(const ConstraintDB& learnts, ReduceStrategy::Score sc, uint32 g) : db(learnts), rs(sc), glue(g) {}
+		uint32 score(const Activity& act)                             const { return ReduceStrategy::asScore(rs, act); }
+		bool operator()(uint32 lhsId, uint32 rhsId)                   const { return (*this)(db[lhsId], db[rhsId]); }
+		bool operator()(const ViewPair& lhs, const ViewPair& rhs)     const { return this->operator()(lhs.second, rhs.second); }
+		bool operator()(Activity lhs, Activity rhs)                   const { return ReduceStrategy::compare(rs, lhs, rhs) < 0;}
 		bool operator()(const Constraint* lhs, const Constraint* rhs) const {
-			return strat.compareScore(
+			return this->operator()(
 				static_cast<const LearntConstraint*>(lhs)->activity(), 
-				static_cast<const LearntConstraint*>(rhs)->activity()) < 0;
+				static_cast<const LearntConstraint*>(rhs)->activity());
 		}
-		const ConstraintDB&     db;
-		const SolverStrategies& strat;      
+		const ConstraintDB&   db;
+		ReduceStrategy::Score rs;
+		uint32                glue;
 	private: CmpScore& operator=(const CmpScore&);
 	};
 	inline  bool hasStopConflict() const;
@@ -919,9 +851,9 @@ private:
 	bool    ccReverseArc(Literal p, uint32 maxLevel, uint32 maxNew, Antecedent& out);
 	void    undoFree(ConstraintDB* x);
 	void    setConflict(Literal p, const Antecedent& a, uint32 data);
-	uint32  reduceLinear(uint32 minR, uint32 maxR);
-	uint32  reduceSorted(uint32 minR, uint32 maxR);
-	uint32  reduceSortedInPlace(uint32 minR, uint32 maxR);
+	DBInfo  reduceLinear(uint32 minR, uint32 maxR, const CmpScore& cmp);
+	DBInfo  reduceSorted(uint32 minR, uint32 maxR, const CmpScore& cmp);
+	DBInfo  reduceSortedInPlace(uint32 minR, uint32 maxR, const CmpScore& cmp);
 	ConstraintDB* allocUndo(Constraint* c);
 	SolverStrategies  strategy_;    // Strategies used by this solver-object
 	Assignment        assign_;      // three-valued assignment
