@@ -52,26 +52,27 @@ struct ParallelStats {
 	ParallelStats() { reset(); }
 	double  cpuTime;  /**< (Estimated) cpu time of the current solver. */
 	uint64  splits;   /**< Number of split requests handled. */
-	uint64  gps;      /**< Number of guiding paths received. */
+	uint64  gps :62;  /**< Number of guiding paths received. */
+	uint64  term:1;   /**< Solver was terminated             */
+	uint64  win :1;   /**< Solver terminated the algorithm   */
 	uint64  gpLits;   /**< Sum of literals in received guiding paths. */
 	uint64  shared;   /**< Number of nogoods shared. */
 	uint64  imps;     /**< Number of initial implications from shared. */
 	uint64  jumps;    /**< Sum of backjumps needed to integrate new implications. */
-	bool    terminated;
 	void    reset() { 
 		std::memset(this, 0, sizeof(*this)); 
 		cpuTime    = 0.0;
-		terminated = false;
 	}
 	void    accu(const ParallelStats& o) {
 		cpuTime  += o.cpuTime;
 		splits   += o.splits;
 		gps      += o.gps;
+		term     |= o.term;
+		win      |= o.win;
 		gpLits   += o.gpLits;
 		shared   += o.shared;
 		imps     += o.imps;
 		jumps    += o.jumps;
-		terminated |= o.terminated;
 	}
 	void newGP(LitVec::size_type length) {
 		++gps;
@@ -84,7 +85,6 @@ struct JumpStats {
 	JumpStats() { reset(); }
 	void reset(){ std::memset(this, 0, sizeof(*this)); }
 	void accu(const JumpStats& o) {
-		jumps   += o.jumps;
 		bJumps  += o.bJumps;
 		jumpSum += o.jumpSum;
 		boundSum+= o.boundSum;
@@ -93,7 +93,6 @@ struct JumpStats {
 		if (o.maxBound  > maxBound)  maxBound  = o.maxBound;
 	}
 	void    update(uint32 dl, uint32 uipLevel, uint32 bLevel) {
-		++jumps;
 		jumpSum += dl - uipLevel; 
 		maxJump = std::max(maxJump, dl - uipLevel);
 		if (uipLevel < bLevel) {
@@ -104,9 +103,6 @@ struct JumpStats {
 		}
 		else { maxJumpEx = maxJump; }
 	}
-	double avgJumpLen()   const { return jumpSum/std::max(1.0,(double)jumps); }
-	double avgJumpLenEx() const { return (jumpSum-boundSum)/std::max(1.0,(double)jumps); }
-	uint64  jumps;    /**< Number of backjumps (i.e. number of analyzed conflicts). */
 	uint64  bJumps;   /**< Number of backjumps that were bounded. */
 	uint64  jumpSum;  /**< Number of levels that could be skipped w.r.t first-uip. */
 	uint64  boundSum; /**< Number of levels that could not be skipped because of backtrack-level.*/
@@ -122,8 +118,12 @@ struct CoreStats {
 	void accu(const CoreStats& o) {
 		choices  += o.choices;
 		conflicts+= o.conflicts;
+		analyzed += o.analyzed;
 		restarts += o.restarts;
+		lRestart  = std::max(o.lRestart, lRestart);
 		models   += o.models;
+		sumCfl   += o.sumCfl;
+		sumLbd   += o.sumLbd;
 		
 		binary   += o.binary;
 		ternary  += o.ternary;
@@ -156,32 +156,89 @@ struct CoreStats {
 		--models;
 		modLits -= size;
 	}
+	double avgLbd() const { return sumLbd / (double)analyzed; }
+	double avgCfl() const { return sumCfl / (double)analyzed; }
 	uint64 choices;   /**< Number of choices performed. */
 	uint64 conflicts; /**< Number of conflicts found. */
+	uint64 analyzed;  /**< Number of backjumps (i.e. number of analyzed conflicts). */
 	uint64 restarts;  /**< Number of restarts. */ 
+	uint64 lRestart;  /**< Number of conflicts on last restart. */
 	uint64 models;    /**< Number of models found. */
-
+	uint64 sumCfl;    /**< Sum of conflict levels. */
+	uint64 sumLbd;    /**< Sum of lbds.            */
 	uint64 learnts[Constraint_t::max_value]; /**< Number of learnt nogoods of type t-1. */
 	uint64 lits[Constraint_t::max_value];    /**< Sum of literals in nogoods of type t-1. */
-	uint64 binary;    /**< Number of learnt binary nogoods. */
-	uint64 ternary;   /**< Number of learnt ternary nogoods.*/
+	uint32 binary;    /**< Number of learnt binary nogoods. */
+	uint32 ternary;   /**< Number of learnt ternary nogoods.*/
 	uint64 deleted;   /**< Sum of learnt nogoods removed. */
 	uint64 modLits;   /**< Sum of decision literals in models. */
 };
 
+struct QueueImpl {
+	explicit QueueImpl(uint32 size) : maxSize(size), wp(0), rp(0) {}
+	bool    full()  const { return size() == maxSize; }
+	uint32  size()  const { return ((rp > wp) * cap()) + wp - rp;}
+	uint32  cap()   const { return maxSize + 1; }
+	void    clear()       { wp = rp = 0; }
+	uint32  top()   const { return buffer[rp]; }
+	void    push(uint32 x){ buffer[wp] = x; if (++wp == cap()) { wp = 0; } }
+	void    pop()         { if (++rp == cap()) { rp = 0; } }
+	uint32  maxSize;
+	uint32  wp;
+	uint32  rp;
+	uint32  buffer[1];
+};
+
+struct SumQueue {
+	static SumQueue* create(uint32 size) {
+		void* m = ::operator new(sizeof(SumQueue) + (size*sizeof(uint32)));
+		return new (m) SumQueue(size);
+	}
+	void destroy() { this->~SumQueue(); ::operator delete(this); }
+	void reset()   { sumLbd = sumCfl = samples = 0; queue.clear(); }
+	void update(uint32 dl, uint32 lbd) {
+		if (samples++ >= queue.maxSize) {
+			uint32 y = queue.top(); 
+			sumLbd  -= (y & 127u);
+			sumCfl  -= (y >> 7u);
+			queue.pop();
+		}
+		sumLbd += lbd;
+		sumCfl += dl;
+		queue.push((dl << 7) + lbd);
+	}
+	double  avgLbd() const { return sumLbd / (double)queue.maxSize; }
+	double  avgCfl() const { return sumCfl / (double)queue.maxSize; }
+	uint32  maxSize()const { return queue.maxSize; }
+	bool    full()   const { return queue.full();  }
+	uint32     sumLbd;
+	uint32     sumCfl;
+	uint32     samples;
+	QueueImpl  queue;
+private: SumQueue(uint32 size) : sumLbd(0), sumCfl(0), samples(0), queue(size) {}
+private: SumQueue(const SumQueue&);
+private: SumQueue& operator=(const SumQueue&);
+};
+
 //! A struct for aggregating statistics of one solve operation.
 struct SolveStats : public CoreStats {
-	SolveStats() : jumps(0), parallel(0) { }
-	SolveStats(const SolveStats& o) : CoreStats(o), jumps(0), parallel(0) { 
+	SolveStats() : jumps(0), queue(0), parallel(0) { }
+	SolveStats(const SolveStats& o) : CoreStats(o), jumps(0), queue(0), parallel(0) { 
 		if (o.jumps)    jumps    = new JumpStats(*o.jumps); 
 		if (o.parallel) parallel = new ParallelStats(*o.parallel);
+		if (o.queue)    enableQueue(o.queue->maxSize());
+	}
+	~SolveStats() { delete jumps; delete parallel; if (queue) queue->destroy();   }
+	void enableJumpStats()        { if (!jumps)   jumps    = new JumpStats();     }
+	void enableParallelStats()    { if (!parallel)parallel = new ParallelStats(); }
+	void enableQueue(uint32 size) { 
+		if (queue && queue->maxSize()!=size) { queue->destroy(); queue = 0; }
+		if (!queue)                          { queue = SumQueue::create(size); }
 	}
-	~SolveStats() { delete jumps; delete parallel; }
-	void enableJumpStats()    { if (!jumps)   jumps    = new JumpStats();   }
-	void enableParallelStats(){ if (!parallel)parallel = new ParallelStats(); }
 	void reset() {  
 		CoreStats::reset();
 		if (jumps)   jumps->reset();
+		if (queue)   queue->reset();
 		if (parallel)parallel->reset();
 	}
 	void accu(const SolveStats& o) {
@@ -189,11 +246,15 @@ struct SolveStats : public CoreStats {
 		if (jumps && o.jumps) jumps->accu(*o.jumps);
 		if (parallel && o.parallel) parallel->accu(*o.parallel);
 	}
-	void updateJumps(uint32 dl, uint32 uipLevel, uint32 bLevel) {
-		if (!jumps) return;
-		jumps->update(dl, uipLevel, bLevel);
+	void updateJumps(uint32 dl, uint32 uipLevel, uint32 bLevel, uint32 lbd) {
+		++analyzed;
+		sumCfl += dl;
+		sumLbd += lbd;
+		if (jumps) { jumps->update(dl, uipLevel, bLevel); }
+		if (queue) { queue->update(dl, lbd); }
 	}
 	JumpStats*     jumps;    /**< Optional jump statistics. */
+	SumQueue*      queue;    /**< Optional queue for running averages. */
 	ParallelStats* parallel; /**< Optional parallel statistics. */
 private:
 	SolveStats& operator=(const SolveStats&);
