@@ -51,7 +51,9 @@ DecisionHeuristic* HeuristicOptions::createHeuristic() const {
 		heu = new ClaspVmtf( m, loops == 1);
 	}
 	else if (name == "vsids") {
-		heu = new ClaspVsids(loops == 1);
+		double m = extra.vsidsDecay <= 0 ? 0.95 : extra.vsidsDecay;
+		while (m > 1.0) { m /= 10; }
+		heu = new ClaspVsids(m, loops == 1);
 	}
 	else if (name == "none") {
 		heu = new SelectFirst();
@@ -106,13 +108,18 @@ bool LocalOptions::validate(std::string& err) {
 		}
 		SolverStrategies* s = &solver_->strategies();
 		s->cflMinAntes = SolverStrategies::no_antes;
-		s->setCompressionStrategy(0);
-		s->saveProgress = 0;
-		solve.restart.sched.init(0, 0.0, 0);
-		solve.reduce.setStrategy(-1.0, 0.0, 0.0);
+		s->compress    = UINT32_MAX;
+		s->saveProgress= 0;
+		solve.restart.disable();
+		solve.reduce.disable();
 		solve.setRandomizeParams(0,0);
 		solve.setShuffleParams(0,0);
-		solve.restart.local = solve.restart.bounded = solve.reduce.reduceOnRestart = false;
+		solve.restart.type = RestartParams::default_restarts;
+		solve.restart.bounded = solve.reduce.reduceOnRestart = false;
+	}
+	if (solve.reduce.baseFrac == 0) {
+		solve.reduce.growSched = ScheduleStrategy::none();
+		solve.reduce.baseFrac  = 3.0;
 	}
 	return true;
 }
@@ -121,19 +128,14 @@ void LocalOptions::initFrom(const LocalOptions& other) {
 	if (other.heuristic_) {
 		heuristic()      = *other.heuristic_;
 	}
-	SolverStrategies* st = &solver_->strategies();
-	st->rng          = other.solver_->strategies().rng;
-	st->search       = other.solver_->strategies().search;
-	st->saveProgress = other.solver_->strategies().saveProgress;
-	st->cflMinAntes  = other.solver_->strategies().cflMinAntes;
-	st->strengthenRecursive = other.solver_->strategies().strengthenRecursive;
-	st->randomWatches= other.solver_->strategies().randomWatches;
-	st->setCompressionStrategy(other.solver_->strategies().compress());
-	st->heuOpts      = other.solver_->strategies().heuOpts;
-	
+	// Only copy simple strategies but not the heuristic
+	DecisionHeuristic* oh = other.solver_->strategies().heuristic.release();
+	DecisionHeuristic* sh = solver_->strategies().heuristic.release();
+	solver_->strategies() = other.solver_->strategies();
+	solver_->strategies().heuristic.reset(sh);
+	other.solver_->strategies().heuristic.reset(oh);
 	solve   = other.solve;
 	loopRep = other.loopRep;
-
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -518,7 +520,7 @@ void ClaspFacade::setProblemSize() const {
 		else {
 			ps = ctx.numConstraints();
 		}
-		o->solve.reduce.setProblemSize(ps);
+		o->solve.reduce.base = ps;
 	}
 }
 
