@@ -108,55 +108,66 @@ SharedLiterals* SharedLiterals::share() {
 /////////////////////////////////////////////////////////////////////////////////////////
 struct ClauseCreator::Watches {
 	Watches(const Solver& s, const Literal* begin, const Literal* end, bool knownOrder) {
-		uint32 size = uint32(end - begin);
+		uint32 size   = uint32(end - begin);
+		simpSize      = size;
+		std::fill_n(lits, (int)Clause::MAX_SHORT_LEN, negLit(0));
 		if (!knownOrder) { init(s, (Literal*)begin, size, lits); }
 		else             { 
-			std::memcpy(lits, begin, std::min(size, uint32(Clause::MAX_SHORT_LEN))*sizeof(Literal));
+			uint32 i= size;
+			while (i-- && Detail::levelAbstraction(s, begin[i]) == 0) {
+				--simpSize;
+			}
+			std::memcpy(lits, begin, std::min(simpSize, uint32(Clause::MAX_SHORT_LEN))*sizeof(Literal));
 			abstr_f = Detail::levelAbstraction(s, lits[0]);
 			abstr_s = Detail::levelAbstraction(s, lits[1]);
 		}
-		for (uint32 i = size; i < Clause::MAX_SHORT_LEN; ++i) { lits[i] = negLit(0); }
+		assert(simpSize <= size);
 	}
 	Watches(const Solver& s, LitVec& in, bool knownOrder) {
 		assert(in.size() > 1);
+		simpSize = in.size();
 		if (!knownOrder) { init(s, &in[0], in.size(), 0); }
 		else             { abstr_f = Detail::levelAbstraction(s, in[0]); abstr_s = Detail::levelAbstraction(s, in[1]);  }
 		lits[0] = in[0];
 		lits[1] = in[1];
 	}
 	void init(const Solver& s, Literal* in, uint32 size, Literal* out) {
-		uint32 pos = 2, fp = 0, sp = 1;
-		abstr_f    = 0, abstr_s = 0;
-		for (uint32 i = 0; i != size; ++i) {
-			Literal p      = in[i];
-			uint32 pos_p   = i;
-			uint32 abstr_p = Detail::levelAbstraction(s, p);
-			if (abstr_p > abstr_f) {
-				std::swap(abstr_p, abstr_f);
-				pos_p = fp;
-				fp    = i;
-			}
-			if (abstr_p > abstr_s) {
-				std::swap(pos_p, sp);
-				abstr_s = abstr_p;
-			}
-			if (out && i > 1) {
-				out[pos++] = in[pos_p];
-				if (pos   == Clause::MAX_SHORT_LEN) pos = 2;
+		abstr_f     = abstr_s = simpSize = 0;
+		uint32 fp   = 0,  sp  = 0;
+		for (uint32 i = 0, j  = 0, abstr_p; i != size; ++i) {
+			Literal p = in[i];
+			abstr_p   = Detail::levelAbstraction(s, p);
+			if (abstr_p) {
+				++simpSize;
+				if (out) {
+					out[j++]= abstr_p > abstr_s ? in[sp] : in[i];
+					if (j  == Clause::MAX_SHORT_LEN) { j = 2; }
+				}
+				if (abstr_p > abstr_f) {
+					assert(abstr_f >= abstr_s);
+					sp      = fp;
+					abstr_s = abstr_f;
+					fp      = i;
+					abstr_f = abstr_p;
+				}
+				else if (abstr_p > abstr_s) {
+					sp      = i;
+					abstr_s = abstr_p;
+				}
 			}
 		}
 		if (out) {
-			if (size > 0) out[0] = in[fp];
-			if (size > 1) out[1] = in[sp];
+			if (simpSize > 0) out[0] = in[fp];
+			if (simpSize > 1) out[1] = in[sp];
 		}
-		else if (size > 2) {
-			if (sp == 0) { std::swap(in[0], in[1]); }
+		else if (size > 1) {
 			std::swap(in[0], in[fp]);
-			std::swap(in[1], in[sp]);
+			std::swap(in[1], sp != 0 ? in[sp] : in[fp]);
 		}
 	}
 	uint32  abstr_f;
 	uint32  abstr_s;
+	uint32  simpSize;
 	Literal lits[Clause::MAX_SHORT_LEN];
 };
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -269,7 +280,7 @@ ClauseCreator::Status ClauseCreator::status(const Solver& s, const Watches& w, u
 	bool notSat = (flags & clause_not_sat) != 0;
 	bool notRoot= (flags & clause_not_root_sat) != 0;
 	if (x == status_conflicting || (x == status_asserting && s.isFalse(w.lits[0]))) {
-		x = (flags & clause_not_conflict) == 0 ? status_unit : status_subsumed;
+		x = (flags & clause_not_conflict) == 0 ? status_unit : status_conflicting;
 	}
 	else if (x == status_sat && (notSat || (notRoot && s.level(w.lits[0].var()) <= s.rootLevel()))) {
 		x = status_subsumed;
@@ -352,7 +363,7 @@ ClauseHead* ClauseCreator::newLearntClause(Solver& s, LitVec& lits, const Clause
 	Detail::Sink sharedPtr(0);
 	sharedPtr.clause = s.sharedContext()->distribute(s, &lits[0], static_cast<uint32>(lits.size()), e);
 	if (lits.size() <= Clause::MAX_SHORT_LEN || sharedPtr.clause == 0) {
-		if (!s.isFalse(lits[1]) || lits.size() < s.strategies().compress()) {
+		if (!s.isFalse(lits[1]) || lits.size() < s.strategies().compress) {
 			ret = Clause::newLearntClause(s, lits, e);
 		}
 		else {
@@ -376,8 +387,8 @@ ClauseCreator::Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flag
 	if (lits.size() > 1) {
 		Watches w(s, lits, (flags & clause_known_order) != 0);
 		Status x = status(s, w, flags);
-		if (x == status_subsumed) { 
-			ret.ok = lits.size() > 0 && s.isTrue(lits[0]);
+		if (x == status_subsumed || x == status_conflicting) { 
+			ret.ok = (x == status_subsumed);
 			return ret;
 		}
 		if (extra.type() == Constraint_t::static_constraint && s.satPrepro()) {
@@ -409,6 +420,7 @@ ClauseCreator::Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flag
 	return ret;
 }
 
+
 ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags, const ClauseInfo& e) {
 	assert(!s.hasConflict() && "ClauseCreator::integrate() - precondition violated!");
 	Detail::Sink shared( (modeFlags & clause_no_release) == 0 ? clause : 0);
@@ -416,34 +428,35 @@ ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause
 	Result  result;
 	Watches w(s, clause->begin(), clause->end(), (modeFlags & clause_known_order) != 0);
 	Status  stat( status(s, w, modeFlags) );
-	if (clause->size() > 1 && stat != status_subsumed) {
-		if (clause->size() > Clause::MAX_SHORT_LEN) {
+	if (stat != status_subsumed && (stat != status_conflicting || (modeFlags & clause_not_conflict) == 0)) {
+		uint32 impSize  = (modeFlags & clause_explicit) != 0 || !s.sharedContext()->updateShortImplications() ? 1 : 3;
+		if (w.simpSize > Clause::MAX_SHORT_LEN) {
 			result.local  = Clause::newShared(s, clause, e, w.lits, shared.clause == 0);
 			shared.clause = 0;
 		}
-		else if (clause->size() > 3 || (modeFlags & clause_explicit) != 0 || !s.sharedContext()->updateShortImplications()) {
-			result.local  = Clause::newClause(s, w.lits, clause->size(), e);
+		else if (w.simpSize > impSize) {
+			result.local  = Clause::newClause(s, w.lits, w.simpSize, e);
 		}
 		else {
-			// implicitly shared via binary/ternary implication graph;
+			// unary clause or implicitly shared via binary/ternary implication graph;
 			// only check for implication/conflict but do not create
 			// a local representation for the clause
-			s.stats.addLearnt(clause->size(), e.type());
+			s.stats.addLearnt(w.simpSize, e.type());
 			modeFlags    |= clause_no_add;
 		}
-		if ((modeFlags & clause_no_add) == 0) { s.addLearnt(result.local, clause->size()); }
+		if ((modeFlags & clause_no_add) == 0) { s.addLearnt(result.local, w.simpSize); }
 	}
 	if ((stat & status_unit) != 0) {
 		Antecedent ante = result.local ? Antecedent(result.local) : Antecedent(~w.lits[1], ~w.lits[2]);
 		uint32 DL       = s.decisionLevel();
 		uint32 impLevel = s.level(w.lits[1].var());
 		result.ok       = s.force(w.lits[0], impLevel, ante);
-		if (clause->size() == 1) { s.stats.addLearnt(1, e.type()); }
-		if (s.stats.parallel)    {
+		if (s.stats.parallel){
 			s.stats.parallel->imps  += 1;
 			s.stats.parallel->jumps += (DL - s.decisionLevel());
 		}
 	}
+	else { result.ok  = stat != status_conflicting; }
 	return result;
 }
 ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags) { 
@@ -557,7 +570,7 @@ bool Clause::updateWatch(Solver& s, uint32 pos) {
 			for (Literal* it  = tailS + idx; it < end; ++it) {
 				if (!s.isFalse(*it)) {
 					std::swap(*it, head_[pos]);
-					data_.local.idx = (++it - tailS);
+					data_.local.idx = static_cast<uint32>(++it - tailS);
 					return true;
 				}
 			}
@@ -782,8 +795,8 @@ Literal* Clause::removeFromTail(Solver& s, Literal* it, Literal* end) {
 uint32 Clause::size() const {
 	LitRange t = const_cast<Clause&>(*this).tail();
 	return !isSentinel(head_[2])
-		? 3 + (t.second - t.first)
-		: 2;
+		? 3u + static_cast<uint32>(t.second - t.first)
+		: 2u;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // mt::SharedLitsClause
