@@ -29,7 +29,7 @@ Enumerator::EnumeratorConstraint::EnumeratorConstraint() : mini_(0), updates_(0)
 void Enumerator::EnumeratorConstraint::attach(Solver& s) {
 	Enumerator* e = s.sharedContext()->enumerator();
 	if (e->mini_) {
-		mini_ = MinimizeBuilder::attach(s, e->mini_);
+		mini_ = e->mini_->attach(s);
 		if (e->optimize()) { s.addWatch(s.sharedContext()->tagLiteral(), this); }
 	}
 }
