@@ -454,10 +454,9 @@ Literal ClaspVmtf::selectRange(Solver&, const Literal* first, const Literal* las
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspVsids selection strategy
 /////////////////////////////////////////////////////////////////////////////////////////
-const double vsidsVarDecay_g = 1 / 0.95;
-
-ClaspVsids::ClaspVsids(bool loops) 
+ClaspVsids::ClaspVsids(double decay, bool loops) 
 	: vars_(GreaterActivity(score_)) 
+	, decay_(1.0 / decay)
 	, inc_(1.0)
 	, scoreLoops_(loops) 
 	, reinit_(true) {}
@@ -505,7 +504,7 @@ void ClaspVsids::newConstraint(const Solver&, const Literal* first, LitVec::size
 			}
 		}
 		if (t == Constraint_t::learnt_conflict) {
-			inc_ *= vsidsVarDecay_g;
+			inc_ *= decay_;
 		}
 	}
 }
