@@ -257,7 +257,7 @@ void Lookahead::undoLevel(Solver& s) {
 			const Literal* b = &a[0]+s.levelStart(s.decisionLevel());
 			if (b->watched()) {
 				// remember current DL for b
-				uint32 dist = ((&a[0]+a.size()) - b);
+				uint32 dist = static_cast<uint32>(((&a[0]+a.size()) - b));
 				imps_.assign(b+1, b + std::min(dist, uint32(2048)));
 			}
 			else if (score.score[b->var()].testedBoth()) {
