@@ -26,17 +26,17 @@ namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
 #define STRATEGY(x) (x)
 
-MinimizeConstraint::MinimizeConstraint(SharedData* d)
+MinimizeConstraint::MinimizeConstraint(SharedData* d, bool owner)
 	: shared_(d)
 	, pos_(d->lits)
 	, undo_(0)
 	, undoTop_(0)
 	, type_(d->weights.empty() ? SINGLE_LEVEL_ARITH : MULTI_LEVEL_ARITH)
-	, ownsShared_(false) {
-	sum_      = new wsum_t[numRules()*4](); // [0-numRules()) stores sum
-	opt_      = &sum_[1 * numRules()];
-	temp_     = &sum_[2 * numRules()];
-	tempOpt_  = &sum_[3 * numRules()];
+	, ownsShared_(owner) {
+	sum_      = new wsum_t[numRules()*4](); // [0-numRules()) stores current sum
+	opt_      = &sum_[1 * numRules()];      // current opt
+	temp_     = &sum_[2 * numRules()];      // temp sum
+	tempOpt_  = &sum_[3 * numRules()];      // temp opt
 	active_[0]= active_[1] = 0;
 }
 
@@ -65,15 +65,19 @@ bool MinimizeConstraint::attach(Solver& s) {
 	undo_    = new UndoInfo[(numL*2)+1]; 
 	undoTop_ = 0;
 	posTop_  = numL+1;
-	std::memcpy(sum_, &shared_->sum()[0], numRules()*sizeof(wsum_t));
+	for (uint32 i = 0; i != numRules(); ++i) {
+		sum_[i]= shared_->sum(i); 
+	}
 	STRATEGY(convert(sum_));
 	*opt_    = std::numeric_limits<wsum_t>::max();
 	for (WeightVec::size_type i = 0; i != up.size(); ++i) {
 		MinimizeConstraint::propagate(s, shared_->lits[up[i]].first, up[i]);
 	}
-	bool hasOpt = restoreOptimum() && s.isTrue(s.sharedContext()->tagLiteral());
-	return !s.hasConflict()
-		&& (!hasOpt || (propagateImpl(s, propagate_new_opt) && integrateNext(s)));
+	if (shared()->mode() == MinimizeMode_t::enumerate) {
+		restoreOptimum();
+	}
+	bool intOpt = s.isTrue(s.sharedContext()->tagLiteral());
+	return !s.hasConflict() && (!intOpt || integrateNext(s));
 }
 
 void MinimizeConstraint::destroy(Solver* s, bool detach) {
@@ -235,17 +239,18 @@ Constraint* MinimizeConstraint::cloneAttach(Solver&) {
 // Stores the current model as the optimum and determines the decision level 
 // on which the search should continue.
 uint32 MinimizeConstraint::commitCurrent(const Solver& s) {
+	uint32 dl          = s.decisionLevel();
 	// store current sum as new optimum in shared...
 	const Optimum* opt = shared_->setOptimum(STRATEGY(sumToOpt()));
-	// ...and *this
-	std::memcpy(tempOpt_, sum_, numRules()*sizeof(wsum_t));
-	uint32 dl = s.decisionLevel();
 	if (shared_->mode() == MinimizeMode_t::optimize) {
 		// temporarily assume next tentative optimum and
 		// compute first level on which constraint is violated
+		std::memcpy(tempOpt_, sum_, numRules()*sizeof(wsum_t));
 		dl = updateOpt(&s, true, opt);
+		// restore to new optimum
+		std::swap(tempOpt_, opt_);
 	}
-	std::swap(tempOpt_, opt_);
+	// else: keep bound during enumeration
 	return dl - 1;
 }
 
@@ -259,8 +264,11 @@ bool MinimizeConstraint::restoreOptimum() {
 // integrates a new tentative optimum
 bool MinimizeConstraint::integrateNext(Solver& s) {
 	assert(!s.hasConflict() && s.isTrue(s.sharedContext()->tagLiteral()));
-	std::swap(tempOpt_, opt_);
-	uint32 cl = updateOpt(&s, true, shared_->optimum());
+	uint32 cl = UINT32_MAX;
+	if (shared_->mode() != MinimizeMode_t::enumerate) {
+		std::swap(tempOpt_, opt_);
+		cl      = updateOpt(&s, true, shared_->optimum());
+	}
 	do {
 		pos_    = shared_->lits;
 		if (cl > s.decisionLevel() && propagateImpl(s, propagate_new_opt)) {
@@ -304,14 +312,14 @@ uint32 MinimizeConstraint::updateOpt(const Solver* s, bool applyStep, const Opti
 		seq = opt->seq;
  		ret = shared_->convert(opt, applyStep, opt_);
  		step= opt->step;
- 		if (seq == opt->seq) {
+		if (seq == opt->seq && opt == shared_->optimum()) {
  			break;
  		}
 		// optimum has changed - reload
 		opt = shared_->optimum();
 	}
 	if (!ret) return 0;
-	if (applyStep && greater(sum_, opt_, STRATEGY(convert(opt_)))) {
+	if (greater(sum_, opt_, STRATEGY(convert(opt_)))) {
 		assert(s);
 		uint32 lev = step.first;
 		if (lev == shared_->maxLevel() && step.second == 1 && (sum_[lev]-opt_[lev]) == 1) {
@@ -378,17 +386,20 @@ bool MinimizeConstraint::implied(wsum_t* lhs, weight_t wOrIdx) {
 /////////////////////////////////////////////////////////////////////////////////////////
 // SharedMinimizeData
 /////////////////////////////////////////////////////////////////////////////////////////
-SharedMinimizeData::SharedMinimizeData(const SumVec& initSum, const SumVec& initOpt, MinimizeMode m) : mode_(m) {
-	sum_         = initSum;
+SharedMinimizeData::SharedMinimizeData(const SumVec& lhsSum, const SumVec& lhsAdjust, const SumVec& rhsSum, MinimizeMode m) : mode_(m) {
+	sum_         = lhsSum;
+	adjust_      = lhsAdjust.empty() ? 0 : new wsum_t[lhsAdjust.size()];
+	std::copy(lhsAdjust.begin(), lhsAdjust.end(), adjust_);
 	opts_[0].seq = 0;
-	opts_[0].opt = initOpt;
-	opts_[0].step= Optimum::Step(numRules()-1, 1);
+	opts_[0].opt = rhsSum;
+	opts_[0].step= Optimum::Step(numRules()-1, 0);
 	opt_         = &opts_[0];
 	next_        = &opts_[1];
 	low_         = -static_cast<int64>(UINT64_MAX/2);
 	hierarch_    = 0;
 }
 SharedMinimizeData::~SharedMinimizeData() {
+	delete [] adjust_;
 }
 
 void SharedMinimizeData::destroy() const {
@@ -400,17 +411,23 @@ void SharedMinimizeData::setMode(MinimizeMode m, int hierarchical)  {
 	mode_     = m; 
 	hierarch_ = 0;
 	if (mode_ == MinimizeMode_t::optimize && hierarchical > 0) {
-		opt_->step = Optimum::Step(0, 1);
-		hierarch_  = (uint32)hierarchical;
+		opt_->step.first = 0;
+		hierarch_        = (uint32)hierarchical;
 	}
 }
 
+MinimizeConstraint* SharedMinimizeData::attach(Solver& s, bool setOwner) {
+	MinimizeConstraint* ret = new MinimizeConstraint(this, setOwner);
+	ret->attach(s);
+	return ret;
+}
+
 const SharedMinimizeData::Optimum* SharedMinimizeData::setOptimum(const wsum_t* newOpt) {
 	Optimum* temp = opt_;
-	next_->seq    = temp->seq+1;
 	next_->opt.assign(newOpt, newOpt+numRules());
 	next_->step   = temp->step;
 	setStep(next_);
+	next_->seq    = temp->seq+1;
 	opt_          = next_;
 	next_         = temp;
 	return opt_;
@@ -424,7 +441,7 @@ bool SharedMinimizeData::optimizeNext(Optimum* opt) {
 		step  = hierarch_ != 3 ? 1 : step/2;
 		return true;
 	}
-	for (uint32 maxL = numRules()-1; lev < maxL; ) {
+	for (uint32 maxL = maxLevel(); lev < maxL && step; ) {
 		++lev;
 		if (opt->opt[lev] > sum_[lev]) {
 			low_ = -static_cast<int64>(UINT64_MAX/2);
@@ -432,12 +449,15 @@ bool SharedMinimizeData::optimizeNext(Optimum* opt) {
 			return true;
 		}
 	}
+	lev = UINT32_MAX;
 	return false;
 }
 
 void SharedMinimizeData::setStep(Optimum* o) {
+	assert(o->step.first != UINT32_MAX);
+	wsum_t& step   = o->step.second;
+	if (!step) step= 1;
 	if (hierarch_ > 1) {
-		wsum_t& step = o->step.second;
 		uint32& lev  = o->step.first;
 		wsum_t  val  = o->opt[lev];
 		low_         = std::max(sum_[lev], low_);
@@ -455,28 +475,26 @@ void SharedMinimizeData::setStep(Optimum* o) {
 		}
 		if ( step == 0 || (val - step) < low_ ) {
 			step = 1;
-			if (!optimizeNext(o)) {
-				step = 0;
-			}
+			optimizeNext(o);
 		}
 	}
 }
 
 bool SharedMinimizeData::convert(const Optimum* opt, bool applyStep, wsum_t* optOut) const {
-	if (applyStep && opt->step.second == 0) return false;
-	applyStep &= opt->opt[0] != std::numeric_limits<wsum_t>::max();
-	uint32 i   = static_cast<uint32>(opt->opt.size());
-	uint32 lev = applyStep ? opt->step.first : i;
-	wsum_t n   = opt->step.second;
-	bool   ap  = !applyStep;
+	wsum_t n    = opt->step.second;
+	applyStep  &= (n != 0);
+	uint32 i    = static_cast<uint32>(opt->opt.size());
+	uint32 lev  = applyStep ? opt->step.first : i;
+	bool   ap   = !applyStep;
 	for (wsum_t t; i--; ) {
-		t = opt->opt[i];
-		if (i > lev || (!ap && (opt->opt[i] - n) < sum_[i])) { 
-			t = std::numeric_limits<wsum_t>::max(); 
+		t         = opt->opt[i];
+		if (i > lev) { 
+			t       = std::numeric_limits<wsum_t>::max(); 
 		}
-		else if (!ap) {
-			ap = true;
-			t  = opt->opt[i] - n;
+		else if (!ap && lev != UINT32_MAX) {
+			t       = opt->opt[i] - n;
+			if (t  >= sum_[i]) { ap = true; }
+			else               { t  = std::numeric_limits<wsum_t>::max(); }
 		}
 		optOut[i] = t;
 	}
@@ -499,29 +517,30 @@ MinimizeBuilder& MinimizeBuilder::addRule(const WeightLitVec& lits, wsum_t initS
 		lits_.pop_back();
 		ready_ = false;
 	}
-	uint32 lev = (uint32)initial_.size();
+	uint32 lev = (uint32)sum_.size();
 	for (WeightLitVec::const_iterator it = lits.begin(); it != lits.end(); ++it) {
 		if (it->second > 0) {
 			lits_.push_back(LitRep(it->first, new Weight(lev, it->second)));
 		}
 	}
-	initial_.push_back(initSum);
+	sum_.push_back(initSum);
 	return *this;
 }
 
 // sets an initial optimum
 MinimizeBuilder& MinimizeBuilder::setOptimum(uint32 lev, wsum_t opt) {
-	if (lev >= opts_.size()) {
-		opts_.resize(lev+1, std::numeric_limits<wsum_t>::max());
+	if (lev >= rhs_.size()) {
+		rhs_.resize(lev+1, std::numeric_limits<wsum_t>::max());
 	}
-	opts_[lev] = opt;
+	rhs_[lev] = opt;
 	return *this;
 }
 
-// adds the weights of the given lit to the appropriate levels
-void MinimizeBuilder::adjustSum(LitRep lit) {
+// adds the weights of the given lit to the appropriate levels in vec
+void MinimizeBuilder::addTo(LitRep lit, SumVec& vec) {
+	vec.resize(numRules());
 	for (Weight* r = lit.second; r; r = r->next) {
-		initial_[r->level] += r->weight;
+		vec[r->level] += r->weight;
 	}
 }
 
@@ -556,7 +575,7 @@ bool MinimizeBuilder::prepare(SharedContext& ctx) {
 		}
 		else {
 			if (s.isTrue(lits_[i].first)) {
-				adjustSum(lits_[i]);
+				addTo(lits_[i], sum_);
 			}
 			Weight::free(lits_[i].second);
 			++i;	
@@ -575,7 +594,7 @@ bool MinimizeBuilder::prepare(SharedContext& ctx) {
 		else if ( (cmp = greaterW.compare(lits_[i], lits_[k])) != 0 ) {
 			LitVec::size_type wMin = cmp > 0 ? k : i;
 			LitVec::size_type wMax = cmp > 0 ? i : k;
-			adjustSum(lits_[wMin]);
+			addTo(lits_[wMin], adjust_);
 			mergeReduceWeight(lits_[wMax], lits_[wMin]);
 			assert(lits_[wMin].second == 0);
 			lits_[j++] = lits_[wMax];
@@ -584,7 +603,7 @@ bool MinimizeBuilder::prepare(SharedContext& ctx) {
 		}
 		else {
 			// weights are equal
-			adjustSum(lits_[i]);
+			addTo(lits_[i], adjust_);
 			Weight::free(lits_[i].second);
 			Weight::free(lits_[k].second);
 			i += 2;
@@ -593,15 +612,15 @@ bool MinimizeBuilder::prepare(SharedContext& ctx) {
 	}
 	shrinkVecTo(lits_, j);
 	std::stable_sort(lits_.begin(), lits_.end(), greaterW);
-	if (initial_.empty()) {
-		initial_.push_back(0);
+	if (sum_.empty()) {
+		sum_.push_back(0);
 	}
 	// add terminating sentinel literal
-	lits_.push_back(LitRep(posLit(0), new Weight(static_cast<uint32>(initial_.size()-1), 0)));
+	lits_.push_back(LitRep(posLit(0), new Weight(static_cast<uint32>(sum_.size()-1), 0)));
 	bool ok = true;
-	for (uint32 i = 0, end = std::min((uint32)initial_.size(), (uint32)opts_.size()); i != end; ++i) {
-		if (initial_[i] != opts_[i]) {
-			ok = initial_[i] < opts_[i];
+	for (uint32 i = 0, end = std::min((uint32)sum_.size(), (uint32)rhs_.size()); i != end; ++i) {
+		if (sum_[i] != rhs_[i]) {
+			ok = sum_[i] < rhs_[i];
 			break;
 		}
 	}
@@ -613,7 +632,7 @@ void MinimizeBuilder::addAssumption(Literal ma) {
 	if (ma != posLit(0)) {
 		weight_t highest = lits_[0].second->weight + 1;
 		lits_.insert(lits_.begin(), LitRep(ma, new Weight(0, highest)));
-		initial_[0] -= highest;
+		sum_[0] -= highest;
 	}
 }
 
@@ -627,12 +646,9 @@ MinimizeBuilder::SharedData* MinimizeBuilder::build(SharedContext& ctx, Literal
 		addAssumption(ma);
 		ready_ = true;
 	}
-	opts_.resize(initial_.size(), std::numeric_limits<wsum_t>::max());
-	if (opts_.back() != std::numeric_limits<wsum_t>::max()) {
-		opts_.back() += 1;
-	}
-	SharedData* srep = new (::operator new(sizeof(SharedData) + (lits_.size()*sizeof(WeightLiteral)))) SharedData(initial_, opts_);
-	if (initial_.size() == 1) {
+	rhs_.resize(sum_.size(), std::numeric_limits<wsum_t>::max());
+	SharedData* srep = new (::operator new(sizeof(SharedData) + (lits_.size()*sizeof(WeightLiteral)))) SharedData(sum_, adjust_, rhs_);
+	if (sum_.size() == 1) {
 		for (LitVec::size_type i = 0; i != lits_.size(); ++i) {
 			srep->lits[i] = WeightLiteral(lits_[i].first, lits_[i].second->weight);
 		}
@@ -663,19 +679,12 @@ MinimizeBuilder::SharedData* MinimizeBuilder::build(SharedContext& ctx, Literal
 MinimizeConstraint* MinimizeBuilder::buildAndAttach(SharedContext& ctx, MinimizeMode mode) {
 	if (SharedData* d = build(ctx)) {
 		d->setMode(mode, false);
-		MinimizeConstraint* m = attach(*ctx.master(), d);
-		m->ownsShared_ = true;
+		MinimizeConstraint* m = d->attach(*ctx.master(), true);
 		return m;
 	}
 	return 0;
 }
 
-MinimizeConstraint* MinimizeBuilder::attach(Solver& s, SharedData* data) {
-	MinimizeConstraint* ret = new MinimizeConstraint(data);
-	ret->attach(s);
-	return ret;
-}
-
 // computes x.weight -= by.weight
 // PRE: x.weight > by.weight
 void MinimizeBuilder::mergeReduceWeight(LitRep& x, LitRep& by) {
