@@ -316,7 +316,7 @@ Constraint* RecordEnumerator::LocalConstraint::cloneAttach(Solver& other) {
 bool RecordEnumerator::LocalConstraint::add(Solver& s, LitVec& lits, const ClauseInfo& e) {
 	ClauseCreator::Result ret;
 	if (!queue) {
-		ret = ClauseCreator::create(s, lits, ClauseCreator::clause_no_add, e);
+		ret = ClauseCreator::create(s, lits, ClauseCreator::clause_no_add|ClauseCreator::clause_known_order, e);
 		if (ret.local) { db.push_back(ret.local); }
 	}
 	else {
