@@ -146,12 +146,13 @@ private:
 struct ParallelSolve::SharedData {
 	typedef tbb::concurrent_queue<const LitVec*> queue;
 	enum MsgFlag {
-		terminate_flag     = 1u, sync_flag  = 2u,  split_flag    = 4u, 
-		restart_flag       = 8u, unsat_flag = 16u, complete_flag = 32u,
-		interrupt_flag     = 64u,  // set on terminate from outside
-		allow_gp_flag      = 128u, // set if splitting mode is active
-		forbid_restart_flag= 256u, // set if restarts are no longer allowed
-		cancel_restart_flag= 512u  // set if last restart request was cancelled
+		terminate_flag        = 1u, sync_flag  = 2u,  split_flag    = 4u, 
+		restart_flag          = 8u, unsat_flag = 16u, complete_flag = 32u,
+		interrupt_flag        = 64u,  // set on terminate from outside
+		allow_gp_flag         = 128u, // set if splitting mode is active
+		forbid_restart_flag   = 256u, // set if restarts are no longer allowed
+		cancel_restart_flag   = 512u, // set if current restart request was cancelled by some thread
+		restart_abandoned_flag=1024u  // set to signal that threads must not give up their gp
 	};
 	enum Message {
 		msg_terminate      = (terminate_flag),
@@ -284,14 +285,10 @@ void ParallelSolve::setForceGP(bool gp) {
 void ParallelSolve::setIntegrate(uint32 grace, uint8 filter) {
 	typedef ThreadOptions::Distribution Dist;
 	intGrace_     = grace;
-	intFlags_     = ClauseCreator::clause_not_root_sat | ClauseCreator::clause_no_add;
+	intFlags_     = ClauseCreator::clause_no_add;
 	intHeuristic_ = filter == Dist::filter_heuristic;
-	if (filter == Dist::filter_sat) {
-		intFlags_ |= (ClauseCreator::clause_not_root_sat | ClauseCreator::clause_not_sat);
-	}
-	else if (filter == Dist::filter_gp) {
-		intFlags_ |= ClauseCreator::clause_not_root_sat;
-	}
+	if (filter != Dist::filter_no) { intFlags_ |= ClauseCreator::clause_not_root_sat; }
+	if (filter == Dist::filter_sat){ intFlags_ |= ClauseCreator::clause_not_sat; }
 }
 
 void ParallelSolve::setRestarts(uint32 maxR, const ScheduleStrategy& rs) {
@@ -378,21 +375,21 @@ void ParallelSolve::startSolve(Solver& s, const SolveParams& p, const LitVec& as
 		if (shared_->ctx->master()->stats.jumps) {
 			s.stats.enableJumpStats();
 		}
-		thread_[id]->attach(s, p); // connect solver<->handler
+		thread_[id]->attach(s, p);                  // connect solver<->handler
+		shared_->setControl(SharedData::sync_flag); // force initial sync with all threads
 		if (id == 0) {
 			// explicity init parallel handler because Solver::endInit() was
 			// already called.
 			thread_[0]->init(s);
 			shared_->path = &assume;
 			shared_->syncT.start();
+			waitOnSync(s);
 		}
 		else if (!shared_->ctx->attach(s) && !shared_->interrupt()) {
-			// problem is UNSAT - terminate algorithm
-			LOG(s.id(), "TRIVIALLY UNSAT!");
-			terminate(s, true);
-			return;
+			bool unsat = !shared_->terminate();
+			if (unsat) { LOG(s.id(), "TRIVIALLY UNSAT!"); }
+			terminate(s, unsat);
 		}
-		waitOnSync(s);
 	}
 }
 
@@ -405,6 +402,7 @@ bool ParallelSolve::doSolve(Solver& s, const SolveParams& p, const LitVec& assum
 	PathPtr a(0); InitParams init = p.init;
 	try {
 		startSolve(s, p, assume);
+		LOG(s.id(), "ENTERING  work loop");
 		for (ValueRep last = value_free; requestWork(s, a);) {
 			thread_[s.id()]->prepareForGP(*a, shared_->maxConflict, a.is_owner());
 			s.stats.reset();
@@ -513,7 +511,7 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 			// do not take over ownership of initial gp!
 			if (a == shared_->path) { out.release(); } 
 			// propagate any new facts now that we have new work
-			if (s.simplify())      { return true; }
+			if (s.simplify())       { return true; }
 			// s now has a conflict - either an artifical stop conflict
 			// or a real conflict - we'll handle it in the next iteration
 			// via the call to popRootLevel()
@@ -558,49 +556,50 @@ void ParallelSolve::terminate(Solver& s, bool complete) {
 		}
 		else {
 			shared_->postMessage(SharedData::msg_terminate, true);
+			s.stats.parallel->win = 1;
 			if (complete) { shared_->setControl(SharedData::complete_flag); }
 		}
 	}
-	if (!complete) { s.stats.parallel->terminated = true; }
+	if (!complete) { s.stats.parallel->term = 1; }
 }
 
 // handles an active synchronization request
 // returns true to signal that s should restart otherwise false
 bool ParallelSolve::waitOnSync(Solver& s) {
 	if (!thread_[s.id()]->handleRestartMessage()) {
-		shared_->setControl(SharedData::forbid_restart_flag);
+		shared_->setControl(SharedData::cancel_restart_flag);
 	}
+	bool hasPath = thread_[s.id()]->hasPath();
 	LOG(s.id(), "WAITING   for synchronization");
 	if (shared_->workSem.wait()) {
 		// last man standing - complete synchronization request
 		shared_->workReq     = 0;
 		shared_->restartReq  = 0;
 		bool unsat           = shared_->hasControl(SharedData::unsat_flag);
-		if (shared_->globalR.index() != maxRestarts_) {
-			shared_->maxConflict = shared_->globalR.current();
-			shared_->globalR.next();
+		bool restart         = !unsat  && shared_->hasControl(SharedData::restart_flag);
+		bool init            = true;
+		if (restart) {
+			init = shared_->allowRestart() && !shared_->hasControl(SharedData::cancel_restart_flag);
+			if (init) { shared_->globalR.next(); }
+			shared_->maxConflict = shared_->allowRestart() && shared_->globalR.idx <= maxRestarts_
+				? shared_->globalR.current()
+				: UINT64_MAX;
 		}
-		else {
-			shared_->maxConflict = UINT64_MAX;
+		else if (unsat && !shared_->enumerator()->optimizeNext()) {
+			// nothing more to do
+			init = false;
+			shared_->setControl(SharedData::terminate_flag | SharedData::complete_flag);
 		}
-		if (unsat || shared_->allowRestart()) {
-			initQueue(); // add initial path to work queue
-			shared_->clearControl(SharedData::cancel_restart_flag);
-			if (unsat && !shared_->enumerator()->optimizeNext()) {
-				// nothing more to do
-				shared_->setControl(SharedData::terminate_flag | SharedData::complete_flag);
-			}
-		}
-		else {
-			shared_->setControl(SharedData::cancel_restart_flag);
-		}
-		shared_->clearControl(SharedData::sync_flag | SharedData::split_flag | SharedData::unsat_flag | SharedData::restart_flag);
+		// add initial path to work queue
+		if (init) { initQueue();  }
+		else      { shared_->setControl(SharedData::restart_abandoned_flag); }
+		shared_->clearControl(SharedData::msg_split | SharedData::msg_sync_unsat | SharedData::msg_sync_restart | SharedData::restart_abandoned_flag | SharedData::cancel_restart_flag);
 		shared_->syncT.stop();
-		LOG(s.id(), "COMPLETED synchronization after %.3fs", shared_->syncT.total());
+		LOG(s.id(), "COMPLETED %s after %.3fs", (unsat ? "unsat-sync" : "restart-sync"), shared_->syncT.total());
 		// wake up all blocked threads
 		shared_->workSem.reset();
 	}
-	return shared_->terminate() || !shared_->hasControl(SharedData::cancel_restart_flag);
+	return shared_->terminate() || (hasPath && !shared_->hasControl(SharedData::restart_abandoned_flag));
 }
 
 // If guiding path scheme is active only one
@@ -683,7 +682,7 @@ bool ParallelSolve::handleMessages(Solver& s) {
 	if (shared_->terminate()) {
 		LOG(s.id(), "TERMINATE message received!");
 		h->handleTerminateMessage();
-		s.stats.parallel->terminated = true;
+		s.stats.parallel->term = 1;
 		s.setStopConflict();
 		return false;
 	}
@@ -735,6 +734,7 @@ void ParallelHandler::attach(Solver& s, const SolveParams& p) {
 	aggStats.enableParallelStats();
 	if (s.stats.jumps) { aggStats.enableJumpStats(); }
 	gp_.reset();
+	gp_.impl= UINT32_MAX;
 	intTail_= 0;
 	error_  = 0;
 	s.addPost(&messageHandler_);
@@ -844,7 +844,7 @@ bool ParallelHandler::propagateFixpoint(Solver& s) {
 	// This is necessary during hierarchical optimization because otherwise
 	// integrating a too strong optimum might irrevocably force the assumption literal
 	// which would defeat its purpose.
-	if ( (s.stats.choices & 63) == 0 && s.isTrue(s.sharedContext()->tagLiteral()) ) {
+	if ( hasPath() && (s.stats.choices & 63) == 0 && s.isTrue(s.sharedContext()->tagLiteral()) ) {
 		if (!s.sharedContext()->enumerator()->update(s, gp_.split)) {
 			return false;
 		}
