@@ -270,7 +270,7 @@ weight_t PrgBodyNode::bound() const {
 weight_t PrgBodyNode::sumWeights() const {
 	return !extended()
 		? (weight_t)size()
-		: (uint32)std::max(extra_.ext->sumWeights, weight_t(0));
+		: std::max(extra_.ext->sumWeights, weight_t(0));
 }
 
 // Returns the weight of the idx'th subgoal in B+/B-
