@@ -36,7 +36,7 @@
 #pragma warning (disable : 4996)
 static int mkstemp(char* templ) {
 	int fd;
-	int x = strlen(templ);
+	std::size_t x = strlen(templ);
 	assert(x >= 6);
 	do {
 		_mktemp(templ);
