@@ -177,15 +177,14 @@ void SatElite::attach(uint32 clauseId, bool initialClause) {
 			updateHeap(v);
 		}
 	}
-	occurs_[c[0].var()].watches.push_back(clauseId);
+	occurs_[c[0].var()].addWatch(clauseId);
 	addToSubQueue(clauseId);
 	stats.clAdded += !initialClause;
 }
 
 void SatElite::detach(uint32 id) {
 	Clause& c  = *clauses_[id];
-	VarVec& watches = occurs_[c[0].var()].watches;
-	watches.erase(std::find(watches.begin(), watches.end(), id));
+	occurs_[c[0].var()].removeWatch(id);
 	for (uint32 i = 0; i != c.size(); ++i) {
 		Var v = c[i].var();
 		occurs_[v].remove(id, c[i].sign(), false);
@@ -198,8 +197,7 @@ void SatElite::detach(uint32 id) {
 
 void SatElite::bceVeRemove(uint32 id, bool freeId, Var ev, bool blocked) {
 	Clause& c  = *clauses_[id];
-	VarVec& watches = occurs_[c[0].var()].watches;
-	watches.erase(std::find(watches.begin(), watches.end(), id));
+	occurs_[c[0].var()].removeWatch(id);
 	uint32 pos = 0;
 	for (uint32 i = 0; i != c.size(); ++i) {
 		Var v = c[i].var();
@@ -306,11 +304,11 @@ bool SatElite::propagateFacts() {
 	while (facts_ != s->numAssignedVars()) {
 		Literal l     = s->trail()[facts_++];
 		OccurList& ov = occurs_[l.var()];
-		LitVec& cls   = occurs_[l.var()].clauses;
-		for (uint32 i = 0, clId; i != cls.size(); ++i) {
-			if      (clauses_[clId=cls[i].var()] == 0)  { continue; }
-			else if (cls[i].sign() == l.sign())         { detach(clId); }
-			else if (!strengthenClause(clId, ~l))       { return false; }
+		ClRange cls   = occurs_[l.var()].clauseRange();
+		for (ClIter x = cls.first; x != cls.second; ++x) {
+			if      (clauses_[x->var()] == 0)        { continue; }
+			else if (x->sign() == l.sign())          { detach(x->var()); }
+			else if (!strengthenClause(x->var(), ~l)){ return false; }
 		}
 		ov.clear();
 		ov.mark(!l.sign());
@@ -337,13 +335,15 @@ bool SatElite::backwardSubsume() {
 			}
 		}
 		// Test against all clauses containing best
-		LitVec& cls   = occurs_[best.var()].clauses;
+		ClWList& cls  = occurs_[best.var()].refs;
 		Literal res   = negLit(0);
 		uint32  j     = 0;
-		for (uint32 i = 0; i != cls.size(); ++i) {
-			uint32 otherId  = cls[i].var();
+		// must use index access because cls might change!
+		for (uint32 i = 0, end = cls.left_size(); i != end; ++i) {
+			Literal cl      = cls.left(i);
+			uint32 otherId  = cl.var();
 			Clause* other   = clauses_[otherId];
-			if (other && other!= &c && (res = subsumes(c, *other, best.sign()==cls[i].sign()?posLit(0):best)) != negLit(0)) {
+			if (other && other!= &c && (res = subsumes(c, *other, best.sign()==cl.sign()?posLit(0):best)) != negLit(0)) {
 				if (res == posLit(0)) {
 					// other is subsumed - remove it
 					detach(otherId);
@@ -359,11 +359,11 @@ bool SatElite::backwardSubsume() {
 					if (res.var() == best.var() || clauses_[otherId] == 0)  { other = 0; }
 				}
 			}
-			if (other && j++ != i)  { cls[j-1] = cls[i]; }
+			if (other && j++ != i)  { cls.left(j-1) = cl; }
 		}
-		cls.erase(cls.begin()+j, cls.end());
+		cls.shrink_left(cls.left_begin()+j);
 		occurs_[best.var()].dirty = 0;
-		assert(occurs_[best.var()].numOcc() == (uint32)occurs_[best.var()].clauses.size());
+		assert(occurs_[best.var()].numOcc() == (uint32)cls.left_size());
 		if (!propagateFacts()) return false;
 	}   
 	queue_.clear();
@@ -430,19 +430,19 @@ bool SatElite::subsumed(LitVec& cl) {
 	for (LitVec::size_type i = 0; i != cl.size(); ++i) {
 		l = cl[i];
 		if (occurs_[l.var()].litMark == 0) { --str; continue; }
-		VarVec& watches = occurs_[l.var()].watches; // all clauses watching either l or ~l
-		VarVec::size_type wj = 0;
-		for (VarVec::size_type w = 0, end = watches.size(); w != end; ++w) {
-			Clause& c = *clauses_[watches[w]];
+		ClWList& cls   = occurs_[l.var()].refs; // right: all clauses watching either l or ~l
+		WIter wj       = cls.right_begin();
+		for (WIter w = wj, end = cls.right_end(); w != end; ++w) {
+			Clause& c = *clauses_[*w];
 			if (c[0] == l)  {
 				if ( (x = findUnmarkedLit(c, 1)) == c.size() ) {
-					while (w != end) { watches[wj++] = watches[w++]; }
-					watches.erase( watches.begin()+wj, watches.end() );
+					while (w != end) { *wj++ = *w++; }
+					cls.shrink_right( wj );
 					return true;
 				}
 				c[0] = c[x];
 				c[x] = l;
-				occurs_[c[0].var()].watches.push_back(watches[w]);
+				occurs_[c[0].var()].addWatch(*w);
 				if (occurs_[c[0].var()].litMark != 0 && findUnmarkedLit(c, x+1) == c.size()) {
 					occurs_[c[0].var()].unmark();  // no longer part of cl
 					++str;
@@ -450,13 +450,13 @@ bool SatElite::subsumed(LitVec& cl) {
 			}
 			else if ( findUnmarkedLit(c, 1) == c.size() ) {
 				occurs_[l.var()].unmark(); // no longer part of cl
-				while (w != end) { watches[wj++] = watches[w++]; }
-				watches.erase( watches.begin()+wj, watches.end() );
+				while (w != end) { *wj++ = *w++; }
+				cls.shrink_right( wj );
 				goto removeLit;
 			}
-			else { watches[wj++] = watches[w]; }  
+			else { *wj++ = *w; }  
 		}
-		watches.erase(watches.begin()+wj, watches.end());
+		cls.shrink_right(wj);
 		if (j++ != i) { cl[j-1] = cl[i]; }
 removeLit:;
 	}
@@ -479,11 +479,10 @@ removeLit:;
 bool SatElite::strengthenClause(uint32 clauseId, Literal l) {
 	Clause& c = *clauses_[clauseId];
 	if (c[0] == l) {
-		VarVec& w = occurs_[c[0].var()].watches;
-		w.erase(std::find(w.begin(), w.end(), clauseId));
+		occurs_[c[0].var()].removeWatch(clauseId);
 		// Note: Clause::strengthen shifts literals after l to the left. Thus
 		// c[1] will be c[0] after strengthen
-		occurs_[c[1].var()].watches.push_back(clauseId);
+		occurs_[c[1].var()].addWatch(clauseId);
 	}
 	++stats.litsRemoved;
 	c.strengthen(l);
@@ -498,22 +497,22 @@ bool SatElite::strengthenClause(uint32 clauseId, Literal l) {
 
 // Split occurrences of v into pos and neg and 
 // mark all clauses containing v
-LitVec& SatElite::splitOcc(Var v, bool mark) {
-	LitVec& cls      = occurs_[v].clauses;
+SatElite::ClRange SatElite::splitOcc(Var v, bool mark) {
+	ClRange cls      = occurs_[v].clauseRange();
 	occurs_[v].dirty = 0;
 	posT_.clear(); negT_.clear();
-	uint32 j = 0;
-	for (uint32 i = 0; i != cls.size(); ++i) {
-		if (Clause* c = clauses_[cls[i].var()]) {
+	ClIter j = cls.first;
+	for (ClIter x = j; x != cls.second; ++x) {
+		if (Clause* c = clauses_[x->var()]) {
 			assert(c->marked() == false);
 			c->setMarked(mark);
-			(cls[i].sign() ? negT_ : posT_).push_back(cls[i].var());
-			if (j != i) cls[j] = cls[i];
+			(x->sign() ? negT_ : posT_).push_back(x->var());
+			if (j != x) *j = *x;
 			++j;
 		}
 	}
-	cls.erase(cls.begin()+j, cls.end());
-	return cls;
+	occurs_[v].refs.shrink_left(j);
+	return occurs_[v].clauseRange();
 }
 
 void SatElite::markAll(const Literal* lits, uint32 size) const {
@@ -538,11 +537,11 @@ bool SatElite::bceVe(Var v, uint32 maxCnt) {
 	resCands_.clear();
 	// distribute clauses on v 
 	// check if number of clauses decreases if we'd eliminate v
-	uint32 bce     = options.bce;
-	LitVec& cls    = splitOcc(v, bce > 1);
-	uint32 cnt     = 0;
-	uint32 markMax = ((uint32)negT_.size() * (bce>1));
-	uint32 blocked = 0;
+	uint32  bce    = options.bce;
+	ClRange cls    = splitOcc(v, bce > 1);
+	uint32  cnt    = 0;
+	uint32  markMax= ((uint32)negT_.size() * (bce>1));
+	uint32  blocked= 0;
 	bool stop      = false;
 	Clause* lhs, *rhs;
 	for (VarVec::const_iterator i = posT_.begin(); i != posT_.end() && !stop; ++i) {
@@ -574,17 +573,18 @@ bool SatElite::bceVe(Var v, uint32 maxCnt) {
 		ctx_->eliminate(v);  // mark var as eliminated
 		// remove old clauses, store them in the elimination table so that
 		// (partial) models can be extended.
-		for (uint32 i = 0; i != cls.size(); ++i) {
+		for (ClIter it = cls.first; it != cls.second; ++it) {
 			// reuse first cnt ids for resolvents
-			if (clauses_[cls[i].var()]) {
-				bceVeRemove(cls[i].var(), i < cnt, v, false);
+			if (clauses_[it->var()]) {
+				bool freeId = (cnt && cnt--);
+				bceVeRemove(it->var(), freeId, v, false);
 			}
 		}
 		// add non trivial resolvents
-		cnt = 0;
 		assert( resCands_.size() % 2 == 0 );
-		for (VarVec::size_type i = 0; i != resCands_.size(); i+=2) {
-			if (!addResolvent(cls[cnt++].var(), *resCands_[i], *resCands_[i+1])) {
+		ClIter it = cls.first;
+		for (VarVec::size_type i = 0; i != resCands_.size(); i+=2, ++it) {
+			if (!addResolvent(it->var(), *resCands_[i], *resCands_[i+1])) {
 				return false;
 			}
 		}
@@ -609,8 +609,9 @@ bool SatElite::bceVe(Var v, uint32 maxCnt) {
 
 bool SatElite::bce() {
 	uint32 ops = 0;
-	for (VarVec& bce = occurs_[0].watches; !bce.empty(); ++ops) {
-		Var v         = bce.back(); bce.pop_back();
+	for (ClWList& bce= occurs_[0].refs; bce.right_size() != 0; ++ops) {
+		Var v          = *(bce.right_end()-1);
+		bce.pop_right();
 		occurs_[v].bce=0; 
 		if ((ops & 1023) == 0)   {
 			if (timeout())         { bce.clear(); return true; }
