@@ -146,6 +146,7 @@ void ShortImplicationsGraph::ImplicationList::move(ImplicationList& other) {
 ShortImplicationsGraph::ShortImplicationsGraph() {
 	bin_[0]  = bin_[1]  = 0;
 	tern_[0] = tern_[1] = 0;
+	shared_  = false;
 }
 ShortImplicationsGraph::~ShortImplicationsGraph() {
 	PodVector<ImplicationList>::destruct(graph_);
@@ -165,9 +166,9 @@ void ShortImplicationsGraph::resize(uint32 nodes) {
 
 uint32 ShortImplicationsGraph::numEdges(Literal p) const { return graph_[p.index()].size(); }
 
-bool ShortImplicationsGraph::addBinary(Literal p, Literal q, bool learnt, bool shared) {
+bool ShortImplicationsGraph::addBinary(Literal p, Literal q, bool learnt) {
 	uint32 added = 0;
-	if (!shared) {
+	if (!shared_) {
 		if (!learnt) { p.clearWatch(); q.clearWatch(); }
 		else         { p.watch(); q.watch(); }
 		getList(~p).push_left(q);
@@ -185,9 +186,9 @@ bool ShortImplicationsGraph::addBinary(Literal p, Literal q, bool learnt, bool s
 	return added > 0;
 }
 
-bool ShortImplicationsGraph::addTernary(Literal p, Literal q, Literal r, bool learnt, bool shared) {
+bool ShortImplicationsGraph::addTernary(Literal p, Literal q, Literal r, bool learnt) {
 	uint32 added = 0;
-	if (!shared) {
+	if (!shared_) {
 		if (!learnt) { p.clearWatch(); q.clearWatch(); r.clearWatch(); }
 		else         { p.watch(); q.watch(); r.watch(); }
 		getList(~p).push_right(std::make_pair(q, r));
@@ -240,7 +241,7 @@ void ShortImplicationsGraph::removeTrue(Solver& s, Literal p) {
 		remove_tern(graph_[(~r).index()], ~p);
 		if (s.value(q.var()) == value_free && s.value(r.var()) == value_free) {
 			// clause is binary on dl 0
-			addBinary(q, r, false, false);
+			addBinary(q, r, false);
 		}
 		// else: clause is SAT and removed when the satisfied literal is processed
 	}
@@ -328,7 +329,8 @@ void SatPreprocessor::reportProgress(char t, uint32 min, uint32 max) {
 // SharedContext
 /////////////////////////////////////////////////////////////////////////////////////////
 SharedContext::SharedContext() 
-	: symTabPtr_(new SharedSymTab()), master_(new Solver), distributor_(0), lastInit_(0), lastTopLevel_(0), shareCount_(1), shareConstr_(false) {
+	: symTabPtr_(new SharedSymTab()), master_(new Solver), distributor_(0), lastInit_(0), lastTopLevel_(0)
+	, shareCount_(1), shareConstr_(false), updateShort_(true), frozen_(false) {
 	addVar(Var_t::atom_body_var);
 	problem_.vars = 0;
 	addEnumerator(0);
@@ -336,7 +338,8 @@ SharedContext::SharedContext()
 }
 
 SharedContext::SharedContext(const SharedContext& rhs,  InitMode) 
-	: master_(new Solver), distributor_(0), lastInit_(0), lastTopLevel_(0), shareCount_(1), shareConstr_(false) {
+	: master_(new Solver), distributor_(0), lastInit_(0), lastTopLevel_(0), shareCount_(1)
+	, shareConstr_(false), updateShort_(true), frozen_(false) {
 	symTabPtr_ = rhs.symTabPtr_;
 	++symTabPtr_->refs;
 	addVar(Var_t::atom_body_var);
@@ -404,6 +407,7 @@ void SharedContext::eliminate(Var v) {
 
 Solver& SharedContext::startAddConstraints(uint32 constraintGuess) {
 	assert(unique());
+	frozen_ = false;
 	if (tag_ == negLit(0)) {
 		// add aux var for tag literal
 		tag_ = posLit(addVar(Var_t::atom_var));
@@ -430,6 +434,43 @@ bool SharedContext::addUnary(Literal p) {
 	return master()->addUnary(p, Constraint_t::static_constraint);
 }
 
+bool SharedContext::learnUnary(Solver& s, Literal p, ConstraintType t) {
+	assert(t != Constraint_t::static_constraint);
+	s.stats.addLearnt(1, t);
+	if (!unique() && (!updateShortImplications() || btig_.addBinary(p, negLit(0), true))) {
+		distribute(s, p, negLit(0), negLit(0), t);
+	}
+	return true;
+}
+
+bool SharedContext::learnBinary(Solver& s, Literal p, Literal q, const ClauseInfo& info) {
+	assert(info.type() != Constraint_t::static_constraint);
+	bool added = true;
+	if (!updateShortImplications()) {
+		Literal lits[2] = {p, q};
+		s.addLearnt(Clause::newShortClause(s, 0, lits, 2, info), 2);
+	}
+	else if ((added=btig_.addBinary(p, q, true)) == true) {
+		s.stats.addLearnt(2, info.type());
+	}
+	if (added && !unique()) { distribute(s, p, q, negLit(0), info.type()); }
+	return added;
+}
+
+bool SharedContext::learnTernary(Solver& s, Literal p, Literal q, Literal r, const ClauseInfo& info) {
+	assert(info.type() != Constraint_t::static_constraint);
+	bool added = true;
+	if (!updateShortImplications()) {
+		Literal lits[3] = {p, q, r};
+		s.addLearnt(Clause::newShortClause(s, 0, lits, 3, info), 3);
+	}
+	else if ((added=btig_.addTernary(p, q, r, true)) == true) {
+		s.stats.addLearnt(3, info.type());
+	}
+	if (added && !unique()) { distribute(s, p, q, r, info.type()); }
+	return added;
+}
+
 void SharedContext::addPost(PostPropagator* p) {
 	assert(unique());
 	return master()->addPost(p);
@@ -469,8 +510,10 @@ bool SharedContext::endInit(uint32 shareCount) {
 	if (!master()->endInit()) return false;
 	lastTopLevel_ = master()->units_;
 	shareCount_   = shareCount;
+	btig_.markShared(shareCount > 1);
 	problem_.constraints_binary = btig_.numBinary();
 	problem_.constraints_ternary= btig_.numTernary();
+	frozen_ = true;
 	return true;
 }
 
