@@ -19,11 +19,9 @@
 //
 #include <clasp/solve_algorithms.h>
 #include <clasp/solver.h>
-#include <clasp/minimize_constraint.h>
 #include <clasp/enumerator.h>
 #include <clasp/lookahead.h>
 #include <cmath>
-using std::log;
 namespace Clasp { 
 /////////////////////////////////////////////////////////////////////////////////////////
 // SolveParams
@@ -32,58 +30,48 @@ SolveParams::SolveParams()
 	: randFreq_(0.0)
 	, shuffleFirst_(0), shuffleNext_(0) {
 }
-
-uint32 ReduceParams::init() const {
-	uint32 ret = static_cast<uint32>(base_/frac_);
-	if      (ret < iMin_) ret = iMin_;
-	else if (ret > iMax_) ret = iMax_;
-	return ret;
+/////////////////////////////////////////////////////////////////////////////////////////
+// ReduceParams
+/////////////////////////////////////////////////////////////////////////////////////////
+uint32 ReduceParams::initDbSize() const {
+	double f   = std::max(0.0001f, baseFrac);
+	uint32 ret = static_cast<uint32>(base/f);
+	return std::min(std::max(ret, baseMin), baseMax);
+}
+uint32 ReduceParams::maxDbSize() const {
+	double x   = std::max(base, baseMin) * dbMaxGrow;
+	return (uint32)std::min(x, double(std::numeric_limits<uint32>::max()));
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // Schedule
 /////////////////////////////////////////////////////////////////////////////////////////
-void ScheduleStrategy::init(uint32 base, double grow, uint64 outer, bool arith) {
-	grow_ = grow == 0.0 || grow >= 1.0 ? grow : 1.0;
-	outer_= (outer ? outer : UINT64_MAX);
-	base_ = base;
-	arith_= static_cast<uint32>(arith);
-	idx_  = 0;
+double growR(uint32 idx, double g)       { return pow(g, (double)idx); }
+double addR(uint32 idx, double a)        { return a * idx; }
+uint32 lubyR(uint32 idx)                 {
+	uint32 i = idx + 1;
+	while ((i & (i+1)) != 0) {
+		i    -= ((1u << log2(i)) - 1);
+	}
+	return (i+1)>>1;
 }
 uint64 ScheduleStrategy::current() const {
 	uint64 x;
-	if      (base_ == 0)      x = UINT64_MAX;
-	else if (grow_ == 0)      x = lubyR();
-	else                      x = growR();
-	return x;
+	if      (base == 0) x = UINT64_MAX;
+	else if (grow == 0) x = static_cast<uint64>(lubyR(idx)) * base;
+	else if (type == 0) x = static_cast<uint64>(growR(idx, grow) * base);
+	else                x = static_cast<uint64>(addR(idx, grow)  + base);
+	return x + !x;
 }
 uint64 ScheduleStrategy::next() {
-	++idx_;
+	++idx;
 	uint64 x = current();
-	if (base_ != 0 && grow_ != 0 && x > outer_) {
-		outer_  = x;
-		idx_    = 0;
-		x       = growR();
+	if (outer && x > outer) {
+		outer = x;
+		idx   = 0;
+		x     = current();
 	}
 	return x;
 }
-uint64 ScheduleStrategy::growR() const { 
-	return arith_ == 0
-		? static_cast<uint64>(base_ * pow(grow_, (double)idx_))
-		: static_cast<uint64>(base_ + (grow_*idx_));
-}
-
-uint64 ScheduleStrategy::lubyR() const {
-	uint32 k = idx_+1;
-	while (k) {
-		uint32 nk = static_cast<uint32>(log((double)k) / log(2.0)) + 1;
-		if (k == ((uint32(1) << nk) - 1)) {
-			return base_ * (uint32(1) << (nk-1));
-		}
-		k -= uint32(1) << (nk-1);
-		++k;
-	}
-	return base_;
-}
 /////////////////////////////////////////////////////////////////////////////////////////
 // solve
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -162,40 +150,66 @@ bool SolveAlgorithm::initPath(Solver& s, const LitVec& path, InitParams& params)
 }
 
 ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p) {
-	if (s.hasConflict()) return false;
-	double maxLearnts         = p.reduce.init();
-	const double boundLearnts = p.reduce.bound();
-	ScheduleStrategy ds       = p.reduce.sched;
-	if (maxLearnts < ds.base()) {
-		ds.setBase(std::max(uint32(5000), uint32(maxLearnts/2)));
-		ds.setGrow(ds.base() < ds.grow() ? std::min(ds.base()*1.0, ds.grow()/2.0) : ds.grow());
-	}
-	if (p.reduce.disable) { maxLearnts = (double)UINT32_MAX; }
-	if (maxLearnts < s.numLearntConstraints()) {
-		maxLearnts = static_cast<double>(s.numLearntConstraints()) + p.reduce.initMin();
-		maxLearnts = std::min(maxLearnts, (double)UINT32_MAX);
-	}
 	typedef Enumerator::ProgressType ProgressType;
-	ScheduleStrategy rs = p.restart.sched;
-	ValueRep result     = value_free;
-	uint32 shuffle      = p.shuffleBase();
-	ProgressType t      = Enumerator::progress_restart;
-	SolveLimits gLimit  = p.limits;     // global limit
-	gLimit.restarts     = std::max(gLimit.restarts, uint64(1));
-	uint64 rsLimit      = rs.current(); // current restart limit
-	uint64 dsLimit      = ds.current(); // current deletion limit
-	uint64 minLimit     = 0;            // min of all limits
+	typedef RestartParams::Type      RestartType;
+	if (s.hasConflict()) return false;
 	SearchLimits sLimit;
 	WeightLitVec inDegree;
+	ScheduleStrategy ds       = p.reduce.cflSched;
+	ScheduleStrategy dg       = p.reduce.growSched;
+	ScheduleStrategy rs       = p.restart.sched;
+	RestartType      rt       = (RestartType)p.restart.type;
+	Solver::DBInfo   db       = {0,0,0};
+	SolveLimits gLimit        = p.limits;     // global limit
+	uint64 rsLimit            = rs.current(); // current restart limit
+	uint64 dsLimit            = ds.current(); // current deletion limit
+	uint64 dgLimit            = dg.current(); // current grow limit
+	uint64 minLimit           = 0;            // min of all limits
+	gLimit.restarts           = std::max(gLimit.restarts, uint64(1));
+	ValueRep result           = value_free;
+	uint32 shuffle            = p.shuffleBase();
+	ProgressType t            = Enumerator::progress_restart;
+	double maxLearnts         = p.reduce.initDbSize();
+	const double boundLearnts = p.reduce.maxDbSize();
+	uint64 lastC              = s.stats.conflicts;
+	uint64 lastR              = s.stats.restarts;
+	bool growOnRestart        = p.reduce.growSched.ignore();
+	if (maxLearnts < ds.base) {
+		uint32 oldBase = ds.base;
+		ds.base        = std::min(uint32(maxLearnts), std::max(uint32(5000), uint32(maxLearnts/2)));
+		if (ds.type == ScheduleStrategy::arithmetic_schedule && ds.grow > 0.0) {
+			double R     = ds.grow / oldBase;
+			ds.grow      = ds.base * R;
+		}
+		dsLimit = ds.current();
+	}
+	if (p.reduce.growSched.disabled() && !growOnRestart) {
+		maxLearnts = (double)UINT32_MAX; 
+	}
+	else if (maxLearnts < s.numLearntConstraints()) {
+		maxLearnts = static_cast<double>(s.numLearntConstraints()) + p.reduce.baseMin;
+		maxLearnts = std::min(maxLearnts, (double)UINT32_MAX);
+	}
+	if (rt == RestartParams::dynamic_restarts) {
+		s.stats.enableQueue(rs.base);
+		s.stats.queue->reset();
+		sLimit.xLbd = (float)rs.grow;
+		sLimit.xCfl = (float)(rs.outer / 1e8);
+		rs          = ScheduleStrategy::arith(16000, 10000);
+		rsLimit     = rs.current();
+	}
+	else if (rt == RestartParams::local_restarts) {
+		sLimit.local= rsLimit;
+		rsLimit     = UINT64_MAX;
+	}
 	while (result == value_free && !gLimit.reached()) {
-		if  (!p.restart.local) { minLimit = std::min(rsLimit, std::min(dsLimit, gLimit.conflicts)); }
-		else                   { minLimit = std::min(dsLimit, gLimit.conflicts); sLimit.local = rsLimit; }
-		sLimit.conflicts = minLimit;
-		sLimit.learnts   = (uint32)maxLearnts;
-		reportProgress(t, s, std::min(sLimit.conflicts, sLimit.local), sLimit.learnts);
-		result    = s.search(sLimit, p.randomProbability());
-		minLimit  = (minLimit - sLimit.conflicts); // number of actual conflicts
-		if (gLimit.conflicts != UINT64_MAX) { gLimit.conflicts -= minLimit; }
+		minLimit        = std::min(dgLimit, std::min(gLimit.conflicts, std::min(rsLimit, dsLimit)));
+		sLimit.learnts  = (uint32)(maxLearnts + (p.reduce.strategy.noGlue*db.pinned));
+		sLimit.conflicts= minLimit;
+		if (t != Enumerator::num_progress_types) { reportProgress(t, s, std::min(minLimit, sLimit.local), sLimit.learnts); }
+		if (sLimit.conflicts)                    { result = s.search(sLimit, p.randomProbability()); }
+		minLimit   = (minLimit - sLimit.conflicts); // number of actual conflicts
+		if (gLimit.conflicts != UINT64_MAX)      { gLimit.conflicts -= minLimit; }
 		if (result == value_true && backtrackFromModel(s)) {
 			result   = value_free; // continue enumeration
 			t        = Enumerator::progress_model;
@@ -204,40 +218,76 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p) {
 			}
 			// After the first solution was found, we allow further restarts only if this
 			// is compatible with the enumerator used. 
-			rsLimit  = !p.restart.bounded && s.backtrackLevel() > s.rootLevel()
-				? static_cast<uint64>(-1)
-				: rs.current();
 			dsLimit  = ds.current();
+			rsLimit  = std::max(rsLimit, rs.current());
+			if (!p.restart.bounded && s.backtrackLevel() > s.rootLevel()) {
+				sLimit = SearchLimits();
+				rsLimit= UINT64_MAX;
+			}
 		}
 		else if (result == value_free){  // limit reached
-			rsLimit -= (!p.restart.local) * minLimit;
-			dsLimit -= minLimit;
-			if (dsLimit == 0 || s.numLearntConstraints() >= sLimit.learnts) {
-				s.reduceLearnts(p.reduce.reduceFrac());
-				dsLimit = dsLimit != 0 ? ds.current() : ds.next();
-				t       = Enumerator::progress_reduce;
-				if (s.numLearntConstraints() >= sLimit.learnts) { maxLearnts += s.numLearntConstraints(); }
-			}
-			else if (s.numFreeVars() != 0 && (rsLimit == 0 || sLimit.local == 0)) {
+			rsLimit -= (rt!=RestartParams::local_restarts) * minLimit;
+			dsLimit -= (ds.base != 0) * minLimit;
+			dgLimit -= (dg.base != 0) * minLimit;
+			minLimit = 0;
+			t        = Enumerator::num_progress_types;
+			if (s.numFreeVars() != 0 && (rsLimit == 0 || sLimit.local == 0 || sLimit.dynamicRestart(s.stats))) {
 				// restart reached - do restart
 				++s.stats.restarts;
 				if (p.restart.cir && (s.stats.restarts % p.restart.cir) == 0 ) {
 					inDegree.clear();
 					s.strategies().heuristic->bump(s, inDegree, p.restart.cirBump / (double)s.inDegree(inDegree));
 				}
+				if (rt == RestartParams::dynamic_restarts) {
+					uint64 num = s.stats.restarts  - lastR;
+					uint64 cfl = s.stats.conflicts - lastC;
+					if (cfl   >=  rs.current()) {
+						double avg = cfl / double(num);
+						lastR      = s.stats.restarts;
+						lastC      = s.stats.conflicts;
+						if      (avg >= 16000.0){ sLimit.xLbd += 0.1f;  rs.reset(); }
+						else if (rsLimit == 0)  { sLimit.xLbd += 0.05f; rs.idx -= (rs.idx != 0); }
+						else if (avg >= 4000.0) { sLimit.xLbd += 0.05f; }
+						else if (avg <  1000.0) { sLimit.xLbd -= 0.05f; }
+						else                    { ++rs.idx; }
+					}
+					rsLimit = rs.current();
+					minLimit= s.stats.queue->samples;
+					s.stats.queue->reset();
+				}
+				s.stats.lRestart = s.stats.analyzed;
 				s.undoUntil(0);
 				t = Enumerator::progress_restart;
-				rsLimit = rs.next();
-				if (p.reduce.reduceOnRestart) { s.reduceLearnts(.33f); }
-				if (maxLearnts != (double)UINT32_MAX && maxLearnts < boundLearnts && (s.numLearntConstraints()+rsLimit) > maxLearnts) {
-					maxLearnts    = std::min(maxLearnts*p.reduce.inc(), (double)UINT32_MAX);
-				}
+				if (rsLimit == 0)                { rsLimit      = rs.next(); }
+				if (sLimit.local == 0)           { sLimit.local = rs.next(); }
+				if (p.reduce.reduceOnRestart)    { db           = s.reduceLearnts(.33f, p.reduce.strategy); }
+				if (growOnRestart && !minLimit)  { minLimit     = rs.current(); }
 				if (s.stats.restarts == shuffle) {
 					shuffle += p.shuffleNext();
 					s.shuffleOnNextSimplify();
 				}
 				--gLimit.restarts;
 			}
+			else if (dsLimit == 0 || s.numLearntConstraints() >= sLimit.learnts) {
+				// reduction reached - remove learnt constraints
+				db      = s.reduceLearnts(p.reduce.remFrac, p.reduce.strategy);
+				dsLimit = dsLimit != 0 ? ds.current() : ds.next();
+				t       = Enumerator::progress_reduce;
+				if (db.size >= sLimit.learnts || db.pinned >= maxLearnts) { 
+					ReduceStrategy t; t.algo = 2; t.score = 2; t.glue = 0;
+					db.pinned /= 2;
+					db.size    = s.reduceLearnts(0.5f, t).size;
+					if (db.size >= sLimit.learnts){
+						maxLearnts += std::max(100.0, s.numLearntConstraints()/10.0);
+					}
+				}
+			}
+			if (dgLimit == 0 || (growOnRestart && t == Enumerator::progress_restart)) {
+				// grow sched reached - increase max db size
+				if (!dgLimit)                                          { dgLimit     = dg.next(); minLimit = dgLimit; }
+				if ((s.numLearntConstraints() + minLimit) > maxLearnts){ maxLearnts *= p.reduce.dbGrow; }
+				if (maxLearnts > boundLearnts)                         { maxLearnts  = boundLearnts; dgLimit = UINT64_MAX; growOnRestart = false; }
+			}
 		}
 	}
 	p.limits = gLimit;
