@@ -172,19 +172,18 @@ bool Solver::PPList::isModel(Solver& s) {
 /////////////////////////////////////////////////////////////////////////////////////////
 SolverStrategies::SolverStrategies()
 	: heuristic(new SelectFirst) 
-	, search(use_learning)
-	, heuOpts(UINT32_MAX)
-	, saveProgress(0) 
+	, compress(250)
+	, saveProgress(0)
+	, cflMinAntes(all_antes)
+	, heuOpts(heu_init_dynamic)
 	, reverseArcs(0)
 	, otfs(0)
-	, cflMinAntes(all_antes)
-	, reduceAlgo(reduce_linear)
-	, reduceScore(score_act)
-	, reduceGlue(0)
-	, strengthenRecursive(false)
-	, randomWatches(false)
-	, updateLbd(false)
-	, compress_(250) {
+	, updateLbd(0)
+	, strRecursive(0)
+	, randomWatches(0)
+	, search(use_learning) {
+	struct X { Heuristic x; RNG y; uint32 z[2]; };
+	static_assert(sizeof(SolverStrategies) == sizeof(X));
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // Solver: Construction/Destruction/Setup
@@ -265,9 +264,12 @@ void Solver::startInit(SharedContext* ctx, uint32 numConsGuess) {
 			undoFree(new ConstraintDB(10)); 
 		}
 	}
-	if (strategy_.heuOpts == UINT32_MAX) {
+	if (strategy_.heuOpts == SolverStrategies::heu_init_dynamic) {
 		strategy_.heuOpts = (id() & 1);
 	}
+	if (strategy_.compress == 0) {
+		strategy_.compress = UINT32_MAX;
+	}
 	strategy_.heuristic->startInit(*this);
 	// disable any funky post propagators during setup
 	post_.disableFunky();
@@ -671,7 +673,7 @@ bool Solver::resolveConflict() {
 	if (decisionLevel() > rootLevel_) {
 		if (decisionLevel() != btLevel_ && strategy_.search != SolverStrategies::no_learning) {
 			uint32 uipLevel = analyzeConflict();
-			stats.updateJumps(decisionLevel(), uipLevel, btLevel_);
+			stats.updateJumps(decisionLevel(), uipLevel, btLevel_, ccInfo_.lbd());
 			undoUntil( uipLevel );
 			return ClauseCreator::create(*this, cc_, ccInfo_);
 		}
@@ -833,13 +835,15 @@ uint32 Solver::analyzeConflict() {
 	strategy_.heuristic->updateReason(*this, conflict_, p);
 	for (;;) {
 		uint32 lhsSize = resSize;
+		uint32 rhsSize = 0;
 		Antecedent rhs = last;
 		for (LitVec::size_type i = 0; i != conflict_.size(); ++i) {
 			Literal& q = conflict_[i];
+			uint32 cl  = level(q.var());
+			rhsSize   += (cl != 0);
 			if (!seen(q.var())) {
 				++resSize;
 				assert(isTrue(q) && "Invalid literal in reason set!");
-				uint32 cl = level(q.var());
 				assert(cl > 0 && "Top-Level implication not marked!");
 				markSeen(q.var());
 				if (cl == decisionLevel()) {
@@ -854,9 +858,7 @@ uint32 Solver::analyzeConflict() {
 		if (resSize != lhsSize) {
 			lhs = 0;
 		}
-		if (conflict_.size() != resSize || lastSimplify_ != units_) {
-			// If lastSimplify_ != units_ conflict_ may contain literals assigned
-			// on level 0 which are not counted in resSize
+		if (rhsSize != resSize) {
 			rhs = 0;
 		}
 		if (doOtfs && (!rhs.isNull() || !lhs.isNull())) {
@@ -869,6 +871,7 @@ uint32 Solver::analyzeConflict() {
 			assign_.undoLast();
 		}
 		if (--onLevel == 0) {
+			++rhsSize;
 			conflict_.push_back(~p);
 			break;
 		}
@@ -952,7 +955,7 @@ ClauseHead* Solver::otfsRemove(ClauseHead* c, const LitVec* newC) {
 uint32 Solver::finalizeConflictClause(ClauseHead* rhs) {
 	// 1. remove redundant literals from conflict clause
 	LitVec::size_type stop = assign_.trail.size();
-	if (strategy_.strengthenRecursive && !ccMin_) {
+	if (!ccMin_ && strategy_.strRecursive) {
 		ccMin_ = new CCMinRecursive;
 	}
 	uint32 onAssert = ccMinimize(cc_, assign_.trail, strategy_.cflMinAntes, ccMin_);
@@ -1179,21 +1182,23 @@ bool Solver::decideNextBranch(double f) {
 }
 // Removes up to maxRem% of the learnt nogoods but
 // keeps those that are locked or are highly active.
-void Solver::reduceLearnts(float maxRem) {
+Solver::DBInfo Solver::reduceLearnts(float remFrac, const ReduceStrategy& rs) {
 	uint32 oldS = numLearntConstraints();
-	uint32 j    = 0;
-	if (maxRem < 1.0f) {    
-		uint32 remMax = static_cast<uint32>(numLearntConstraints() * std::min(1.0f, std::max(0.15f, maxRem)));
+	DBInfo r    = {0,0,0};
+	if (remFrac < 1.0f) {    
+		uint32 remMax = static_cast<uint32>(numLearntConstraints() * std::min(1.0f, std::max(0.15f, remFrac)));
 		uint32 remMin = static_cast<uint32>(numLearntConstraints() * 0.15f);
-		switch (strategies().reduceAlgo) {
-			case SolverStrategies::reduce_linear:   j = reduceLinear(remMin, remMax); break;
-			case SolverStrategies::reduce_in_place: j = reduceSortedInPlace(remMin, remMax); break;
-			case SolverStrategies::reduce_stable:   
-			default: j = reduceSorted(remMin, remMax); break;
+		CmpScore cmp(learnts_, (ReduceStrategy::Score)rs.score, rs.glue);
+		switch (rs.algo) {
+			case ReduceStrategy::reduce_linear:   r = reduceLinear(remMin, remMax, cmp); break;
+			case ReduceStrategy::reduce_in_place: r = reduceSortedInPlace(remMin, remMax, cmp); break;
+			case ReduceStrategy::reduce_stable:   
+			default: r = reduceSorted(remMin, remMax, cmp); break;
 		}
 	}
 	else {
 		// remove all nogoods that are not locked
+		LitVec::size_type j = 0;
 		for (LitVec::size_type i = 0; i != learnts_.size(); ++i) {
 			LearntConstraint* c = static_cast<LearntConstraint*>(learnts_[i]);
 			if (c->locked(*this)) {
@@ -1204,71 +1209,80 @@ void Solver::reduceLearnts(float maxRem) {
 				c->destroy(this, true);
 			}
 		}
+		r.size = r.locked = j;
 	}
-	shrinkVecTo(learnts_, j);
+	shrinkVecTo(learnts_, r.size);
 	stats.deleted += oldS - numLearntConstraints();
+	return r;
 }
 
 // Removes up to maxR of the learnt nogoods.
 // Keeps those that are locked or have a high activity.
-uint32 Solver::reduceLinear(uint32, uint32 maxR) {
+Solver::DBInfo Solver::reduceLinear(uint32, uint32 maxR, const CmpScore& sc) {
 	// compute average activity
 	uint64 scoreSum  = 0;
 	for (LitVec::size_type i = 0; i != learnts_.size(); ++i) {
-		scoreSum += strategies().score(static_cast<LearntConstraint*>(learnts_[i])->activity());
+		scoreSum += sc.score(static_cast<LearntConstraint*>(learnts_[i])->activity());
 	}
 	double avgAct    = (scoreSum / (double) numLearntConstraints());
 	// constraints with socre > 1.5 times the average are "active"
 	double scoreThresh = avgAct * 1.5;
-	double scoreMax    = (double)strategies().score(maxActivity());
+	double scoreMax    = (double)sc.score(maxActivity());
 	if (scoreThresh > scoreMax) {
 		scoreThresh = (scoreMax + (scoreSum / (double) numLearntConstraints())) / 2.0;
 	}
 	// remove up to maxR constraints but keep "active" and locked once
-	const uint32 glue = strategies().reduceGlue;
-	uint32 j = 0;
+	const uint32 glue = sc.glue;
+	DBInfo       res  = {0,0,0};
 	for (LitVec::size_type i = 0; i != learnts_.size(); ++i) {
 		LearntConstraint* c          = static_cast<LearntConstraint*>(learnts_[i]);
 		LearntConstraint::Activity a = c->activity();
-		if (maxR == 0 || c->locked(*this) || strategies().score(a) > scoreThresh || a.lbd() <= glue) {
+		bool isLocked                = c->locked(*this);
+		bool isGlue                  = (sc.score(a) > scoreThresh || a.lbd() <= glue);
+		if (maxR == 0 || isLocked || isGlue) {
+			res.pinned                += isGlue;
+			res.locked                += isLocked;
 			c->decreaseActivity();
-			learnts_[j++] = c;
+			learnts_[res.size++]       = c;
 		}
 		else {
 			--maxR;
 			c->destroy(this, true);
 		}
 	}
-	return j;
+	return res;
 }
 
 // Removes up to maxR of the most inactive constraints
 // while reordering learnts_
-uint32 Solver::reduceSortedInPlace(uint32 minR, uint32 maxR) {
+Solver::DBInfo Solver::reduceSortedInPlace(uint32 minR, uint32 maxR, const CmpScore& sc) {
 	assert(minR <= maxR && maxR <= learnts_.size());
-	CmpScore cmp(learnts_, strategies());
 	ConstraintDB::iterator hBeg = learnts_.begin();
 	ConstraintDB::iterator hEnd = learnts_.begin();
-	const uint32 glue           = strategies().reduceGlue;
+	const uint32 glue           = sc.glue;
+	DBInfo res                  = {0,0,0};
 	// move maxR least active constraints to front of learnts
 	for (LitVec::size_type i = 0, end = learnts_.size(); i != end; ++i) {
 		LearntConstraint* c = static_cast<LearntConstraint*>(learnts_[i]);
-		if (!c->locked(*this)) {
-			if (minR || (maxR && c->activity().lbd() > glue)) {
+		bool isGlue         = c->activity().lbd() <= glue;
+		bool isLocked       = c->locked(*this);
+		if (!isLocked && !isGlue) {
+			if (minR || maxR) {
 				learnts_[i] = *hEnd;
 				*hEnd       = c;
 				++hEnd;
 				--maxR;
-				if      (!minR)       { std::push_heap(hBeg, hEnd, cmp); }
-				else if (--minR == 0) { std::make_heap(hBeg, hEnd, cmp); }
+				if      (!minR)       { std::push_heap(hBeg, hEnd, sc); }
+				else if (--minR == 0) { std::make_heap(hBeg, hEnd, sc); }
 			}
-			else if (cmp(c, learnts_[0])) {
-				std::pop_heap(hBeg, hEnd, cmp);
+			else if (sc(c, learnts_[0])) {
+				std::pop_heap(hBeg, hEnd, sc);
 				learnts_[i] = *(hEnd-1);
 				*(hEnd-1)   = c;
-				std::push_heap(hBeg, hEnd, cmp);
+				std::push_heap(hBeg, hEnd, sc);
 			}
 		}
+		else { res.pinned += isGlue; res.locked += isLocked; }
 	}
 	// remove all constraints in heap
 	for (ConstraintDB::iterator it = hBeg; it != hEnd; ++it) {
@@ -1281,37 +1295,43 @@ uint32 Solver::reduceSortedInPlace(uint32 minR, uint32 maxR) {
 		c->decreaseActivity();
 		learnts_[j++] = c;
 	}
-	return j;
+	res.size = j;
+	return res;
 }
 
 // Removes up to maxR of the most inactive constraints
 // without reordering learnts_
-uint32 Solver::reduceSorted(uint32 minR, uint32 maxR) {
+Solver::DBInfo Solver::reduceSorted(uint32 minR, uint32 maxR, const CmpScore& sc) {
 	assert(minR <= maxR && maxR <= learnts_.size());
-	VarVec temp; temp.reserve(maxR);
-	CmpScore cmp(learnts_, strategies());
-	const uint32 glue = strategies().reduceGlue;
+	typedef PodVector<CmpScore::ViewPair>::type TempVec;
+	TempVec temp; temp.reserve(maxR);
+	const uint32 glue = sc.glue;
+	DBInfo       res  = {0,0,0};
 	// add maxR most inactive constraints to heap
+	CmpScore::ViewPair vp;
 	for (LitVec::size_type i = 0; i != learnts_.size(); ++i) {
 		LearntConstraint* c = static_cast<LearntConstraint*>(learnts_[i]);
-		if (!c->locked(*this)) {
-			if (minR || (maxR && c->activity().lbd() > glue)) {
-				temp.push_back(i);
+		vp                  = CmpScore::ViewPair(i, c->activity());
+		bool isGlue         = vp.second.lbd() <= glue;
+		bool isLocked       = c->locked(*this);
+		if (!isLocked && !isGlue) {
+			if (minR || maxR) {
+				temp.push_back(vp);
 				--maxR;
-				if      (!minR)       { std::push_heap(temp.begin(), temp.end(), cmp); }
-				else if (--minR == 0) { std::make_heap(temp.begin(), temp.end(), cmp); }
+				if      (!minR)       { std::push_heap(temp.begin(), temp.end(), sc); }
+				else if (--minR == 0) { std::make_heap(temp.begin(), temp.end(), sc); }
 			}
-			else if (cmp(i, temp[0])) {
-				std::pop_heap(temp.begin(), temp.end(), cmp);
-				temp.back() = i;
-				std::push_heap(temp.begin(), temp.end(), cmp);
+			else if (sc(vp, temp[0])) {
+				std::pop_heap(temp.begin(), temp.end(), sc);
+				temp.back() = vp;
+				std::push_heap(temp.begin(), temp.end(), sc);
 			}
 		}
+		else { res.pinned += isGlue; res.locked += isLocked; }
 	}
-	while (!temp.empty()) {
-		learnts_[temp.back()]->destroy(this, true);
-		learnts_[temp.back()] = 0;
-		temp.pop_back();
+	for (TempVec::const_iterator it = temp.begin(), end = temp.end(); it != end; ++it) {
+		learnts_[it->first]->destroy(this, true);
+		learnts_[it->first] = 0;
 	}
 	uint32 j = 0;
 	for (LitVec::size_type i = 0; i != learnts_.size(); ++i) {
@@ -1320,7 +1340,8 @@ uint32 Solver::reduceSorted(uint32 minR, uint32 maxR) {
 			learnts_[j++] = learnts_[i];
 		}
 	}
-	return j;
+	res.size = j;
+	return res;
 }
 
 uint32 Solver::computeLbd(Literal p, const Literal* first, const Literal* last) {
@@ -1334,7 +1355,7 @@ uint32 Solver::computeLbd(Literal p, const Literal* first, const Literal* last)
 			++lbd;
 		}
 	}
-	return lbd;
+	return lbd + (strategies().updateLbd > 1);
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // The basic DPLL-like search-function
@@ -1356,9 +1377,9 @@ BEGIN_SEARCH:
 				++conflicts;
 			}
 			limit.conflicts -= std::min<uint64>(conflicts, limit.conflicts);
-			if (local && (local->back()+=conflicts) >= limit.local)        { limit.local = 0;    }
-			if (hasConflict() || (decisionLevel() == 0 && !simplify()))    { return value_false; }
-			if (numLearntConstraints() > limit.learnts || limit.reached()) { return value_free;  }
+			if (local && (local->back()+=conflicts) >= limit.local)            { limit.local = 0;    }
+			if (hasConflict() || (decisionLevel() == 0 && !simplify()))        { return value_false; }
+			if (numLearntConstraints() > limit.learnts || limit.reached(stats)){ return value_free;  }
 		}
 		if      (decideNextBranch(rf)) { conflicts = !propagate(); }
 		else if (post_.isModel(*this)) { break;             }
