@@ -583,7 +583,7 @@ void DefaultUnfoundedCheck::createLoopFormula() {
 		solver_->setReason(loopAtoms_[0], ante);
 	}
 	else {
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver_, &(*activeClause_)[1], (uint32)activeClause_->size() - 1, (uint32)activeClause_->sw()-1, (uint32)loopAtoms_.size()); 
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver_, &(*activeClause_)[1], (uint32)activeClause_->size() - 1, (uint32)activeClause_->sw()-1, (uint32)loopAtoms_.size(), activeClause_->info()); 
 		solver_->addLearnt(lf, lf->size());
 		for (VarVec::size_type i = 0; i < loopAtoms_.size(); ++i) {
 			assert(solver_->isTrue(loopAtoms_[i]) && solver_->reason(loopAtoms_[i]) == this);
