@@ -26,39 +26,39 @@
 
 namespace Clasp { namespace mt {
 
-struct SharedWeightConstraintLits {
-	SharedWeightConstraintLits(uint32 size, bool hasWeight) : shared_lits(size, true, hasWeight) {
+struct SharedWeightLits {
+	SharedWeightLits(uint32 size, bool hasWeight) : shared_lits(size, true, hasWeight) {
 		shared_counter = 1; 
 	}
-	std::atomic<int>     shared_counter;
-	WeightConstraintLits shared_lits;
+	std::atomic<int> shared_counter;
+	WeightLits       shared_lits;
 };
 
-SharedWeightConstraintLits* toSharedLits(WeightConstraintLits* x) {
+SharedWeightLits* toSharedLits(WeightLits* x) {
 	assert(x->sharable() && "WeightConstraintLits are not shareable!");
 #define OFFSETOF(S, x)  (size_t)&reinterpret_cast<const volatile unsigned char&>((((S*)0)->*&S::x))
-	return reinterpret_cast<SharedWeightConstraintLits*>(
-		reinterpret_cast<unsigned char*>(x) - OFFSETOF(SharedWeightConstraintLits, shared_lits)
+	return reinterpret_cast<SharedWeightLits*>(
+		reinterpret_cast<unsigned char*>(x) - OFFSETOF(SharedWeightLits, shared_lits)
 	);
 #undef OFFSETOF 
 }
 
 } // Clasp::mt
 
-WeightConstraintLits* WeightConstraintLits::share() {
+WeightLits* WeightLits::clone() {
 	if (sharable()) {
 		++mt::toSharedLits(this)->shared_counter;
 		return this;
 	}
 	else {
-		std::size_t bytes = sizeof(WeightConstraintLits) + (size() << uint32(hasWeights()))*sizeof(Literal);
-		WeightConstraintLits* x = new (::operator new(bytes)) WeightConstraintLits();
+		std::size_t bytes = sizeof(WeightLits) + (size() << uint32(hasWeights()))*sizeof(Literal);
+		WeightLits* x     = new (::operator new(bytes)) WeightLits();
 		std::memcpy(x, this, bytes);
 		return x;
 	}
 }
 
-void WeightConstraintLits::release() {
+void WeightLits::destroy() {
 	if (!sharable()) {
 		::operator delete (this);
 	}
@@ -68,9 +68,10 @@ void WeightConstraintLits::release() {
 	}
 }
 
-bool WeightConstraintLits::unique() const {
-	return !sharable() || mt::toSharedLits(const_cast<WeightConstraintLits*>(this))->shared_counter == 1;
+bool WeightLits::unique() const {
+	return !sharable() || mt::toSharedLits(const_cast<WeightLits*>(this))->shared_counter == 1;
 }
+
 /////////////////////////////////////////////////////////////////////////////////////////
 // WeightConstraint
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -113,13 +114,27 @@ bool WeightConstraint::newWeightConstraint(SharedContext& ctx, Literal con, Weig
 	}
 	bool   hasW = lits.front().second != lits.back().second;
 	uint32 size = (uint32)lits.size()+1;
-	void* m     = ::operator new(sizeof(WeightConstraint) + (size+hasW)*sizeof(UndoInfo));
-	Constraint* c = new (m) WeightConstraint(ctx, con, lits, bound, sumW);
+	uint32 nb   = sizeof(WeightConstraint) + (size+hasW)*sizeof(UndoInfo);
+	void*  m    = 0;
+	WL*    sL   = 0;
+	if (!ctx.shareConstraints()) {
+		nb        = ((nb + 3) / 4)*4;
+		uint32 ls = sizeof(WL) + (size << uint32(hasW))*sizeof(Literal);
+		m         = ::operator new (nb + ls);
+		sL        = new (reinterpret_cast<unsigned char*>(m)+nb) WL(size, false, hasW);
+	}
+	else {
+		m         = ::operator new(nb);
+		void* t   = ::operator new(sizeof(mt::SharedWeightLits) + (size << uint32(hasW))*sizeof(Literal));
+		sL        = &(new (t) mt::SharedWeightLits(size, hasW))->shared_lits;
+	}
+	assert(m && (reinterpret_cast<uintp>(m) & 7u) == 0);
+	Constraint* c = new (m) WeightConstraint(ctx, con, lits, bound, sumW, sL);
 	if (out) *out = c;
 	ctx.add(c);
 	return !s.hasConflict();
 }
-	
+
 // removes assigned and merges duplicate/complementary literals
 // return: achievable weight
 // post  : lits is sorted by decreasing weights
@@ -189,83 +204,76 @@ weight_t WeightConstraint::canonicalize(Solver& s, WeightLitVec& lits, weight_t&
 	return sumW;
 }
 
-WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLitVec& lits, uint32 bound, uint32 sumWeights) {
-	const bool hasWeights = lits.front().second != lits.back().second;
-	if (!ctx.shareConstraints()) {
-		void* m = ::operator new(sizeof(WeightConstraintLits) + ((1+lits.size()) << uint32(hasWeights))*sizeof(Literal));
-		lits_   = new (m) WeightConstraintLits(uint32(1+lits.size()), false, hasWeights);
-	}
-	else {
-		void* m = ::operator new(sizeof(mt::SharedWeightConstraintLits) + ((1+lits.size()) << uint32(hasWeights))*sizeof(Literal));
-		mt::SharedWeightConstraintLits* x = new (m) mt::SharedWeightConstraintLits(uint32(1+lits.size()), hasWeights);
-		lits_ = &x->shared_lits;
-	}
-	Literal* p    = lits_->lits_;
-	Literal* h    = (Literal*)undo_;
-	Solver&  s    = *ctx.master();
-	bool needData       = lits_->hasWeights();
-	bound_[WL::FFB_BTB]	= (sumWeights-bound)+1;// ffb-btb
-	bound_[WL::FTB_BFB]	= bound;               // ftb-bfb
-	*p++                = ~W;                  // store constraint literal
-	if (hasWeights) *p++= Literal::fromRep(1); // and weight if necessary
-	ctx.setFrozen(W.var(), true);              // exempt from variable elimination
-	if (needData) ctx.requestData(W.var());
-	active_ = s.value(W.var())==value_free     // if unassigned
-		? NOT_ACTIVE                             // both constraints are relevant
-		: WL::FFB_BTB+s.isFalse(W);              // otherwise only one is relevant
-	for (uint32 i = 0, end = (uint32)lits.size(); i != end; ++i) {
-		*p++ = h[i+1] = lits[i].first;           // store constraint literal
-		if (hasWeights) {                        // followed by weight 
-			*p++= Literal::fromRep(lits[i].second);// if necessary
-		}
-		addWatch(s, i+1, WL::FTB_BFB);           // watches  lits[idx]
-		addWatch(s, i+1, WL::FFB_BTB);           // watches ~lits[idx]
-		ctx.setFrozen(lits[i].first.var(), true);// exempt from variable elimination
-		if (needData) ctx.requestData(lits[i].first.var());
+WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLitVec& lits, uint32 bound, uint32 sumWeights, WL* out) {
+	const bool hasW = lits.front().second != lits.back().second;
+	lits_           = out;
+	ownsLit_        = !out->sharable();
+	Literal* p      = lits_->lits_;
+	Literal* h      = (Literal*)undo_;
+	Solver&  s      = *ctx.master();
+	weight_t w      = 1;
+	Var      mv     = W.var();
+	bound_[FFB_BTB]	= (sumWeights-bound)+1;// ffb-btb
+	bound_[FTB_BFB]	= bound;               // ftb-bfb
+	*p++            = ~W;                  // store constraint literal
+	if (hasW) *p++  = Literal::fromRep(w); // and weight if necessary
+	ctx.setFrozen(W.var(), true);          // exempt from variable elimination
+	active_ = s.value(W.var())==value_free // if unassigned
+		? NOT_ACTIVE                         // both constraints are relevant
+		: FFB_BTB+s.isFalse(W);              // otherwise only one is relevant
+	for (uint32 i = 0, j = 1, end = (uint32)lits.size(); i != end; ++i, ++j) {
+		*p++ = h[j] = lits[i].first;         // store constraint literal
+		w    = lits[i].second;               // followed by weight 
+		if (hasW) *p++= Literal::fromRep(w); // if necessary
+		addWatch(s, j, FTB_BFB);             // watches  lits[idx]
+		addWatch(s, j, FFB_BTB);             // watches ~lits[idx]
+		ctx.setFrozen(h[j].var(), true);     // exempt from variable elimination
+		mv   = std::max(mv, h[j].var());
 	}
+	if (isWeight()) ctx.requestData(mv+1); // weight constraints can become unit more than once
 	// init heuristic
 	uint32 off = active_ != NOT_ACTIVE;
 	h[0]       = W;
 	s.strategies().heuristic->newConstraint(s, h+off, lits.size()+(1-off), Constraint_t::static_constraint);
 	// init undo stack
-	up_                 = undoStart();         // undo stack is initially empty
+	up_                 = undoStart();     // undo stack is initially empty
 	undo_[0].data       = 0;
 	undo_[up_].data     = 0;
-	setBpIndex(1);                             // where to start back propagation
+	setBpIndex(1);                         // where to start back propagation
 	if (active_ == NOT_ACTIVE) {
-		addWatch(s, 0, WL::FTB_BFB);             // watch con in both phases
-		addWatch(s, 0, WL::FFB_BTB);             // in order to allow for backpropagation
+		addWatch(s, 0, FTB_BFB);             // watch con in both phases
+		addWatch(s, 0, FFB_BTB);             // in order to allow for backpropagation
 	}
 	else {
-		uint32 d = active_;                      // propagate con
-		WeightConstraint::propagate(s, ~lits_->lit(0, (ActiveConstraint)active_), d);
+		uint32 d = active_;                  // propagate con
+		WeightConstraint::propagate(s, ~lit(0, (ActiveConstraint)active_), d);
 	}
 }
 
 WeightConstraint::WeightConstraint(Solver& s, const WeightConstraint& other) {
-	lits_           = other.lits_->share();
-	WL::Iterator it = lits_->begin();
+	lits_           = other.lits_->clone();
+	ownsLit_        = 0;
 	Literal* heu    = (Literal*)undo_;
-	heu[0]          = ~*it;
+	heu[0]          = ~lits_->lit(0);
 	bound_[0]	      = other.bound_[0];
 	bound_[1]	      = other.bound_[1];
 	active_         = other.active_;
 	if (active_ == NOT_ACTIVE && s.value(heu[0].var()) == value_free) {
-		addWatch(s, 0, WL::FTB_BFB);  // watch con in both phases
-		addWatch(s, 0, WL::FFB_BTB);  // in order to allow for backpropagation
+		addWatch(s, 0, FTB_BFB);  // watch con in both phases
+		addWatch(s, 0, FFB_BTB);  // in order to allow for backpropagation
 	}
-	for (uint32 i = 1; i < lits_->size(); ++i) {
-		heu[i]      = *++it;
+	for (uint32 i = 1, end = size(); i < end; ++i) {
+		heu[i]      = lits_->lit(i);
 		if (s.value(heu[i].var()) == value_free) {
-			addWatch(s, i, WL::FTB_BFB);  // watches  lits[i]
-			addWatch(s, i, WL::FFB_BTB);  // watches ~lits[i]
+			addWatch(s, i, FTB_BFB);  // watches  lits[i]
+			addWatch(s, i, FFB_BTB);  // watches ~lits[i]
 		}
 	}
 	// Initialize heuristic with literals (no weights) in constraint.
 	uint32 off = active_ != NOT_ACTIVE;
-	s.strategies().heuristic->newConstraint(s, heu+off, lits_->size()-off, Constraint_t::static_constraint);
+	s.strategies().heuristic->newConstraint(s, heu+off, size()-off, Constraint_t::static_constraint);
 	// Init undo stack
-	std::memcpy(undo_, other.undo_, sizeof(UndoInfo)*(lits_->size()+lits_->hasWeights()));
+	std::memcpy(undo_, other.undo_, sizeof(UndoInfo)*(size()+isWeight()));
 	up_ = other.up_;
 }
 
@@ -273,24 +281,23 @@ WeightConstraint::~WeightConstraint() {
 }
 
 Constraint* WeightConstraint::cloneAttach(Solver& other) { 
-	void* m = ::operator new(sizeof(WeightConstraint) + (lits_->size()+lits_->hasWeights())*sizeof(UndoInfo));
+	void* m = ::operator new(sizeof(WeightConstraint) + (size()+isWeight())*sizeof(UndoInfo));
 	return new (m) WeightConstraint(other, *this);
-
 }
 
 void WeightConstraint::addWatch(Solver& s, uint32 idx, ActiveConstraint c) {
 	// Add watch only if c is relevant.
 	if (uint32(c^1) != active_) {
 		// Use LSB to store the constraint that watches the literal.
-		s.addWatch(~lits_->lit(idx, c), this, (idx<<1)+c);
+		s.addWatch(~lit(idx, c), this, (idx<<1)+c);
 	}
 }
 
 void WeightConstraint::destroy(Solver*, bool) {
-	lits_->release();
-	void* mem = static_cast<Constraint*>(this);
+	if (ownsLit_ == 0) { lits_->destroy(); }
+	void* mem    = static_cast<Constraint*>(this);
 	this->~WeightConstraint();
-	::operator delete(mem);
+	::operator delete(mem);	
 }
 
 // Returns the numerical highest decision level watched by this constraint.
@@ -304,7 +311,7 @@ uint32 WeightConstraint::highestUndoLevel(Solver& s) const {
 // undo stack. If the current decision level is not watched, an undo watch is added
 // so that the bound can be adjusted once the solver backtracks.
 void WeightConstraint::updateConstraint(Solver& s, uint32 idx, ActiveConstraint c) {
-	bound_[c] -= lits_->weight(idx);
+	bound_[c] -= weight(idx);
 	if (highestUndoLevel(s) != s.decisionLevel()) {
 		s.addUndoWatch(s.decisionLevel(), this);
 	}
@@ -337,7 +344,7 @@ void WeightConstraint::updateConstraint(Solver& s, uint32 idx, ActiveConstraint
 Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d) {
 	// determine the affected constraint and its body literal
 	ActiveConstraint c  = (ActiveConstraint)(d&1);
-	Literal body        = lits_->lit(0, c);
+	Literal body        = lit(0, c);
 	if ( uint32(c^1) == active_ || s.isTrue(body) ) {
 		// the other constraint is active or this constraint is already satisfied; 
 		// nothing to do
@@ -346,8 +353,8 @@ Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d
 	// the constraint is not yet satisfied; update it and
 	// check if we can now propagate any literals.
 	updateConstraint(s, d >> 1, c);
-	if (bound_[c] <= 0 || (lits_->hasWeights() && litSeen(0))) {
-		uint32 reasonData = !lits_->hasWeights() ? UINT32_MAX : up_;
+	if (bound_[c] <= 0 || (isWeight() && litSeen(0))) {
+		uint32 reasonData = !isWeight() ? UINT32_MAX : up_;
 		if (!litSeen(0)) {
 			// forward propagate constraint to true
 			active_ = c;
@@ -356,16 +363,16 @@ Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d
 		else {
 			// backward propagate false constraint
 			uint32 n = getBpIndex();
-			for (const uint32 end = lits_->size(); n != end && (bound_[c] - lits_->weight(n)) < 0; ++n) {
+			for (const uint32 end = size(); n != end && (bound_[c] - weight(n)) < 0; ++n) {
 				if (!litSeen(n)) {
 					active_   = c;
-					Literal x = lits_->lit(n, c);
+					Literal x = lit(n, c);
 					if (!s.force(x, this, reasonData)) {
 						return PropResult(false, true);
 					}
 				}
 			}
-			assert(n == 1 || n == lits_->size() || lits_->hasWeights());
+			assert(n == 1 || n == size() || isWeight());
 			setBpIndex(n);
 		}
 	}
@@ -378,13 +385,13 @@ Constraint::PropResult WeightConstraint::propagate(Solver& s, Literal, uint32& d
 void WeightConstraint::reason(Solver& s, Literal p, LitVec& r) {
 	assert(active_ != NOT_ACTIVE);
 	Literal x;
-	uint32 stop = !lits_->hasWeights() ? up_ : s.reasonData(p);
+	uint32 stop = !isWeight() ? up_ : s.reasonData(p);
 	assert(stop <= up_);
 	for (uint32 i = undoStart(); i != stop; ++i) {
 		UndoInfo u = undo_[i];
 		// Consider only lits that are relevant to the active constraint
 		if (u.constraint() == (ActiveConstraint)active_) {
-			x = lits_->lit(u.idx(), u.constraint());
+			x = lit(u.idx(), u.constraint());
 			r.push_back( ~x );
 		}
 	}
@@ -393,13 +400,13 @@ void WeightConstraint::reason(Solver& s, Literal p, LitVec& r) {
 bool WeightConstraint::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 	assert(active_ != NOT_ACTIVE);
 	Literal x;
-	uint32 stop = !lits_->hasWeights() ? up_ : s.reasonData(p);
+	uint32 stop = !isWeight() ? up_ : s.reasonData(p);
 	assert(stop <= up_);
 	for (uint32 i = undoStart(); i != stop; ++i) {
 		UndoInfo u = undo_[i];
 		// Consider only lits that are relevant to the active constraint
 		if (u.constraint() == (ActiveConstraint)active_) {
-			x = lits_->lit(u.idx(), u.constraint());
+			x = lit(u.idx(), u.constraint());
 			if (!s.ccMinimize(~x, rec)) {
 				return false;
 			}
@@ -414,31 +421,30 @@ void WeightConstraint::undoLevel(Solver& s) {
 	for (UndoInfo u; up_ != undoStart() && s.value(lits_->var((u=undoTop()).idx())) == value_free;) {
 		assert(litSeen(u.idx()));
 		toggleLitSeen(u.idx());
-		bound_[u.constraint()] += lits_->weight(u.idx());
+		bound_[u.constraint()] += weight(u.idx());
 		--up_;
 	}
 	if (!litSeen(0)) active_ = NOT_ACTIVE;
 }
 
 bool WeightConstraint::simplify(Solver& s, bool) {
-	typedef WL::LiteralIterator Iter;
 	if (bound_[0] <= 0 || bound_[1] <= 0) {
-		for (Iter it = lits_->begin(), end = lits_->end(); it != end; ++it) {
-			s.removeWatch( (*it), this );
-			s.removeWatch(~(*it), this );
+		for (uint32 i = 0, end = size(); i != end; ++i) {
+			s.removeWatch( lits_->lit(i), this );
+			s.removeWatch(~lits_->lit(i), this );
 		}
 		return true;
 	}
-	else if (s.value(lits_->var(0)) != value_free && (active_ == NOT_ACTIVE || lits_->hasWeights())) {
+	else if (s.value(lits_->var(0)) != value_free && (active_ == NOT_ACTIVE || isWeight())) {
 		if (active_ == NOT_ACTIVE) {
-			Literal W = ~*lits_->begin();
-			active_   = WL::FFB_BTB+s.isFalse(W);	
+			Literal W = ~lits_->lit(0);
+			active_   = FFB_BTB+s.isFalse(W);	
 		}
-		for (Iter it = lits_->begin(), end = lits_->end(); it != end; ++it) {
-			s.removeWatch(active_ == WL::FTB_BFB ? ~(*it) : *it, this);
+		for (uint32 i = 0, end = size(); i != end; ++i) {
+			s.removeWatch(lit(i, (ActiveConstraint)active_), this);
 		}
 	}
-	if (lits_->unique() && lits_->size() > 4 && (up_ - undoStart()) > lits_->size()/2) {
+	if (lits_->unique() && size() > 4 && (up_ - undoStart()) > size()/2) {
 		Literal*     lits = lits_->lits_;
 		const uint32 inc  = 1 + lits_->hasWeights();
 		uint32       end  = lits_->size_*inc;
@@ -482,11 +488,11 @@ uint32 WeightConstraint::estimateComplexity(const Solver& s) const {
 	weight_t w1 = bound_[0];
 	weight_t w2 = bound_[1];
 	uint32 r    = 2;
-	for (uint32 i = 1; i != lits_->size(); ++i) {
+	for (uint32 i = 1; i != size(); ++i) {
 		if (s.value(lits_->var(i)) == value_free) {
 			++r;
-			if ( (w1 -= lits_->weight(i)) <= 0 ) break;
-			if ( (w2 -= lits_->weight(i)) <= 0 ) break;
+			if ( (w1 -= weight(i)) <= 0 ) break;
+			if ( (w2 -= weight(i)) <= 0 ) break;
 		}
 	}
 	return r;
