@@ -49,7 +49,8 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCreateLearntClauseConflict);
 	CPPUNIT_TEST(testCreateNonAssertingLearntClauseAsserting);
 	CPPUNIT_TEST(testCreateBogusUnit);
-
+	
+	CPPUNIT_TEST(testInitWatches);
 	CPPUNIT_TEST(testIntegrateEmpty);
 	CPPUNIT_TEST(testIntegrateUnit);
 	CPPUNIT_TEST(testIntegrateUnitSAT);
@@ -60,6 +61,10 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testIntegrateSATBug1);
 	CPPUNIT_TEST(testIntegrateSATBug2);
 	CPPUNIT_TEST(testIntegrateSATBug3);
+	CPPUNIT_TEST(testIntegrateSATBug4);
+	CPPUNIT_TEST(testIntegrateKnownOrderBug);
+	CPPUNIT_TEST(testIntegrateNotConflictingBug);
+	CPPUNIT_TEST(testIntegrateSimplify);
 	CPPUNIT_TEST(testIntegrateSAT);
 	CPPUNIT_TEST(testIntegrateUnsat);
 	CPPUNIT_TEST(testIntegrateAssertingBelowBT);
@@ -288,6 +293,31 @@ public:
 		creator.start(Constraint_t::learnt_other).add(a).add(~c).end();
 	}
 
+	void testInitWatches() {
+		LitVec cl;
+		cl.push_back(a);
+		cl.push_back(b);
+		cl.push_back(c);
+		cl.push_back(d);
+		solver().assume(~b)   && solver().propagate();
+		solver().force(~c, 0) && solver().propagate();
+		solver().assume(~a)   && solver().propagate();
+		solver().assume(d)    && solver().propagate();
+		// aF@2 bF@1 cF@1 dT@3 -> dT@3 aF@2 cF@1 bF@1
+		LitVec temp = cl;
+		ClauseCreator::initWatches(solver(), temp, false);
+		CPPUNIT_ASSERT(temp[0] == d);
+		CPPUNIT_ASSERT(temp[1] == a);
+		
+		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
+		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_add);
+		temp.clear();
+		r.local->clause()->toLits(temp);
+		CPPUNIT_ASSERT(temp[0] == d);
+		CPPUNIT_ASSERT(temp[1] == a);
+		r.local->destroy(&solver(), true);
+	}
+
 	void testIntegrateEmpty() {
 		LitVec cl;
 		solver().assume(~a) && solver().propagate();
@@ -433,6 +463,50 @@ public:
 		CPPUNIT_ASSERT(solver().isTrue(c));
 	}
 
+	void testIntegrateSATBug4() {
+		LitVec cl;
+		cl.push_back(a); cl.push_back(b);
+		solver().force(~a, 0);
+		solver().assume(b);
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unit) != 0);
+		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_explicit);
+	}
+
+	void testIntegrateKnownOrderBug() {
+		LitVec cl;
+		cl.push_back(a);
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unit) != 0);
+		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_known_order);
+		CPPUNIT_ASSERT(solver().isTrue(a));
+	}
+
+	void testIntegrateNotConflictingBug() {
+		LitVec cl;
+		cl.push_back(a); cl.push_back(b);
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		solver().assume(~a) && solver().propagate();
+		solver().force(~b,0)&& solver().propagate();
+		CPPUNIT_ASSERT(ClauseCreator::status(solver(), p->begin(), p->end()) == ClauseCreator::status_conflicting);
+		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_conflict);
+		CPPUNIT_ASSERT(r.ok == false);
+		CPPUNIT_ASSERT(r.local == 0);
+	}
+	void testIntegrateSimplify() {
+		LitVec cl;
+		cl.push_back(a); cl.push_back(b); cl.push_back(c);
+		cl.push_back(d); cl.push_back(e); cl.push_back(f);
+		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
+		solver().force(~d, 0) && solver().propagate();
+		solver().assume(~a)   && solver().propagate();
+		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_add);
+		CPPUNIT_ASSERT(r.ok);
+		CPPUNIT_ASSERT(r.local != 0);
+		cl.clear();
+		r.local->toLits(cl);
+		CPPUNIT_ASSERT(cl.size() == 5 && std::find(cl.begin(), cl.end(), d) == cl.end());
+	}
 	void testIntegrateSAT() {
 		LitVec cl;
 		cl.push_back(a); cl.push_back(b); cl.push_back(c); cl.push_back(d);
