@@ -382,7 +382,7 @@ public:
 	}
 
 	void testStrengthenContracted() {
-		solver->strategies().setCompressionStrategy(4);
+		solver->strategies().compress = 4;
 		ctx.endInit(1);
 		ClauseCreator creator(solver);
 		creator.startAsserting(Constraint_t::learnt_conflict, a1);
@@ -416,7 +416,7 @@ public:
 	}
 
 	void testStrengthenBug() {
-		solver->strategies().setCompressionStrategy(6);
+		solver->strategies().compress = 6;
 		ctx.endInit(1);
 		LitVec clause;
 		clause.push_back(a1);
@@ -438,7 +438,7 @@ public:
 	}
 
 	void testStrengthenContractedNoExtend() {
-		solver->strategies().setCompressionStrategy(4);
+		solver->strategies().compress = 4;
 		ctx.endInit(1);
 		LitVec clause;
 		clause.push_back(a1);
@@ -559,7 +559,7 @@ public:
 	}
 	
 	void testContraction() {
-		solver->strategies().setCompressionStrategy(6);
+		solver->strategies().compress = 6;
 		ctx.endInit(1);
 		ClauseCreator creator(solver);
 		creator.startAsserting(Constraint_t::learnt_conflict, a1);
