@@ -55,13 +55,11 @@ public:
 		SharedContext ctx1;
 		SharedContext ctx2(ctx1, SharedContext::init_share_symbols);
 		SharedContext ctx3(ctx1, SharedContext::init_share_symbols);
+		
+		ctx1.addPost(new Lookahead(Lookahead::body_lookahead, false));
+		ctx2.addPost(new Lookahead(Lookahead::atom_lookahead, false));
+		ctx3.addPost(new Lookahead(Lookahead::hybrid_lookahead, false));
 
-		DecisionHeuristic* lookBody   = new UnitHeuristic(Lookahead::body_lookahead);
-		DecisionHeuristic* lookAtom   = new UnitHeuristic(Lookahead::atom_lookahead);
-		DecisionHeuristic* lookHybrid = new UnitHeuristic(Lookahead::hybrid_lookahead);
-		ctx1.master()->strategies().heuristic.reset(lookBody);
-		ctx2.master()->strategies().heuristic.reset(lookAtom);
-		ctx3.master()->strategies().heuristic.reset(lookHybrid);
 		ProgramBuilder api;
 		api.startProgram(ctx1, ProgramBuilder::EqOptions().noEq().noScc())
 			.setAtomName(1, "x").setAtomName(2, "a1").setAtomName(3, "a2").setAtomName(4, "a3")
@@ -87,12 +85,10 @@ public:
 		SharedContext ctx2(ctx1, SharedContext::init_share_symbols);
 		SharedContext ctx3(ctx1, SharedContext::init_share_symbols);
 		
-		DecisionHeuristic* lookBody   = new UnitHeuristic(Lookahead::body_lookahead);
-		DecisionHeuristic* lookAtom   = new UnitHeuristic(Lookahead::atom_lookahead);
-		DecisionHeuristic* lookHybrid = new UnitHeuristic(Lookahead::hybrid_lookahead);
-		ctx1.master()->strategies().heuristic.reset(lookBody);
-		ctx2.master()->strategies().heuristic.reset(lookAtom);
-		ctx3.master()->strategies().heuristic.reset(lookHybrid);
+		ctx1.addPost(new Lookahead(Lookahead::body_lookahead, false));
+		ctx2.addPost(new Lookahead(Lookahead::atom_lookahead, false));
+		ctx3.addPost(new Lookahead(Lookahead::hybrid_lookahead, false));
+		
 		ProgramBuilder api;
 		api.startProgram(ctx1, ProgramBuilder::EqOptions().noEq().noScc())
 			.setAtomName(1, "x").setAtomName(2, "c1").setAtomName(3, "c2").setAtomName(4, "c3")
