@@ -56,6 +56,8 @@ class MinimizeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testMultiAssignment);
 	CPPUNIT_TEST(testBugBacktrackFromFalse);
 	CPPUNIT_TEST(testBugBacktrackToTrue);
+	CPPUNIT_TEST(testBugInitOptHierarch);
+	CPPUNIT_TEST(testBugAdjustSum);
 	CPPUNIT_TEST(testWeightNullBug);
 	CPPUNIT_TEST(testAdjust);
 	CPPUNIT_TEST(testAdjustFact);
@@ -102,7 +104,7 @@ public:
 		newMin = MinimizeBuilder().addRule(min).buildAndAttach(ctx);
 		CPPUNIT_ASSERT(newMin->numRules() == 1);
 		CPPUNIT_ASSERT(countMinLits() == 3);
-		CPPUNIT_ASSERT(newMin->shared()->sum()[0] == 1);
+		CPPUNIT_ASSERT(newMin->shared()->sum(0) == 1);
 		const SharedMinimizeData* data = newMin->shared();
 		CPPUNIT_ASSERT(data->lits[0].first == a && data->lits[0].second == 3);
 		for (const WeightLiteral* it = data->lits; !isSentinel(it->first); ++it) {
@@ -147,7 +149,7 @@ public:
 		CPPUNIT_ASSERT(newMin->numRules() == 3);
 		CPPUNIT_ASSERT(countMinLits() == 6);
 		const SharedMinimizeData* data = newMin->shared();
-		CPPUNIT_ASSERT(data->sum()[0] == 1 && data->sum()[1] == 0 && data->sum()[2] == 2);
+		CPPUNIT_ASSERT(data->sum(0) == 1 && data->sum(1) == 0 && data->sum(2) == 2);
 		CPPUNIT_ASSERT(data->lits[0].first == b);
 		CPPUNIT_ASSERT(data->weights.size() == 11);
 		for (const WeightLiteral* it = data->lits; !isSentinel(it->first); ++it) {
@@ -194,7 +196,7 @@ public:
 		builder.addRule(min);
 		newMin = builder.buildAndAttach(ctx);
 		CPPUNIT_ASSERT(countMinLits() == 5);
-		CPPUNIT_ASSERT(newMin->shared()->sum()[0] == 1 && newMin->shared()->sum()[1] == 1);
+		CPPUNIT_ASSERT(newMin->shared()->sum(0) == 1 && newMin->shared()->sum(1) == 1);
 
 		ctx.master()->assume(c) && ctx.master()->propagate();
 		CPPUNIT_ASSERT(newMin->sum(1) == 0);
@@ -212,7 +214,7 @@ public:
 		builder.addRule(min);
 		newMin = builder.buildAndAttach(ctx);
 		CPPUNIT_ASSERT(countMinLits() == 1);
-		CPPUNIT_ASSERT(newMin->shared()->sum()[0] == 1 && newMin->shared()->sum()[1] == 0);
+		CPPUNIT_ASSERT(newMin->shared()->sum(0) == 1 && newMin->shared()->sum(1) == 0);
 	}
 	
 	void testOrder() {
@@ -729,6 +731,51 @@ public:
 		CPPUNIT_ASSERT(solver.value(b.var()) == value_free);
 	}
 
+	void testBugInitOptHierarch() {
+		WeightLitVec aMin, bMin;
+		aMin.push_back( WeightLiteral(a, 1) );
+		aMin.push_back( WeightLiteral(b, 1) );
+		bMin.push_back( WeightLiteral(c, 1) );
+		bMin.push_back( WeightLiteral(d, 1) );
+		data = MinimizeBuilder()
+			.addRule(aMin)
+			.addRule(bMin)
+			.setOptimum(0, 1)
+			.setOptimum(1, 1)
+			.build(ctx);
+		data->setMode(MinimizeMode_t::optimize, 1);
+		newMin = data->attach(*ctx.master());
+		CPPUNIT_ASSERT(ctx.master()->value(a.var()) == value_free);
+		ctx.master()->assume(a) && ctx.master()->propagate();
+		CPPUNIT_ASSERT(ctx.master()->isFalse(b));
+		CPPUNIT_ASSERT(ctx.master()->value(c.var()) == value_free);
+		ctx.master()->assume(c) && ctx.master()->propagate();
+		CPPUNIT_ASSERT(ctx.master()->isFalse(d));
+	}
+
+	void testBugAdjustSum() {
+		WeightLitVec aMin, bMin;
+		aMin.push_back( WeightLiteral(a, 1) );
+		aMin.push_back( WeightLiteral(b, 1) );
+		bMin.push_back( WeightLiteral(~a, 1) );
+		bMin.push_back( WeightLiteral(d, 1) );
+		data = MinimizeBuilder()
+			.addRule(aMin)
+			.addRule(bMin)
+			.build(ctx);
+		data->setMode(MinimizeMode_t::optimize, 0);
+		newMin = data->attach(*ctx.master());
+		SharedMinimizeData::SumVec opt;
+		opt.push_back(1);
+		opt.push_back(1);
+		setOptimum(*ctx.master(), opt, true);
+		// a ~b ~d is a valid model!
+		CPPUNIT_ASSERT(ctx.master()->value(a.var()) == value_free);
+		CPPUNIT_ASSERT(ctx.master()->assume(a) && ctx.master()->propagate());
+		CPPUNIT_ASSERT(ctx.master()->assume(~b) && ctx.master()->propagate());
+		CPPUNIT_ASSERT(ctx.master()->assume(~d) && ctx.master()->propagate());
+	}
+
 	void testWeightNullBug() {
 		WeightLitVec min1;
 		min1.push_back( WeightLiteral(a, 1) );
@@ -896,7 +943,7 @@ private:
 		return newMin->integrateNext(s);
 	}
 	MinimizeConstraint* createMin(Solver& s, SharedMinimizeData* data) {
-		return MinimizeBuilder::attach(s, data);
+		return data->attach(s);
 	}	
 	SharedContext ctx;
 	MinimizeConstraint* newMin;
