@@ -140,6 +140,15 @@ class ProgramBuilderTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testIncrementalBackpropCompute);
 	CPPUNIT_TEST(testIncrementalBackpropSolver);
 
+	CPPUNIT_TEST(testIncrementalFreezeUnfreeze);
+	CPPUNIT_TEST(testIncrementalSymbolUpdate);
+
+	CPPUNIT_TEST(testIncrementalFreezeDefined);
+	CPPUNIT_TEST(testIncrementalUnfreezeDefined);
+	CPPUNIT_TEST(testIncrementalImplicitUnfreeze);
+	CPPUNIT_TEST(testIncrementalRedefine);
+	CPPUNIT_TEST(testIncrementalGetAssumptions);
+
 	CPPUNIT_TEST(testMergeValue);
 	CPPUNIT_TEST_SUITE_END();	
 public:
@@ -1711,6 +1720,129 @@ public:
 		CPPUNIT_ASSERT(a->value() == value_true);
 		CPPUNIT_ASSERT(x->value() == value_weak_true);
 	}
+
+	void testIncrementalFreezeUnfreeze() {
+		builder.startProgram(ctx);
+		// I0:
+		// {a}.
+		// freeze{x}.
+		// unfreeze{x}.
+		builder.updateProgram()
+			.setAtomName(1, "a").setAtomName(2, "x")
+			.startRule(CHOICERULE).addHead(1).endRule()
+			.freeze(2)
+			.unfreeze(2)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT_EQUAL(negLit(0), ctx.symTab()[2].lit);
+
+		// I1:
+		// freeze(y).
+		// y :- x.
+		builder.updateProgram()
+			.setAtomName(3, "y")
+			.freeze(3)
+			.startRule().addHead(3).addToBody(2, true).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT_EQUAL(true, ctx.master()->isFalse(ctx.symTab()[2].lit));
+	}
+	void testIncrementalSymbolUpdate() {
+		builder.startProgram(ctx);
+		// I0:
+		// {a}.
+		builder.updateProgram()
+			.setAtomName(1, "a")
+			.startRule(CHOICERULE).addHead(1).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		// I1:
+		// {_unnamed, b}.
+		builder.updateProgram()
+			.setAtomName(3, "b")
+			.startRule(CHOICERULE).addHead(2).addHead(3).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(ctx.symTab().find(2) == 0);
+		CPPUNIT_ASSERT(!isSentinel(ctx.symTab()[3].lit));
+	}
+
+	void testIncrementalFreezeDefined() {
+		builder.startProgram(ctx);
+		// I0:
+		// {a}.
+		builder.updateProgram()
+			.setAtomName(1, "a")
+			.startRule(CHOICERULE).addHead(1).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		// I1:
+		builder.updateProgram()
+			.freeze(1)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(builder.getAtom(1)->frozen() == false);
+	}
+	void testIncrementalUnfreezeDefined() {
+		builder.startProgram(ctx);
+		// I0:
+		// {a}.
+		builder.updateProgram()
+			.setAtomName(1, "a")
+			.startRule(CHOICERULE).addHead(1).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		// I1:
+		builder.updateProgram()
+			.unfreeze(1)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(!ctx.master()->isFalse(builder.getLiteral(1)));
+	}
+	void testIncrementalImplicitUnfreeze() {
+		builder.startProgram(ctx);
+		// I0:
+		// freeze(a).
+		builder.updateProgram()
+			.setAtomName(1, "a")
+			.freeze(1)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(builder.getAtom(1)->frozen() == true);
+		CPPUNIT_ASSERT(!ctx.master()->isFalse(builder.getLiteral(1)));
+		// I1:
+		// {a}.
+		builder.updateProgram()
+			.startRule(CHOICERULE).addHead(1).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		CPPUNIT_ASSERT(builder.getAtom(1)->frozen() == false);
+	}
+	void testIncrementalRedefine() {
+		builder.startProgram(ctx);
+		// I0:
+		// {a}.
+		builder.updateProgram()
+			.setAtomName(1, "a")
+			.startRule(CHOICERULE).addHead(1).endRule()
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		// I1:
+		// {a}.
+		CPPUNIT_ASSERT_THROW(builder.updateProgram().startRule(CHOICERULE).addHead(1).endRule(), RedefinitionError);
+	}
+	void testIncrementalGetAssumptions() {
+		builder.startProgram(ctx);
+		// I0:
+		builder.updateProgram()
+			.setAtomName(1, "a").setAtomName(2, "b")
+			.freeze(1).freeze(2)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		LitVec a;
+		builder.getAssumptions(a);
+		CPPUNIT_ASSERT(a.size() == 2 && a[0] == ~builder.getLiteral(1) && a[1] == ~builder.getLiteral(2));
+	}
 	void testMergeValue() {
 		PrgNode lhs, rhs;
 		ValueRep ok[15] = {
