@@ -189,10 +189,10 @@ public:
 		Solver& s = *ctx.master();
 		const SolverStrategies& x = s.strategies();
 		CPPUNIT_ASSERT(x.heuristic.get() != 0);
-		CPPUNIT_ASSERT(x.strengthenRecursive == false);
+		CPPUNIT_ASSERT(x.strRecursive == false);
 		CPPUNIT_ASSERT(x.cflMinAntes == SolverStrategies::all_antes);
 		CPPUNIT_ASSERT(x.search == SolverStrategies::use_learning);
-		CPPUNIT_ASSERT(x.compress() == uint32(250));
+		CPPUNIT_ASSERT(x.compress == uint32(250));
 		CPPUNIT_ASSERT(x.randomWatches == false);
 
 		CPPUNIT_ASSERT_EQUAL(0u, s.numVars());
@@ -291,15 +291,12 @@ public:
 	}
 
 	void testCmpScores() {
-		Solver& s = *ctx.master();
+		ReduceStrategy rs;
 		LearntConstraint::Activity a1 = makeActivity(100, 5);
 		LearntConstraint::Activity a2 = makeActivity(90, 3);
-		s.strategies().reduceScore    = SolverStrategies::score_act;
-		CPPUNIT_ASSERT(s.strategies().compareScore(a1, a2) > 0);
-		s.strategies().reduceScore    = SolverStrategies::score_lbd;
-		CPPUNIT_ASSERT(s.strategies().compareScore(a1, a2) < 0);
-		s.strategies().reduceScore    = SolverStrategies::score_both;
-		CPPUNIT_ASSERT(s.strategies().compareScore(a1, a2) > 0);
+		CPPUNIT_ASSERT(rs.compare(ReduceStrategy::score_act, a1, a2) > 0);
+		CPPUNIT_ASSERT(rs.compare(ReduceStrategy::score_lbd, a1, a2) < 0);
+		CPPUNIT_ASSERT(rs.compare(ReduceStrategy::score_both, a1, a2) > 0);
 	}
 
 	void testPreferredLitByType() {
@@ -1068,8 +1065,8 @@ public:
 		CPPUNIT_ASSERT_EQUAL(uint64(9),st.lits[1]);
 		CPPUNIT_ASSERT_EQUAL(uint64(10),st.learnts[0]);
 		CPPUNIT_ASSERT_EQUAL(uint64(9),st.learnts[1]);
-		CPPUNIT_ASSERT_EQUAL(uint64(6),st.binary);
-		CPPUNIT_ASSERT_EQUAL(uint64(5),st.ternary);
+		CPPUNIT_ASSERT_EQUAL(uint32(6),st.binary);
+		CPPUNIT_ASSERT_EQUAL(uint32(5),st.ternary);
 		CPPUNIT_ASSERT_EQUAL(uint64(10),st.deleted);
 	}
 
