@@ -475,7 +475,7 @@ void ParsedOptions::assign(const Option& o, const std::string& value) {
 	if (badState || !o.value()->parse(o.name(), value, Value::value_parsed)) {
 		badState 
 			? throw ValueError("", ValueError::multiple_occurences, o.name(), value)
-			: throw ValueError("", ValueError::invalid_default, o.name(), value);
+			: throw ValueError("", ValueError::invalid_value, o.name(), value);
 	}
 }
 ///////////////////////////////////////////////////////////////////////////////
